@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -42,11 +42,66 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type);
 
 typedef struct hostAddrList {
   HostAddr addr;
-  struct hostAddrList *next;
+  struct hostAddrList *next, *prev;
 } HostAddrList;
 
-static HostAddrList *hostAddrList_head = NULL;
+static HostAddrList *hostAddrList_head = NULL, *hostAddrList_tail = NULL;
 
+typedef struct {
+  time_t dump_date;
+  char hostname[MAX_LEN_SYM_HOST_NAME];
+} HostNameCache;
+
+/* **************************************************** */
+
+void cacheHostName(HostAddr *addr, char* symbolic) {
+  HostNameCache name;
+  datum data_data, key_data;
+
+  accessMutex(&myGlobals.serialLockMutex, "cacheHostName");
+
+  name.dump_date = myGlobals.actTime;
+  safe_snprintf(__FILE__, __LINE__, name.hostname, sizeof(name.hostname), "%s", symbolic);
+
+  key_data.dptr = (char*)addr, key_data.dsize = sizeof(HostAddr);
+  data_data.dptr = (char*)&name, data_data.dsize = (int)(strlen(name.hostname)+sizeof(name.dump_date)+1);
+
+  if(gdbm_store(myGlobals.resolverCacheFile, key_data, data_data, GDBM_REPLACE) != 0)
+    traceEvent(CONST_TRACE_ERROR, "While adding host name %s", symbolic);
+
+  // traceEvent(CONST_TRACE_INFO, "Cached host name %s", symbolic);
+
+  releaseMutex(&myGlobals.serialLockMutex);
+}
+
+/* **************************************************** */
+
+char* getHostNameFromCache(HostAddr *addr, char *buf, u_int buf_len) {
+  char *ret;
+  datum return_data, key_data;
+
+  accessMutex(&myGlobals.serialLockMutex, "getHostNameFromCache");
+
+  key_data.dptr = (char*)addr, key_data.dsize = sizeof(HostAddr);
+  return_data = gdbm_fetch(myGlobals.resolverCacheFile, key_data);
+
+  if(return_data.dptr != NULL) {
+    HostNameCache *dump = (HostNameCache*)return_data.dptr;
+
+    safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s", dump->hostname);
+    free(return_data.dptr);
+    ret = buf;
+  } else {
+    /* Not found */
+    ret = NULL;
+  }
+
+  releaseMutex(&myGlobals.serialLockMutex);
+
+  // traceEvent(CONST_TRACE_INFO, "Resolved %s", ret ? ret : "<none>");
+
+  return(ret);
+}
 
 /* **************************************** */
 
@@ -95,6 +150,8 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
   traceEvent(CONST_TRACE_INFO, "updateDeviceHostNameInfo(%s <--> %s)", symbolic, addrtostr(&addr));
 #endif
 
+  cacheHostName(&addr, symbolic);
+
   for(i=0; i<myGlobals.numDevices; i++) {
     if(!myGlobals.device[i].virtualDevice)
       updateDeviceHostNameInfo(addr, symbolic, i, type);
@@ -104,7 +161,6 @@ static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
 /* ************************************ */
 
 static void queueAddress(HostAddr elem) {
-  int i;
   HostAddrList *cloned = NULL;
 
   if(myGlobals.runningPref.numericFlag == noDnsResolution) return;
@@ -122,7 +178,8 @@ static void queueAddress(HostAddr elem) {
   if(myGlobals.addressQueuedCurrent > 16384) {
     myGlobals.addressUnresolvedDrops++;
   } else {
-    /* First check if the address we want to resolve is already in queue */
+    /* First check if the address we want to resolve is already in queue 
+       waiting to be resolved */
     HostAddrList *head = hostAddrList_head;
 
     while(head != NULL) {
@@ -138,11 +195,16 @@ static void queueAddress(HostAddr elem) {
     traceEvent(CONST_TRACE_ERROR, "queueAddress(%s)", addrtostr(&elem));
 #endif
 
-    if((cloned = (HostAddrList*)malloc(sizeof(HostAddrList))) != NULL) {
+    if((cloned = (HostAddrList*)calloc(1, sizeof(HostAddrList))) != NULL) {
       memcpy(&cloned->addr, &elem, sizeof(HostAddr));
-      cloned->next = hostAddrList_head;
+      if(hostAddrList_head) hostAddrList_head->prev = cloned;
+      cloned->next = hostAddrList_head, cloned->prev = NULL;
       hostAddrList_head = cloned;
-      signalCondvar(&myGlobals.queueAddressCondvar);
+
+      if(hostAddrList_tail == NULL)
+	hostAddrList_tail = cloned; /* First element of the list */
+
+      signalCondvar(&myGlobals.queueAddressCondvar, 0);
       myGlobals.addressQueuedCurrent++;
       if(myGlobals.addressQueuedCurrent > myGlobals.addressQueuedMax)
 	myGlobals.addressQueuedMax = myGlobals.addressQueuedCurrent;
@@ -154,21 +216,70 @@ static void queueAddress(HostAddr elem) {
 
 /* ************************************ */
 
-void* dequeueAddress(void *_i) {
-  int dqaIndex = (int)((long)_i);
+static void processAddressResRequest(HostAddrList *elem) {
+  if(elem) {
+    struct hostent *he = NULL;
+    int family, size;
+    char theAddr[64];
+#if defined(HAVE_GETHOSTBYADDR_R)
+    struct hostent _hp, *__hp;
+    char buffer[4096]; /* It MUST be 4096 as otherwise on Linux will fail */
+#endif
 
-  traceEvent(CONST_TRACE_INFO,
-	     "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running",
-             (long unsigned int)pthread_self(), dqaIndex+1);
+    memset(theAddr, 0, sizeof(theAddr));
+    addrget(&elem->addr, theAddr, &family, &size);
 
-  while(myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN) {
-    HostAddrList *elem;
+#if defined(HAVE_GETHOSTBYADDR_R)
+#ifdef SOLARIS
+    he = gethostbyaddr_r((const char*)theAddr, size,
+			 family, &_hp,
+			 buffer, sizeof(buffer),
+			 &h_errno);
+#else
+#if 0
+    traceEvent(CONST_TRACE_INFO, "About to resolve %s [family=%d][size=%d]", addrtostr(&elem->addr), family, size);
+#endif
+    if(gethostbyaddr_r((const char*)theAddr, size,
+		       family, &_hp,
+		       buffer, sizeof(buffer),
+		       &__hp, &h_errno) == 0) {
+
+      if(h_errno == 0)
+	he = &_hp;
+      else
+	he = NULL;
+    } else
+      he = NULL;
+#endif
+#else
+    he = gethostbyaddr(theAddr, size, family);
+#endif
+
+    if((he != NULL) && (he->h_name != NULL)) {
+      updateHostNameInfo(elem->addr, he->h_name, FLAG_HOST_SYM_ADDR_TYPE_NAME);
+      accessAddrResMutex("dequeueAddress"); myGlobals.resolvedAddresses++; releaseAddrResMutex();
+    } else {
+#if 0
+      traceEvent(CONST_TRACE_ERROR, "Address resolution failure [%d][%s]", h_errno, hstrerror(h_errno));
+#endif
+      accessAddrResMutex("dequeueAddress"); myGlobals.failedResolvedAddresses++; releaseAddrResMutex();
+    }
+
+    memset(elem, 0, sizeof(HostAddr));
+    free(elem);
+  }
+}
+
+/* ************************************ */
+
+static void* dequeueNextAddress(void) {
+  HostAddrList *elem;
 
 #ifdef DEBUG
     traceEvent(CONST_TRACE_INFO, "DEBUG: Waiting for address to resolve...");
 #endif
 
-    while(hostAddrList_head == NULL) {
+    while(hostAddrList_tail == NULL) {
       if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
       waitCondvar(&myGlobals.queueAddressCondvar);
     }
@@ -177,74 +288,52 @@ void* dequeueAddress(void *_i) {
     traceEvent(CONST_TRACE_INFO, "DEBUG: Address resolution started...");
 #endif
 
-    if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN) break;
-
     accessAddrResMutex("dequeueAddress");
 
-    if(hostAddrList_head != NULL) {
-      elem = hostAddrList_head;
-      hostAddrList_head = hostAddrList_head->next;
+    if(hostAddrList_tail != NULL) {
+      elem = hostAddrList_tail;
+      hostAddrList_tail = hostAddrList_tail->prev;
+
+      if(hostAddrList_head == elem)
+	hostAddrList_head = NULL;
+
+      if(elem->prev != NULL)
+	elem->prev->next = NULL;
+
       if(myGlobals.addressQueuedCurrent > 0) myGlobals.addressQueuedCurrent--;
     } else
       elem = NULL;
 
     releaseAddrResMutex();
 
-    if(elem) {
-      struct hostent *he = NULL;
-      int family, size;
-      char theAddr[64];
-#if defined(HAVE_GETHOSTBYADDR_R)
-      struct hostent _hp, *__hp;
-      char buffer[4096]; /* It MUST be 4096 as otherwise on Linux will fail */
-#endif
+    return(elem);
+}
 
-      memset(theAddr, 0, sizeof(theAddr));
-      addrget(&elem->addr, theAddr, &family, &size);
+/* ************************************ */
 
-#if defined(HAVE_GETHOSTBYADDR_R)
-#ifdef SOLARIS
-      he = gethostbyaddr_r((const char*)theAddr, size,
-			   family, &_hp,
-			   buffer, sizeof(buffer),
-			   &h_errno);
-#else
-#if 0
-      traceEvent(CONST_TRACE_INFO, "About to resolve %s [family=%d][size=%d]", addrtostr(&elem->addr), family, size);
-#endif
-      if(gethostbyaddr_r((const char*)theAddr, size,
-			 family, &_hp,
-			 buffer, sizeof(buffer),
-			 &__hp, &h_errno) == 0) {
+void* dequeueAddress(void *_i) {
+  int dqaIndex = (int)((long)_i);
+  HostAddrList *elem;
 
-	if(h_errno == 0)
-	  he = &_hp;
-	else
-	  he = NULL;
-      } else
-	he = NULL;
-#endif
-#else
-      he = gethostbyaddr(theAddr, size, family);
-#endif
+  traceEvent(CONST_TRACE_INFO,
+	     "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread running",
+             (long unsigned int)pthread_self(), dqaIndex+1);
 
-      if((he != NULL) && (he->h_name != NULL)) {
-	updateHostNameInfo(elem->addr, he->h_name, FLAG_HOST_SYM_ADDR_TYPE_NAME);
-	accessAddrResMutex("dequeueAddress"); myGlobals.resolvedAddresses++; releaseAddrResMutex();
-      } else {
-#if 0
-	traceEvent(CONST_TRACE_ERROR, "Address resolution failure [%d][%s]", h_errno, hstrerror(h_errno));
-#endif
-	accessAddrResMutex("dequeueAddress"); myGlobals.failedResolvedAddresses++; releaseAddrResMutex();
-      }
-
-      memset(elem, 0, sizeof(HostAddr));
-      free(elem);
-    }
+  while(myGlobals.ntopRunState <= FLAG_NTOPSTATE_RUN) {
+    elem = dequeueNextAddress();
+    processAddressResRequest(elem);
   } /* endless loop */
 
   myGlobals.dequeueAddressThreadId[dqaIndex] = 0;
 
+  /* We're shutting down so let'e empty the queue */
+  do {    
+    elem = dequeueNextAddress();
+    
+    if(elem != NULL)
+      free(elem);
+  } while(elem != NULL);
+  
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread terminated [p%d]",
              (long unsigned int)pthread_self(), dqaIndex+1,
 #ifndef WIN32
@@ -266,7 +355,8 @@ void cleanupAddressQueue(void) {
 /* ************************************ */
 
 char* _intop(struct in6_addr *addr, char *buf, u_short buflen) {
-  return (char *)inet_ntop(AF_INET6, addr, buf, buflen);
+  char *ret = (char *)inet_ntop(AF_INET6, addr, buf, buflen);
+  return(ret);
 }
 
 /* ************************************ */
@@ -392,6 +482,8 @@ void ipaddr2str(HostTraffic *el, HostAddr hostIpAddress,
     traceEvent(CONST_TRACE_ERROR, "Recycling %s = %s ", addrtostr(&hostIpAddress), el->hostResolvedName);
 #endif
     strcpy(el->hostResolvedName, h->hostResolvedName), el->hostResolvedNameType = h->hostResolvedNameType;
+  } else if(getHostNameFromCache(&el->hostIpAddress, el->hostResolvedName, sizeof(el->hostResolvedName)) != NULL) {
+    el->hostResolvedNameType = FLAG_HOST_SYM_ADDR_TYPE_NAME;
   } else
     queueAddress(hostIpAddress);
 }
@@ -577,7 +669,7 @@ static int _ns_name_ntop(const u_char *src,
     return (-1);
   }
   *dn++ = '\0';
-  return (dn - dst);
+  return((int)(dn - dst));
 }
 
 /* ************************************ */
@@ -653,8 +745,9 @@ static int _ns_name_unpack(const u_char *msg,
 	errno = EMSGSIZE;
 	return (-1);
       }
+
       if (len < 0)
-	len = srcp - src + 1;
+	len = (int)(srcp - src + 1);
       srcp = msg + (((n & 0x3f) << 8) | (*srcp & 0xff));
       if (srcp < msg || srcp >= eom) {  /* Out of range. */
 	errno = EMSGSIZE;
@@ -679,7 +772,7 @@ static int _ns_name_unpack(const u_char *msg,
   }
   *dstp = '\0';
   if (len < 0)
-    len = srcp - src;
+    len = (int)(srcp - src);
   return (len);
 }
 
@@ -735,13 +828,13 @@ static int _dn_skipname(const u_char *ptr, const u_char *eom) {
 
   if (_ns_name_skip(&ptr, eom) == -1)
     return (-1);
-  return (ptr - saveptr);
+  return((int)(ptr - saveptr));
 }
 
 /* ************************************ */
 
 static void msdns_filter_name(char *msg) {
-  int i, j, max = strlen(msg);
+  int i, j, max = (int)strlen(msg);
 
   for(i=0, j=0; i<max; i++) {
 #ifdef MDNS_DEBUG
@@ -1084,7 +1177,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
       char *a, *b, *c, *d, dnsBuf[128], *strtokState = NULL;
       unsigned long theDNSaddr;
 
-      len = strlen((char*)bp);
+      len = (int)strlen((char*)bp);
 
       if(bp[0] == '_') {
 	/* Multicast DNS */
@@ -1114,7 +1207,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
 	  if(addr_list_idx >= MAX_ADDRESSES) break; /* Further check */
 	  memcpy(&addr_list[addr_list_idx++], (char*)&theDNSaddr, sizeof(char*));
 	  hostPtr->addrLen = INADDRSZ;
-	  hostPtr->addrList[0] = theDNSaddr;
+	  hostPtr->addrList[0] = (u_int32_t) theDNSaddr;
 
 	  n = (short)dn_expand_(answer.qb2, eom, cp, (char *)bp, buflen);
 	  if (n < 0) {
@@ -1122,7 +1215,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
 	    continue;
 	  }
 	  cp += n;
-	  len = strlen((char *)bp) + 1;
+	  len = (int)(strlen((char *)bp) + 1);
 	  memcpy(hostPtr->name, bp, len);
 	  haveAnswer = TRUE;
 	}
@@ -1155,7 +1248,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
       hostPtr->addrLen = dlen;
       origClass = class;
       hostPtr->addrType = (class == C_IN) ? AF_INET : AF_UNSPEC;
-      len = strlen((char *)bp) + 1;
+      len = (int)(strlen((char *)bp) + 1);
       memcpy(hostPtr->name, bp, len);
     }
 
@@ -1166,7 +1259,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
     {
       u_int32_t     padding;
 
-      padding=((u_int32_t)((long)bp)) % sizeof(u_int32_t);
+      padding = (u_int32_t)(((long)bp) % sizeof(u_int32_t));
       bp += padding;
       buflen -= padding;
     }
@@ -1244,7 +1337,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
       handleMdnsName(srcHost, sport, bp);
 
     cp += n;
-    len = strlen((char *)bp) + 1;
+    len = (int)(strlen((char *)bp) + 1);
 
     if (cp + 3 * INT16SZ + INT32SZ > eom)
       return(transactionId);
@@ -1265,7 +1358,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
 
 /* **************************************** */
 
-void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
+void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId, const struct pcap_pkthdr *h, const u_char *p) {
   HostTraffic *el;
 
   for(el=getFirstHost(actualDeviceId);
@@ -1282,12 +1375,11 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
 	  traceEvent(CONST_TRACE_WARNING,
 		     "Two MAC addresses found for the same IP address %s: [%s/%s] (spoofing detected?)",
 		     el->hostNumIpAddress, hostToCheck->ethAddressString, el->ethAddressString);
-	  dumpSuspiciousPacket(actualDeviceId);
+	  dumpSuspiciousPacket(actualDeviceId, h, p);
 	}
       }
     }
   }
-
 }
 
 /* **************************************** */
