@@ -5,7 +5,7 @@
 # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 #
 # Copyright (C) 2004-06 Rocco Carbone <rocco@ntop.org>
-#               2006-10 Luca Deri <deri@ntop.org>
+#               2006-12 Luca Deri <deri@ntop.org>
 #
 # -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 #
@@ -159,7 +159,7 @@ EOF
           
     rm -f plugins/Makefile
     rm -f plugins/Makefile.in
-
+    rm -rf nDPI
     rm -f *~
 
     exit 0
@@ -216,6 +216,20 @@ if test "$GNU_OR_DIE" -eq 0; then
   exit 1
 fi
 
+SVN=`find_command 'svn'`
+($SVN --version) < /dev/null > /dev/null 2>&1 ||
+{
+  echo
+  echo "You must have svn/subversion installed to compile $progname."
+  echo "Download the appropriate package for your distribution, or get the"
+  echo "source from http://subversion.tigris.org"
+  GNU_OR_DIE=0
+}
+
+if test "$GNU_OR_DIE" -eq 0; then
+  exit 1
+fi
+
 # Check versions...
 libtoolversion=`$LIBTOOL --version < /dev/null 2>&1 | grep libtool | cut -d " " -f 4`
 echo "    libtool ..... ${libtoolversion}"
@@ -420,6 +434,28 @@ rm -f /tmp/acin* /tmp/acout*
 rm -f autoha*
 rm -f confdefs.h
 
+# Get nDPI
+
+echo "9. Downloading nDPI..."
+
+NDPI_URL=https://svn.ntop.org/svn/ntop/trunk/nDPI/
+if test -d nDPI; then
+    echo "nDPI already available"
+else
+    svn co $NDPI_URL
+fi
+
+dnl> nDPI compilation
+if test -f NDPI_LIB; then
+    echo "nDPI already compiled"
+else
+    echo "10. Compiling nDPI..."
+    cd nDPI; ./configure --with-pic; make; cd ..
+fi
+
+
+echo
+echo "Now we're ready to compile ntop"
 
 # Local Variables: 
 # mode:shell-script 
