@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -113,10 +113,10 @@ char* formatAdapterSpeed(Counter numBits, char* outStr, int outStrLen) {
 /* ******************************* */
 
 char* formatSeconds(unsigned long sec, char* outStr, int outStrLen) {
-  unsigned int hour=0, min=0, days=0;
+  u_int hour = 0, min = 0, days = 0;
 
   if(sec >= 3600) {
-    hour = (sec / 3600);
+    hour = (u_int)(sec / 3600);
 
     if(hour > 0) {
       if(hour >= 24) {
@@ -129,7 +129,7 @@ char* formatSeconds(unsigned long sec, char* outStr, int outStrLen) {
       hour = 0;
   }
 
-  min = (sec / 60);
+  min = (u_int)(sec / 60);
   if(min > 0) sec -= min*60;
 
   if(days > 0) {
@@ -142,7 +142,7 @@ char* formatSeconds(unsigned long sec, char* outStr, int outStrLen) {
       yearStr[0] = '\0';
 
     safe_snprintf(__FILE__, __LINE__, outStr, outStrLen, "%s%u day%s %u:%02u:%02lu", 
-		yearStr, days, (days>1)?"s":"", hour, min, sec);
+		  yearStr, days, (days>1)?"s":"", hour, min, sec);
   } else if(hour > 0) {
     safe_snprintf(__FILE__, __LINE__, outStr, outStrLen, "%u:%02u:%02lu", hour, min, sec);
   } else if(min > 0) {
@@ -214,7 +214,7 @@ char* formatLatency(struct timeval tv, u_short sessionState, char* outStr, int o
     return("&nbsp;");
   } else {
     safe_snprintf(__FILE__, __LINE__, outStr, outStrLen, "%.02f&nbsp;ms",
-	    (float)(tv.tv_sec*1000+(float)tv.tv_usec/1000));
+		  ((float)tv.tv_sec*1000+(float)tv.tv_usec/1000));
     return(outStr);
   }
 }
@@ -287,9 +287,14 @@ char* _formatTime(time_t *theTime, char* outStr, int outStrLen, char* file, int
   struct tm *locTime;
   struct tm myLocTime;
 
-  locTime = localtime_r(theTime, &myLocTime);
-  if(strftime(outStr, outStrLen, CONST_LOCALE_TIMESPEC, locTime) == 0)
-    traceEvent(CONST_TRACE_ERROR, "Buffer too short @ %s:%d for formatTime()", file, line);
+  if((locTime = localtime_r(theTime, &myLocTime)) == NULL) {
+    traceEvent(CONST_TRACE_WARNING, "localtime_r failed:  [%d/%s]", errno, strerror(errno));
+    outStr[0] = '\0';
+  } else {
+    if(strftime(outStr, outStrLen, CONST_LOCALE_TIMESPEC, locTime) == 0)
+      traceEvent(CONST_TRACE_ERROR, "Buffer too short @ %s:%d for formatTime() [%s]",
+		 file, line, outStr);
+  }
 
   return(outStr);
 }
