@@ -321,6 +321,7 @@ static void wrtFloatItm(FILE *fDescr, int lang, char *indent, char *name,
 
 /* *************************** */
 
+#if 0
 static void wrtIntFloatItm(FILE *fDescr, int lang, char *indent, int name,
 			   float value, char last, int numEntriesSent) {
   char buf[80];
@@ -328,6 +329,7 @@ static void wrtIntFloatItm(FILE *fDescr, int lang, char *indent, int name,
   wrtFloatItm(fDescr, lang, indent, (lang == FLAG_XML_LANGUAGE) ? "number" : buf,
 	      value, last, numEntriesSent);
 }
+#endif
 
 /* *************************** */
 
@@ -589,10 +591,10 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
       }
     }
 
-    if(checkFilter(filter, "pktSent"))
-      wrtLlongItm(fDescr, lang, "\t", "pktSent",   el->pktSent, ',', numEntries);
-    if(checkFilter(filter, "pktRcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "pktRcvd", el->pktRcvd, ',', numEntries);
+    if(checkFilter(filter, "pktsSent"))
+      wrtLlongItm(fDescr, lang, "\t", "pktsSent",   el->pktsSent, ',', numEntries);
+    if(checkFilter(filter, "pktsRcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "pktsRcvd", el->pktsRcvd, ',', numEntries);
 
     if(checkFilter(filter, "ipv4BytesSent"))
       wrtLlongItm(fDescr, lang, "\t", "ipv4BytesSent", el->ipv4BytesSent, ',', numEntries);
@@ -602,23 +604,23 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
     /* *************************************** */
 
     if(!shortView) {
-      if(checkFilter(filter, "pktDuplicatedAckSent"))
-	wrtLlongItm(fDescr, lang, "\t", "pktDuplicatedAckSent",el->pktDuplicatedAckSent, ',', numEntries);
-      if(checkFilter(filter, "pktDuplicatedAckRcvd"))
-	wrtLlongItm(fDescr, lang, "\t", "pktDuplicatedAckRcvd",el->pktDuplicatedAckRcvd, ',', numEntries);
-      if(checkFilter(filter, "pktBroadcastSent"))
-	wrtLlongItm(fDescr, lang, "\t", "pktBroadcastSent",  el->pktBroadcastSent, ',', numEntries);
+      if(checkFilter(filter, "pktsDuplicatedAckSent"))
+	wrtLlongItm(fDescr, lang, "\t", "pktsDuplicatedAckSent",el->pktsDuplicatedAckSent, ',', numEntries);
+      if(checkFilter(filter, "pktsDuplicatedAckRcvd"))
+	wrtLlongItm(fDescr, lang, "\t", "pktsDuplicatedAckRcvd",el->pktsDuplicatedAckRcvd, ',', numEntries);
+      if(checkFilter(filter, "pktsBroadcastSent"))
+	wrtLlongItm(fDescr, lang, "\t", "pktsBroadcastSent",  el->pktsBroadcastSent, ',', numEntries);
     }
 
     if(checkFilter(filter, "bytesMulticastSent"))
       wrtLlongItm(fDescr, lang, "\t", "bytesMulticastSent", el->bytesMulticastSent, ',', numEntries);
-    if(checkFilter(filter, "pktMulticastSent"))
-      wrtLlongItm(fDescr, lang, "\t", "pktMulticastSent",  el->pktMulticastSent, ',', numEntries);
+    if(checkFilter(filter, "pktsMulticastSent"))
+      wrtLlongItm(fDescr, lang, "\t", "pktsMulticastSent",  el->pktsMulticastSent, ',', numEntries);
 
     if(checkFilter(filter, "bytesMulticastRcvd"))
       wrtLlongItm(fDescr, lang, "\t", "bytesMulticastRcvd", el->bytesMulticastRcvd, ',', numEntries);
-    if(checkFilter(filter, "pktMulticastRcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "pktMulticastRcvd",  el->pktMulticastRcvd, ',', numEntries);
+    if(checkFilter(filter, "pktsMulticastRcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "pktsMulticastRcvd",  el->pktsMulticastRcvd, ',', numEntries);
 
     /* *************************************** */
 
@@ -664,32 +666,12 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
       if(checkFilter(filter, "peakSentThpt"))
 	wrtFloatItm(fDescr, lang, "\t", "peakSentThpt",    el->peakSentThpt, ',', numEntries);
 
-      if(checkFilter(filter, "actualTThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "actualTThpt",  el->actualTThpt, ',', numEntries);
-      if(checkFilter(filter, "averageTThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "averageTThpt", el->averageTThpt, ',', numEntries);
-      if(checkFilter(filter, "peakTThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "peakTThpt",    el->peakTThpt, ',', numEntries);
-
-      if(checkFilter(filter, "actualRcvdPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "actualRcvdPktThpt", el->actualRcvdPktThpt, ',', numEntries);
-      if(checkFilter(filter, "averageRcvdPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "averageRcvdPktThpt",el->averageRcvdPktThpt, ',', numEntries);
-      if(checkFilter(filter, "peakRcvdPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "peakRcvdPktThpt", el->peakRcvdPktThpt, ',', numEntries);
-      if(checkFilter(filter, "actualSentPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "actualSentPktThpt", el->actualSentPktThpt, ',', numEntries);
-      if(checkFilter(filter, "averageSentPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "averageSentPktThpt", el->averageSentPktThpt, ',', numEntries);
-      if(checkFilter(filter, "peakSentPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "peakSentPktThpt", el->peakSentPktThpt, ',', numEntries);
-
-      if(checkFilter(filter, "actualTPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "actualTPktThpt", el->actualTPktThpt, ',', numEntries);
-      if(checkFilter(filter, "averageTPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "averageTPktThpt",el->averageTPktThpt, ',', numEntries);
-      if(checkFilter(filter, "peakTPktThpt"))
-	wrtFloatItm(fDescr, lang, "\t", "peakTPktThpt", el->peakTPktThpt, ',', numEntries);
+      if(checkFilter(filter, "actualThpt"))
+	wrtFloatItm(fDescr, lang, "\t", "actualThpt",  el->actualThpt, ',', numEntries);
+      if(checkFilter(filter, "averageThpt"))
+	wrtFloatItm(fDescr, lang, "\t", "averageThpt", el->averageThpt, ',', numEntries);
+      if(checkFilter(filter, "peakThpt"))
+	wrtFloatItm(fDescr, lang, "\t", "peakThpt",    el->peakThpt, ',', numEntries);
     }
 
     if(checkFilter(filter, "ipv6BytesSent"))
@@ -794,32 +776,6 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
       }
     } /* shortView */
 
-    if((!shortView) && (el->protoIPTrafficInfos && checkFilter(filter, "IP"))) {
-      char *lastKey = NULL;
-
-      initWriteKey(fDescr, lang, "\t", "IP", numEntries);
-
-      for(j=0; j<myGlobals.numIpProtosToMonitor; j++) {
-	if((myGlobals.ipTrafficProtosNames[j] != NULL)
-	   && (el->protoIPTrafficInfos[j] != NULL)) {
-	  if(j > 0) { endWriteKey(fDescr, lang,"\t\t", lastKey, ','); }
- 
-	  initWriteKey(fDescr, lang, "\t\t", (lastKey = myGlobals.ipTrafficProtosNames[j]), numEntries);
-	  wrtLlongItm(fDescr, lang,"\t\t\t","sentLoc",
-		      el->protoIPTrafficInfos[j]->sentLoc, ',', numEntries);
-	  wrtLlongItm(fDescr, lang,"\t\t\t","sentRem",
-		      el->protoIPTrafficInfos[j]->sentRem, ',', numEntries);
-	  wrtLlongItm(fDescr, lang,"\t\t\t","rcvdLoc",
-		      el->protoIPTrafficInfos[j]->rcvdLoc, ',', numEntries);
-	  wrtLlongItm(fDescr, lang,"\t\t\t","rcvdFromRem",
-		      el->protoIPTrafficInfos[j]->rcvdFromRem, ' ', numEntries);
-	}
-      } /* for */
-
-      if(lastKey != NULL) { endWriteKey(fDescr, lang,"\t\t", lastKey, ','); }
-      endWriteKey(fDescr, lang,"\t", "IP", ',');
-    }
-
     /* ***************************************** */
 
     if(!shortView) {
@@ -1100,7 +1056,7 @@ void dumpNtopHashIndexes(FILE *fDescr, char* options, int actualDeviceId) {
 
 void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
   char intoabuf[32], key[16], localbuf[32], filter[128], *keyName = NULL;
-  int lang=DEFAULT_FLAG_LANGUAGE, i, numEntries, localView=0;
+  int lang=DEFAULT_FLAG_LANGUAGE, i, numEntries;
   unsigned short shortView = 0;
 
   memset(key, 0, sizeof(key));
@@ -1134,8 +1090,6 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	  strncpy(key, &tmpStr[i+1], sizeof(key));
 	} else if(strcmp(tmpStr, "view") == 0) {
 	  if(!strcmp(&tmpStr[i+1], "short")) shortView = 1;
-	} else if(strcmp(tmpStr, "restrict") == 0) {
-	  if(!strcmp(key, "local")) localView = 1; /* not yet used */
 	} else if(strcmp(tmpStr, "filter") == 0) {
 	  strncpy(filter, &tmpStr[i+1], sizeof(filter));
 	}
@@ -1228,8 +1182,6 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       wrtLlongItm(fDescr, lang, "\t", "otherIpBytes",myGlobals.device[i].otherIpBytes, ',', numEntries);
     if(checkFilter(filter, "icmpBytes"))
       wrtLlongItm(fDescr, lang, "\t", "icmpBytes",myGlobals.device[i].icmpBytes, ',', numEntries);
-    if(checkFilter(filter, "dlcBytes"))
-      wrtLlongItm(fDescr, lang, "\t", "dlcBytes",myGlobals.device[i].dlcBytes, ',', numEntries);
 
     if(checkFilter(filter, "stpBytes"))
       wrtLlongItm(fDescr, lang, "\t", "stpBytes",myGlobals.device[i].stpBytes, ',', numEntries);
@@ -1239,8 +1191,6 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       wrtLlongItm(fDescr, lang, "\t", "netbiosBytes",myGlobals.device[i].netbiosBytes, ',', numEntries);
     if(checkFilter(filter, "arpRarpBytes"))
       wrtLlongItm(fDescr, lang, "\t", "arpRarpBytes",myGlobals.device[i].arpRarpBytes, ',', numEntries);
-    if(checkFilter(filter, "egpBytes"))
-      wrtLlongItm(fDescr, lang, "\t", "egpBytes",myGlobals.device[i].egpBytes, ',', numEntries);
     if(checkFilter(filter, "greBytes"))
       wrtLlongItm(fDescr, lang, "\t", "greBytes",myGlobals.device[i].greBytes, ',', numEntries);
     if(checkFilter(filter, "ipv6Bytes"))
@@ -1311,41 +1261,6 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	wrtFloatItm(fDescr, lang, "\t", "throughput", myGlobals.device[i].throughput, ',', numEntries);
       if(checkFilter(filter, "packetThroughput"))
 	wrtFloatItm(fDescr, lang, "\t", "packetThroughput",myGlobals.device[i].packetThroughput, ',', numEntries);
-
-      /* ********************************* */
-
-      if(checkFilter(filter, "last60MinutesThpt")) {
-	initWriteKey(fDescr, lang, "\t", "last60MinutesThpt", numEntries);
-
-	for(j=0; j<59; j++) {
-	  wrtIntFloatItm(fDescr, lang,"\t\t",j+1, myGlobals.device[i].last60MinutesThpt[j].trafficValue, ',', numEntries);
-	}
-	wrtIntFloatItm(fDescr, lang,"\t\t",j+1, myGlobals.device[i].last60MinutesThpt[j].trafficValue, ' ', numEntries);
-	endWriteKey(fDescr, lang,"\t", "last60MinutesThpt", ',');
-      }
-
-      /* ********************************* */
-
-      if(checkFilter(filter, "last24HoursThpt")) {
-	initWriteKey(fDescr, lang, "\t", "last24HoursThpt", numEntries);
-
-	for(j=0; j<23; j++) {
-	  wrtIntFloatItm(fDescr, lang, "\t\t", j+1, myGlobals.device[i].last24HoursThpt[j].trafficValue, ',', numEntries);
-	}
-	wrtIntFloatItm(fDescr, lang,"\t\t",j+1,myGlobals.device[i].last24HoursThpt[j].trafficValue, ' ', numEntries);
-	endWriteKey(fDescr, lang,"\t", "last24HoursThpt", ',');
-      }
-      /* ********************************* */
-
-      if(checkFilter(filter, "last30daysThpt")) {
-	initWriteKey(fDescr, lang, "\t", "last30daysThpt", numEntries);
-
-	for(j=0; j<29; j++) {
-	  wrtIntFloatItm(fDescr, lang,"\t\t",j+1,myGlobals.device[i].last30daysThpt[j], ',', numEntries);
-	}
-	wrtIntFloatItm(fDescr, lang,"\t\t",j+1,myGlobals.device[i].last30daysThpt[j], ' ', numEntries);
-	endWriteKey(fDescr, lang,"\t", "last30daysThpt", ',');
-      }
     }
 
     /* ********************************* */
@@ -1353,39 +1268,29 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
     if(checkFilter(filter, "IP")) {
       char *hostKey = NULL;
 
-      if(myGlobals.device[i].ipProtoStats != NULL) {
-	if(!shortView) { initWriteKey(fDescr, lang, "\t", "IP", numEntries); }
-
-	for(j=0; j<myGlobals.numIpProtosToMonitor; j++) {
-	  if((myGlobals.ipTrafficProtosNames[j] != NULL) && (!shortView)) {
-	    if(j > 0) endWriteKey(fDescr, lang, "\t\t", hostKey, ',');
-	    initWriteKey(fDescr, lang, "\t\t", (hostKey = myGlobals.ipTrafficProtosNames[j]), numEntries);
-	    wrtLlongItm(fDescr, lang, "\t\t\t", "local",
-			myGlobals.device[i].ipProtoStats[j].local, ',', numEntries);
-	    wrtLlongItm(fDescr, lang, "\t\t\t", "local2remote",
-			myGlobals.device[i].ipProtoStats[j].local2remote, ',', numEntries);
-	    wrtLlongItm(fDescr, lang, "\t\t\t", "remote2local",
-			myGlobals.device[i].ipProtoStats[j].remote2local, ',', numEntries);
-	    wrtLlongItm(fDescr, lang, "\t\t\t", "remote",
-			myGlobals.device[i].ipProtoStats[j].remote, ' ', numEntries);
-	  } else {
-	    TrafficCounter ctr;
-
-	    ctr.value =
-	      myGlobals.device[i].ipProtoStats[j].local.value+
-	      myGlobals.device[i].ipProtoStats[j].local2remote.value+
-	      myGlobals.device[i].ipProtoStats[j].remote2local.value+
-	      myGlobals.device[i].ipProtoStats[j].remote.value;
-
-	    wrtLlongItm(fDescr, lang, "\t", myGlobals.ipTrafficProtosNames[j], ctr, ',', numEntries);
-	  }
-	}
+      if(!shortView) { initWriteKey(fDescr, lang, "\t", "IP", numEntries); }
 
+      for(j=0; j<myGlobals.l7.numSupportedProtocols; j++) {
 	if(!shortView) {
-	  if(hostKey != NULL) endWriteKey(fDescr, lang,"\t\t", hostKey, ',');
-	  endWriteKey(fDescr, lang,"\t", "IP", ',');
+	  TrafficCounter tc;
+	  
+	  if(j > 0) endWriteKey(fDescr, lang, "\t\t", hostKey, ',');
+	  initWriteKey(fDescr, lang, "\t\t", (hostKey = getProtoName(0, j)), numEntries);
+	  tc.value = myGlobals.device[i].l7.protoTraffic[j];
+	  wrtLlongItm(fDescr, lang, "\t\t\t", "bytes", tc, ',', numEntries);
+	} else {
+	  TrafficCounter ctr;
+
+	  ctr.value = myGlobals.device[i].l7.protoTraffic[j];
+
+	  wrtLlongItm(fDescr, lang, "\t", getProtoName(0, j), ctr, ',', numEntries);
 	}
       }
+
+      if(!shortView) {
+	if(hostKey != NULL) endWriteKey(fDescr, lang,"\t\t", hostKey, ',');
+	endWriteKey(fDescr, lang,"\t", "IP", ',');
+      }
     }
 
     /* ********************************* */
