@@ -3,7 +3,7 @@
  *
  *                          http://www.ntop.org
  *
- *          Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *          Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -37,7 +37,7 @@ int optopt;
 #endif
 
 #ifdef WIN32
-char *version, *osName, *author, *buildDate, *configureDate,
+char *version, *osName, *ntop_author, *buildDate, *configureDate,
   *configure_parameters,
   *host_system_type,
   *target_system_type,
@@ -90,6 +90,10 @@ void initGdbm(char *prefDirectory,  /* Directory with persistent files */
 
   initSingleGdbm(&myGlobals.macPrefixFile,    "macPrefix.db",    spoolDirectory, FALSE,  &statbuf);
   initSingleGdbm(&myGlobals.fingerprintFile,  "fingerprint.db",  spoolDirectory, FALSE,  &statbuf);
+  initSingleGdbm(&myGlobals.serialFile,       "hostSerials.db",  spoolDirectory, TRUE,   &statbuf);
+  initSingleGdbm(&myGlobals.resolverCacheFile,"resolverCache.db",spoolDirectory, TRUE,   &statbuf);
+  initSingleGdbm(&myGlobals.topTalkersFile,   "topTalkers.db",   spoolDirectory, FALSE,  &statbuf);
+
   createVendorTable(&statbuf);
 
   checkCommunities(); /* Check if communities are defined */
@@ -114,6 +118,10 @@ static void allocateOtherHosts(void) {
   myGlobals.otherHostEntry->hostResolvedNameType = FLAG_HOST_SYM_ADDR_TYPE_FAKE;
   strcpy(myGlobals.otherHostEntry->ethAddressString, "00:00:00:00:00:00");
   myGlobals.otherHostEntry->portsUsage = NULL;
+
+  myGlobals.otherHostEntry->l7.traffic = (ProtoTraffic*)calloc(myGlobals.l7.numSupportedProtocols+1, sizeof(ProtoTraffic));  
+  myGlobals.otherHostEntry->serialHostIndex = ++myGlobals.hostSerialCounter; /* Start from 1 (0 = UNKNOWN_SERIAL_INDEX) */
+  myGlobals.otherHostEntry->magic = CONST_MAGIC_NUMBER;
 }
 
 /* ************************************ */
@@ -132,14 +140,15 @@ void extend8021Qmtu(void) {
  */
 void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_started[]) {
   int i, bufLen;
-  char *startedAs, *defaultPath;
+  char *startedAs, *defaultPath, *c;
 
   /*
    * Notice the program name
    */
-  myGlobals.program_name = strrchr(argv[0], CONST_PATH_SEP);
-  myGlobals.program_name =
-    (!myGlobals.program_name || !myGlobals.program_name[0]) ? (argv[0]) : (++myGlobals.program_name);
+  if((c = strrchr(argv[0], CONST_PATH_SEP)) == NULL)
+    myGlobals.program_name = argv[0];
+  else
+    myGlobals.program_name = &c[1];
 
   /*
    * save command line parameters
@@ -154,18 +163,9 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
     myGlobals.runningPref.daemonMode = 1;
   }
 
-  /* note that by default ntop will merge network interfaces */
-  if(myGlobals.runningPref.mergeInterfaces == 0)
-    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled by default");
-  else
-    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge enabled by default");
-
   myGlobals.checkVersionStatus = FLAG_CHECKVERSION_NOTCHECKED;
   myGlobals.checkVersionStatusAgain = 1;
 
-  /* Other flags (to be set via command line options one day) */
-  myGlobals.enableFragmentHandling = 1;
-
   /* Search paths */
   myGlobals.dataFileDirs    = _dataFileDirs;
   myGlobals.pluginDirs      = _pluginDirs;
@@ -287,13 +287,15 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   createMutex(&myGlobals.portsMutex);       /* Avoid race conditions while handling ports */
 
   for(i=0; i<NUM_SESSION_MUTEXES; i++)
-    createMutex(&myGlobals.tcpSessionsMutex[i]); /* data to synchronize TCP sessions access */
+    createMutex(&myGlobals.sessionsMutex[i]); /* data to synchronize sessions access */
 
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
   createMutex(&myGlobals.purgeMutex);       /* synchronize purging */
   createMutex(&myGlobals.securityItemsMutex);
   createMutex(&myGlobals.hostsHashLockMutex);
 
+  createMutex(&myGlobals.serialLockMutex);  /* Serial host locking */
+
   for(i=0; i<CONST_HASH_INITIAL_SIZE; i++) {
     createMutex(&myGlobals.hostsHashMutex[i]);
     myGlobals.hostsHashMutexNumLocks[i] = 0;
@@ -406,8 +408,13 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   myGlobals.numPurgedHosts = myGlobals.numTerminatedSessions = 0;
 
+  /* Dummy value just to be safe: it will be set later on */
+  myGlobals.l7.numSupportedProtocols = 2 * IPOQUE_MAX_SUPPORTED_PROTOCOLS;
+
   myGlobals.broadcastEntry = (HostTraffic*)malloc(sizeof(HostTraffic));
   memset(myGlobals.broadcastEntry, 0, sizeof(HostTraffic));
+  myGlobals.broadcastEntry->l7.traffic = (ProtoTraffic*)calloc(myGlobals.l7.numSupportedProtocols+1, 
+							       sizeof(ProtoTraffic));
   resetHostsVariables(myGlobals.broadcastEntry);
 
   /* Set address to FF:FF:FF:FF:FF:FF */
@@ -425,6 +432,8 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
   setHostFlag(FLAG_BROADCAST_HOST, myGlobals.broadcastEntry);
   setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, myGlobals.broadcastEntry);
   memset(&myGlobals.broadcastEntry->hostSerial, 0, sizeof(HostSerial));
+  myGlobals.broadcastEntry->serialHostIndex = ++myGlobals.hostSerialCounter; /* Start from 1 (0 = UNKNOWN_SERIAL_INDEX) */
+  myGlobals.broadcastEntry->magic = CONST_MAGIC_NUMBER;
 
   allocateOtherHosts();
 
@@ -432,14 +441,14 @@ void initNtopGlobals(int argc, char * argv[], int argc_started, char *argv_start
 
   bufLen = 0;
   for (i=0; i<argc_started; i++) {
-    bufLen += (2 + strlen(argv_started[i]));
+    bufLen += (int)(2 + strlen(argv_started[i]));
   }
 
   startedAs = (char*)malloc(bufLen);
   memset(startedAs, 0, (size_t) bufLen);
   for (i=0; i<argc_started; i++) {
     if (argv_started[i] != NULL) {
-      int displ = strlen(startedAs);
+      int displ = (int)strlen(startedAs);
 
       snprintf(&startedAs[displ], bufLen-displ, "%s ", argv_started[i]);
     }
@@ -506,6 +515,43 @@ static void loadGeoIP(void) {
 
 /* ********************************* */
 
+static void debug_printf(u_int32_t protocol, void *id_struct, 
+			 ipq_log_level_t log_level, const char *format, ...) { ; }
+
+static void *malloc_wrapper(unsigned long size) { return malloc(size); }
+
+/* ********************************* */
+
+void initL7DeviceDiscovery(int deviceId) {
+  IPOQUE_PROTOCOL_BITMASK all;
+  u32 detection_tick_resolution = 1000;
+
+  if(myGlobals.runningPref.disablenDPI) return;
+
+  myGlobals.device[deviceId].l7.l7handler = ipoque_init_detection_module(detection_tick_resolution, malloc_wrapper, debug_printf);
+  if(myGlobals.device[deviceId].l7.l7handler == NULL) {
+    traceEvent(CONST_TRACE_ERROR, "Unable to initialize L7 engine: disabling L7 discovery for deviceId %u", deviceId);
+    return;
+  }
+
+  // enable all protocols
+  IPOQUE_BITMASK_SET_ALL(all);
+  ipoque_set_protocol_detection_bitmask2(myGlobals.device[deviceId].l7.l7handler, &all);
+
+  /* ************************** */
+
+  createMutex(&myGlobals.device[deviceId].l7.l7Mutex);
+}
+
+/* ********************************* */
+
+static void initL7Discovery(void) {
+  myGlobals.l7.proto_size = ipoque_detection_get_sizeof_ipoque_id_struct();
+  myGlobals.l7.flow_struct_size = ipoque_detection_get_sizeof_ipoque_flow_struct();
+}
+
+/* ********************************* */
+
 void initNtop(char *devices) {
   char value[32];
 
@@ -515,11 +561,13 @@ void initNtop(char *devices) {
   initIPServices();
   handleProtocols();
 
+  myGlobals.l7.numSupportedProtocols = IPOQUE_MAX_SUPPORTED_PROTOCOLS;
+
   if(myGlobals.numIpProtosToMonitor == 0)
     addDefaultProtocols();
 
   /*
-   * initialize memory and data.
+   * Initialize memory and data.
    */
   initDevices(devices);
 
@@ -528,10 +576,16 @@ void initNtop(char *devices) {
   if(myGlobals.runningPref.enableSessionHandling)
     initPassiveSessions();
 
+  initL7Discovery();
+
   /* ********************************** */
 
   initGdbm(myGlobals.dbPath, myGlobals.spoolPath, 0);
 
+  /* We just initialized gdbm: let's now dump serials */
+  dumpHostSerial(&myGlobals.broadcastEntry->hostSerial, myGlobals.broadcastEntry->serialHostIndex);
+  dumpHostSerial(&myGlobals.otherHostEntry->hostSerial, myGlobals.otherHostEntry->serialHostIndex);
+
   if(myGlobals.runningPref.daemonMode) {
     /*
       Before bacoming a daemon we need o make sure that
@@ -611,6 +665,12 @@ void initNtop(char *devices) {
   init_maps();
   loadGeoIP();
 
+  /* Note that by default ntop will merge network interfaces */
+  if(myGlobals.runningPref.mergeInterfaces == 0)
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled by default");
+  else
+    traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge enabled by default");
+
   if(fetchPrefsValue("globals.displayPolicy", value, sizeof(value)) == -1) {
     myGlobals.hostsDisplayPolicy = showAllHosts /* 0 */;
     storePrefsValue("globals.displayPolicy", "0");
