@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *         Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *         Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -125,6 +125,33 @@ typedef struct ether80211q {
   u_int16_t protoType;
 } Ether80211q;
 
+/* PPPoE - Courtesy of Andreas Pfaller Feb2003 */
+#ifdef HAVE_LINUX_IF_PPPOX_H
+#include <linux/if_pppox.h>
+#else
+/* Extracted and modified from the Linux header for other systems - BMS Mar2003 */
+/* And for Linux systems without if_pppox.h - BMS Apr2003 */
+struct pppoe_tag {
+  u_int16_t tag_type;
+  u_int16_t tag_len;
+  char tag_data;
+};
+
+struct pppoe_hdr {
+#ifdef CFG_LITTLE_ENDIAN
+  u_int8_t ver : 4;
+  u_int8_t type : 4;
+#else
+  u_int8_t type : 4;
+  u_int8_t ver : 4;
+#endif
+  u_int8_t code;
+  u_int16_t sid;
+  u_int16_t length;
+  struct pppoe_tag tag;
+};
+#endif
+
 typedef struct _mac_t {
     u_int8_t mact_octet[6];
 } mac_t;
@@ -140,12 +167,14 @@ typedef struct hostAddr {
 #define Ip4Address addr._hostIp4Address
 
 #define Ip6Address addr._hostIp6Address
-#define SIZEOF_HOSTSERIAL   8
+#define SIZEOF_HOSTSERIAL    8
 
-#define SERIAL_NONE         0
-#define SERIAL_MAC          1
-#define SERIAL_IPV4         2
-#define SERIAL_IPV6         3
+#define UNKNOWN_SERIAL_INDEX 0
+
+#define SERIAL_NONE          0
+#define SERIAL_MAC           1
+#define SERIAL_IPV4          2
+#define SERIAL_IPV6          3 
 
 typedef struct _ethSerial {
   u_char  ethAddress[LEN_ETHERNET_ADDRESS];
@@ -158,13 +187,35 @@ typedef struct _ipSerial {
 } IpSerial;
 
 typedef struct hostSerial {
-  u_char serialType;     /* 0 == empty */
+  u_int8_t serialType;     /* 0 == empty */
   union {
     EthSerial ethSerial; /* hostSerial == SERIAL_MAC */
     IpSerial  ipSerial;  /* hostSerial == SERIAL_IPV4/SERIAL_IPV6 */
   } value;
 } HostSerial;
 
+typedef u_int32_t HostSerialIndex;
+
+typedef struct {
+  time_t dump_date;
+  HostSerialIndex idx;
+} HostSerialIndexDump;
+
+typedef struct {
+  time_t dump_date;
+  HostSerial serial;
+} HostSerialDump;
+
+/*
+  extern int emptySerial(HostSerialIndex *a);
+  extern int cmpSerial(HostSerialIndex *a, HostSerialIndex *b);
+  extern int copySerial(HostSerialIndex *a, HostSerialIndex *b);
+*/
+#define emptySerial(a)    (*a == UNKNOWN_SERIAL_INDEX)
+#define cmpSerial(a, b)   (*a == *b)
+#define copySerial(a, b)  { *a = *b; }
+#define setEmptySerial(a) { *a = UNKNOWN_SERIAL_INDEX; }
+
 #ifdef WIN32
 #define pid_t unsigned int
 #ifndef RETSIGTYPE
@@ -238,11 +289,8 @@ typedef struct protocolsList {
 #define pthread_rwlock_wrlock    pthread_mutex_lock
 #define pthread_rwlock_unlock    pthread_mutex_unlock
 #define pthread_rwlock_destroy   pthread_mutex_destroy
-#ifdef SOLARIS
+/* Fix courtesy of Mark Gibbons <mgibbons@oemcomp.com> */
 #define pthread_rwlock_trywrlock pthread_mutex_trylock
-#else /* SOLARIS */
-#define pthread_rwlock_trywrlock pthread_mutex_trywrlock
-#endif /* SOLARIS */
 #endif
 #endif
 
@@ -321,15 +369,20 @@ typedef struct trafficCounter {
   u_char modified;
 } TrafficCounter;
 
+/* ******************************** */
+
+inline static void incrementTrafficCounter(TrafficCounter *ctr, Counter value) { if(value > 0) ctr->value += value, ctr->modified = 1; }
+inline static void resetTrafficCounter(TrafficCounter *ctr) { ctr->value = 0, ctr->modified = 0; }
+
 /* ************* Types Definition ********************* */
 
 typedef struct thptEntry {
   float trafficValue;
   /* ****** */
-  HostSerial topHostSentSerial, secondHostSentSerial, thirdHostSentSerial;
+  HostSerialIndex topHostSentSerial, secondHostSentSerial, thirdHostSentSerial;
   TrafficCounter topSentTraffic, secondSentTraffic, thirdSentTraffic;
   /* ****** */
-  HostSerial topHostRcvdSerial, secondHostRcvdSerial, thirdHostRcvdSerial;
+  HostSerialIndex topHostRcvdSerial, secondHostRcvdSerial, thirdHostRcvdSerial;
   TrafficCounter topRcvdTraffic, secondRcvdTraffic, thirdRcvdTraffic;
 } ThptEntry;
 
@@ -365,7 +418,7 @@ typedef struct simpleProtoTrafficInfo {
 
 typedef struct usageCounter {
   TrafficCounter value;
-  HostSerial peersSerials[MAX_NUM_CONTACTED_PEERS]; /* host serial */
+  HostSerialIndex peersSerials[MAX_NUM_CONTACTED_PEERS]; /* host serial */
 } UsageCounter;
 
 /* *********************** */
@@ -451,7 +504,6 @@ typedef struct nonIPTraffic {
 
   /* Non IP */
   TrafficCounter   stpSent, stpRcvd; /* Spanning Tree */
-  TrafficCounter   dlcSent, dlcRcvd;
   TrafficCounter   arp_rarpSent, arp_rarpRcvd;
   TrafficCounter   arpReqPktsSent, arpReplyPktsSent, arpReplyPktsRcvd;
   TrafficCounter   netbiosSent, netbiosRcvd;
@@ -469,14 +521,36 @@ typedef struct trafficDistribution {
 /* *********************** */
 
 typedef struct portUsage {
-  u_short        port, clientUses, serverUses;
-  HostSerial     clientUsesLastPeer, serverUsesLastPeer;
+  u_short         port, clientUses, serverUses;
+  HostSerialIndex clientUsesLastPeer, serverUsesLastPeer;
   TrafficCounter clientTraffic, serverTraffic;
   struct portUsage *next;
 } PortUsage;
 
 /* *********************** */
 
+typedef struct hostTalker {
+  HostSerialIndex hostSerial;
+  float bps /* bytes/sec */;
+} HostTalker;
+
+/* *********************** */
+
+typedef struct hostTalkerSeries {
+  HostSerialIndex hostSerial;
+  float total_bps /* bytes/sec */;
+  float bps_series[60 /* 1 x minute */];
+} HostTalkerSeries;
+
+/* *********************** */
+
+typedef struct topTalkers {
+  time_t when;
+  HostTalker senders[MAX_NUM_TOP_TALKERS], receivers[MAX_NUM_TOP_TALKERS];
+} TopTalkers;
+
+/* *********************** */
+
 typedef struct virtualHostList {
   char *virtualHostName;
   TrafficCounter bytesSent, bytesRcvd; /* ... by the virtual host */
@@ -600,11 +674,15 @@ typedef struct networkDelay {
   u_int num_samples;
   double total_delay;
   u_int16_t peer_port;
-  HostSerial last_peer;
+  HostSerialIndex last_peer;
 } NetworkDelay;
 
 /* **************************** */
 
+typedef struct {
+  Counter bytesSent, bytesRcvd;
+} ProtoTraffic;
+
 #define hostIp4Address hostIpAddress.Ip4Address
 #define hostIp6Address hostIpAddress.Ip6Address
 
@@ -612,10 +690,11 @@ typedef struct networkDelay {
 typedef struct hostTraffic {
   u_int8_t         to_be_deleted; /* 1 = the host will be deleted in the next purge loop */
   u_short          magic;
-  u_short          l2Family;    /* 0 = Ethernet */
+  u_int8_t         l2Host;    /* 1 = Ethernet, 0 = IP and above */
   u_int            hostTrafficBucket; /* Index in the **hash_hostTraffic list */
   u_short          refCount;         /* Reference counter */
   HostSerial       hostSerial;
+  HostSerialIndex  serialHostIndex;  /* Stored in myGlobals.serialFile and valid until ntop restart */
   HostAddr         hostIpAddress;
   u_int16_t        vlanId;          /* VLAN Id (-1 if not set) */
   u_int16_t        ifId;            /* Interface Id [e.g. for NetFlow] (-1 if not set) */
@@ -638,26 +717,25 @@ typedef struct hostTraffic {
   TrafficCounter   greSent, greRcvd, grePktSent, grePktRcvd, lastGrePktSent, lastGrePktRcvd;
   TrafficCounter   ipsecSent, ipsecRcvd, ipsecPktSent, ipsecPktRcvd, lastIpsecPktSent, lastIpsecPktRcvd;
 
+  /* Sketches */
+  CM_type          *sent_to_matrix, *recv_from_matrix;
+
   NonIPTraffic     *nonIPTraffic;
   NonIpProtoTrafficInfo *nonIpProtoTrafficInfos; /* Info about further non IP protos */
 
   fd_set           flags;
-  TrafficCounter   pktSent, pktRcvd, pktSentSession, pktRcvdSession,
-    pktDuplicatedAckSent, pktDuplicatedAckRcvd;
-  TrafficCounter   lastThptPktSent, lastThptPktRcvd;
-  TrafficCounter   pktBroadcastSent, bytesBroadcastSent;
-  TrafficCounter   pktMulticastSent, bytesMulticastSent,
-    pktMulticastRcvd, bytesMulticastRcvd;
-  TrafficCounter   lastBytesSent, lastHourBytesSent,
-    bytesSent, bytesSentLoc, bytesSentRem, bytesSentSession;
-  TrafficCounter   lastBytesRcvd, lastHourBytesRcvd, bytesRcvd,
-    bytesRcvdLoc, bytesRcvdFromRem, bytesRcvdSession;
-  float            actualRcvdThpt, lastHourRcvdThpt, averageRcvdThpt, peakRcvdThpt,
-    actualSentThpt, lastHourSentThpt, averageSentThpt, peakSentThpt,
-    actualTThpt, averageTThpt, peakTThpt;
-  float            actualRcvdPktThpt, averageRcvdPktThpt, peakRcvdPktThpt,
-    actualSentPktThpt, averageSentPktThpt, peakSentPktThpt,
-    actualTPktThpt, averageTPktThpt, peakTPktThpt;
+  TrafficCounter   pktsSent, pktsRcvd, pktsSentSession, pktsRcvdSession;
+  TrafficCounter   pktsDuplicatedAckSent, pktsDuplicatedAckRcvd;
+  TrafficCounter   pktsBroadcastSent, bytesBroadcastSent;
+  TrafficCounter   pktsMulticastSent, bytesMulticastSent;
+  TrafficCounter   pktsMulticastRcvd, bytesMulticastRcvd;
+  TrafficCounter   lastBytesSent, lastHourBytesSent;
+  TrafficCounter   bytesSent, bytesSentLoc, bytesSentRem, bytesSentSession;
+  TrafficCounter   lastBytesRcvd, lastHourBytesRcvd, bytesRcvd;
+  TrafficCounter   bytesRcvdLoc, bytesRcvdFromRem, bytesRcvdSession;
+  float            actualRcvdThpt, lastHourRcvdThpt, averageRcvdThpt, peakRcvdThpt;
+  float            actualSentThpt, lastHourSentThpt, averageSentThpt, peakSentThpt;
+  float            actualThpt, averageThpt /* REMOVE */, peakThpt;
   unsigned short   actBandwidthUsage, actBandwidthUsageS, actBandwidthUsageR;
   TrafficDistribution *trafficDistribution;
   u_int32_t        numHostSessions;
@@ -689,13 +767,15 @@ typedef struct hostTraffic {
   IcmpHostInfo       *icmpInfo;
 
   ShortProtoTrafficInfo **ipProtosList;        /* List of myGlobals.numIpProtosList entries */
-  ProtoTrafficInfo      **protoIPTrafficInfos; /* Info about IP traffic generated/rcvd by this host */
+  Counter                 totContactedSentPeers, totContactedRcvdPeers; /* # of different contacted peers */
+  struct hostTraffic *next;              /* pointer to the next element */
 
-  Counter          totContactedSentPeers, totContactedRcvdPeers; /* # of different contacted peers */
   UsageCounter     contactedSentPeers;   /* peers that talked with this host */
   UsageCounter     contactedRcvdPeers;   /* peers that talked with this host */
-  UsageCounter     contactedRouters;     /* routers contacted by this host */
-  struct hostTraffic *next;              /* pointer to the next element */
+
+  struct {
+    ProtoTraffic *traffic;
+  } l7;
 } HostTraffic;
 
 /* **************************** */
@@ -745,7 +825,7 @@ typedef struct portCounter {
 /* IP Session Information */
 typedef struct ipSession {
   u_short magic;
-  u_int8_t proto;                   /* IPPROTO_TCP / IPPROTO_UDP                  */
+  u_int8_t proto;                   /* IPPROTO_TCP / IPPROTO_UDP...               */
   u_char isP2P;                     /* Set to 1 if this is a P2P session          */
   u_int8_t knownProtocolIdx;        /* Mark this as a special protocol session    */
   HostTraffic* initiator;           /* initiator address                          */
@@ -755,15 +835,13 @@ typedef struct ipSession {
   HostAddr remotePeerRealIp;        /* Real IP address (if masqueraded and known) */
   char *virtualPeerName;            /* Name of a virtual host (e.g. HTTP virtual host) */
   u_short dport;                    /* remote peer address (port)               */
-  time_t firstSeen;                 /* time when the session has been initiated */
-  time_t lastSeen;                  /* time when the session has been closed    */
+  time_t firstSeen;            /* time when the session has been initiated */
+  time_t lastSeen;             /* time when the session has been closed    */
   u_long pktSent, pktRcvd;
   TrafficCounter bytesSent;         /* # bytes sent (initiator -> peer) [IP]    */
   TrafficCounter bytesRcvd;         /* # bytes rcvd (peer -> initiator)[IP]     */
   TrafficCounter bytesProtoSent;    /* # bytes sent (Protocol [e.g. HTTP])      */
   TrafficCounter bytesProtoRcvd;    /* # bytes rcvd (Protocol [e.g. HTTP])      */
-  TrafficCounter bytesFragmentedSent;     /* IP Fragments                       */
-  TrafficCounter bytesFragmentedRcvd;     /* IP Fragments                       */
   u_int minWindow, maxWindow;       /* TCP window size                          */
   struct timeval synTime, synAckTime, ackTime; /* Used to calcolate nw delay */
   struct timeval clientNwDelay, serverNwDelay; /* Network Delay/Latency         */
@@ -771,8 +849,6 @@ typedef struct ipSession {
   u_short numFinAcked;              /* # ACK pkts rcvd                          */
   u_int32_t lastAckIdI2R;           /* ID of the last ACK rcvd                  */
   u_int32_t lastAckIdR2I;           /* ID of the last ACK rcvd                  */
-  u_short numDuplicatedAckI2R;      /* # duplicated ACKs                        */
-  u_short numDuplicatedAckR2I;      /* # duplicated ACKs                        */
   TrafficCounter bytesRetranI2R;    /* # bytes retransmitted (due to duplicated ACKs) */
   TrafficCounter bytesRetranR2I;    /* # bytes retransmitted (due to duplicated ACKs) */
   u_int32_t finId[MAX_NUM_FIN];     /* ACK ids we're waiting for                */
@@ -785,8 +861,13 @@ typedef struct ipSession {
   u_char  passiveFtpSession;        /* checked if this is a passive FTP session */
   u_char  voipSession;              /* checked if this is a VoIP session */
   char *session_info;               /* Info about this session (if any) */
-  char *guessed_protocol;
   struct ipSession *next;
+  struct {
+    u_int8_t proto_guessed;
+    u_int16_t major_proto;
+    struct ipoque_flow_struct *flow;
+    struct ipoque_id_struct *src, *dst;
+  } l7;
 } IPSession;
 
 /* ************************************* */
@@ -1106,7 +1187,6 @@ typedef struct netFlowGlobals {
 #ifdef HAVE_SCTP
   int netFlowInSctpSocket;
 #endif
-  u_char netFlowAssumeFTP, enableSessionHandling;
   u_short netFlowInPort;
   struct in_addr netFlowIfAddress, netFlowIfMask;
   char *netFlowWhiteList, *netFlowBlackList;
@@ -1149,33 +1229,6 @@ typedef struct netFlowGlobals {
 
 /* *********************************** */
 
-typedef struct cpacket_counter {
-  char *name;
-  u_long bytes, packets;
-  struct cpacket_counter *next;
-} cPacketCounter;
-
-typedef struct cpacket_globals {
-  u_char cpacketDebug;
-
-  /* Flow reception */
-  int cpacketInSocket, cpacketDeviceId;
-  u_short cpacketInPort;
-  u_long numPktsRcvd;
-
-  /* Stats */
-  ProbeInfo deviceList[MAX_NUM_PROBES];
-  u_int32_t statsProcessed;
-  
-  /* Counters */
-  cPacketCounter *counter_list_head, *last_head;
-
-  pthread_t cpacketThread;
-  int threadActive;
-} cPacketGlobals;
-
-/* *********************************** */
-
 #define MAX_NUM_SFLOW_INTERFACES      4096
 
 typedef struct ifCounters {
@@ -1243,6 +1296,15 @@ typedef struct sFlowGlobals {
 
 /* *********************************** */
 
+typedef struct {
+  u_int  hostsno;        /* # of valid entries in the following table */
+  u_int  actualHashSize;
+  struct hostTraffic **hash_hostTraffic;
+  u_short hashListMaxLookups;
+} HostsHashInfo;
+
+/* *********************************** */
+
 typedef struct ntopInterface {
   char *name;                    /* Interface name (e.g. eth0) */
   char *uniqueIfName;            /* Unique interface name used to save data on disk */
@@ -1322,13 +1384,10 @@ typedef struct ntopInterface {
   TrafficCounter otherIpBytes;
 
   TrafficCounter icmpBytes;
-  TrafficCounter dlcBytes;
-  TrafficCounter ipxBytes;
   TrafficCounter stpBytes;        /* Spanning Tree */
   TrafficCounter ipsecBytes;
   TrafficCounter netbiosBytes;
   TrafficCounter arpRarpBytes;
-  TrafficCounter egpBytes;
   TrafficCounter greBytes;
   TrafficCounter ipv6Bytes;
   TrafficCounter icmp6Bytes;
@@ -1366,40 +1425,34 @@ typedef struct ntopInterface {
   float  packetThroughput;
 
   unsigned long numThptSamples;
-  ThptEntry last60MinutesThpt[60], last24HoursThpt[24];
-  float last30daysThpt[30];
-  u_short last60MinutesThptIdx, last24HoursThptIdx, last30daysThptIdx;
+  TopTalkers last60MinTopTalkers[60], last24HoursTopTalkers[24];
 
   SimpleProtoTrafficInfo tcpGlobalTrafficStats, udpGlobalTrafficStats, icmpGlobalTrafficStats;
-  SimpleProtoTrafficInfo *ipProtoStats;
   SecurityDeviceProbes securityPkts;
 
   TrafficCounter numEstablishedTCPConnections; /* = # really established connections */
 
   pthread_t pcapDispatchThreadId;
 
-  u_int  hostsno;        /* # of valid entries in the following table */
-  u_int  actualHashSize;
-  struct hostTraffic **hash_hostTraffic;
-
-  u_short hashListMaxLookups;
+  HostsHashInfo hosts;
 
   /* ************************** */
 
-  IpFragment *fragmentList;
-  IPSession **tcpSession;
-  u_short numTcpSessions, maxNumTcpSessions;
+  IPSession **sessions;
+  u_int numSessions, maxNumSessions;
 
   /* ************************** */
 
   NetFlowGlobals *netflowGlobals;  /* NetFlow */
   SflowGlobals   *sflowGlobals;    /* sFlow */
-  cPacketGlobals *cpacketGlobals;  /* cPacket */
 
   /* ********************* */
   
-  /* The variable below is used to collect stats concerning interface subnets */
-  HostTraffic networkHost[MAX_NUM_INTERFACE_NETWORKS];
+  struct {
+    PthreadMutex l7Mutex;
+    struct ipoque_detection_module_struct *l7handler;
+    Counter *protoTraffic;
+  } l7;
 } NtopInterface;
 
 /* *********************************** */
@@ -1773,11 +1826,9 @@ typedef struct _userPref {
   int  maxNumLines;              /* -e | --max-table-rows */
   bool trackOnlyLocalHosts;      /* -g | --track-local-hosts */
   char *devices;                 /* -i | --interface */
-  bool enableOtherPacketDump;    /* -j | --create-other-packets */
   char *pcapLog;                 /* -l | --pcap-log */
   char *localAddresses;          /* -m | --local-subnets */
   DnsResolutionMode numericFlag; /* -n | --numeric-ip-addresses */
-  bool dontTrustMACaddr;         /* -o | --no-mac */
   char *protoSpecs;              /* -p | --protocols */
   bool enableSuspiciousPacketDump;  /* -q | --create-suspicious-packets */
   int  refreshRate;              /* -r | --refresh-time */
@@ -1820,6 +1871,8 @@ typedef struct _userPref {
   bool disableStopcap;           /* --disable-stopcap '142' */
 
   bool disableMutexExtraInfo;    /* --disable-mutexextrainfo '145' */
+  bool disablenDPI;              /* --disable-ndpi '146' */
+  bool disablePython;            /* --disable-python '147' */
   bool skipVersionCheck;         /* --skip-version-check '150' */
   char *knownSubnets;            /* --known-subnets '151' */
 } UserPref;
@@ -1852,14 +1905,11 @@ typedef struct ntopGlobals {
 #ifndef WIN32
   char *effectiveUserName;
   int  userId, groupId;         /* 'u' */
-#else
-  u_char useU3;                 /* --U3 */
 #endif
   char *dbPath;                 /* 'P' */
   char *spoolPath;              /* 'Q' */
   struct fileList *pcap_file_list; /* --pcap-file-list */
   /* Other flags (these could set via command line options one day) */
-  bool enableFragmentHandling;
 
   HostsDisplayPolicy hostsDisplayPolicy;
   LocalityDisplayPolicy localityDisplayPolicy;
@@ -1875,7 +1925,7 @@ typedef struct ntopGlobals {
   NtopInterface *device;   /* pointer to the network interfaces table */
 
   /* Database */
-  GDBM_FILE pwFile, prefsFile, macPrefixFile, fingerprintFile;
+  GDBM_FILE pwFile, prefsFile, macPrefixFile, fingerprintFile, serialFile, topTalkersFile, resolverCacheFile;
 
   /* the table of broadcast entries */
   HostTraffic *broadcastEntry;
@@ -1883,6 +1933,9 @@ typedef struct ntopGlobals {
   /* the table of other hosts entries */
   HostTraffic *otherHostEntry;
 
+  /* Host serial */
+  u_int32_t hostSerialCounter;
+
   /* Administrative */
   char *shortDomainName;
 #if defined(MAX_NUM_BAD_IP_ADDRESSES) && (MAX_NUM_BAD_IP_ADDRESSES > 0)
@@ -1911,6 +1964,9 @@ typedef struct ntopGlobals {
   PthreadMutex hostsHashMutex[CONST_HASH_INITIAL_SIZE];
   volatile u_short hostsHashMutexNumLocks[CONST_HASH_INITIAL_SIZE];
 
+  /* Host Serial */
+  PthreadMutex serialLockMutex;
+
   /*
    * SIH - Scan Idle Hosts - optional
    */
@@ -1935,7 +1991,7 @@ typedef struct ntopGlobals {
    * Control mutexes
    */
   PthreadMutex gdbmMutex, portsMutex;
-  PthreadMutex tcpSessionsMutex[NUM_SESSION_MUTEXES];
+  PthreadMutex sessionsMutex[NUM_SESSION_MUTEXES];
   PthreadMutex purgePortsMutex;
   PthreadMutex securityItemsMutex;
 #ifdef FORPRENPTL
@@ -2055,25 +2111,6 @@ typedef struct ntopGlobals {
                 numHandledSSIRequests;
   u_short webServerRequestQueueLength;
 
-  /* Memory cache */
-  HostTraffic *hostsCache[MAX_HOSTS_CACHE_LEN];
-  u_short      hostsCacheLen, hostsCacheLenMax;
-  int          hostsCacheReused;
-
-#ifdef PARM_USE_SESSIONS_CACHE
-  IPSession   *sessionsCache[MAX_SESSIONS_CACHE_LEN];
-  u_short      sessionsCacheLen, sessionsCacheLenMax;
-  int          sessionsCacheReused;
-#endif
-
-  /* Peer2Peer Protocol Indexes */
-  u_short SshIdx, HttpIdx, FTPIdx, EdonkeyIdx, BitTorrentIdx, 
-    VoipIdx, FacebookIdx, TwitterIdx, YouTubeIdx, SkypeIdx;
-
-  /* Hash table collisions - counted during load */
-  int ipxsapHashLoadCollisions;
-  /* Hash table sizes - counted during load */
-  int ipxsapHashLoadSize;
   /* Hash table collisions - counted during use */
   int hashCollisionsLookup;
 
@@ -2125,4 +2162,12 @@ typedef struct ntopGlobals {
 
   /* RRD */
   time_t rrdTime;
+  
+  /* Message display */
+  u_char lowMemoryMsgShown;
+
+  struct {
+    u_short numSupportedProtocols;
+    u_int16_t flow_struct_size, proto_size;
+  } l7;
 } NtopGlobals;
