@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -37,124 +37,89 @@ struct bar_elements {
 
 /* ******************************************************************* */
 
-static void send_graph_header(u_char is_pie) {
-  sendString("<HTML>\n"
-	     "<HEAD>\n"
-	     "<META HTTP-EQUIV=REFRESH CONTENT=30>\n"
-	     "<META HTTP-EQUIV=Pragma CONTENT=no-cache>\n"
-	     "<META HTTP-EQUIV=Cache-Control CONTENT=no-cache>\n"
-	     "<script type=\"text/javascript\" src=\"/MochiKit/MochiKit.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/excanvas.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/Base.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/Layout.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/Canvas.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/SweetCanvas.js\"></script>\n"
-	     "<script type=\"text/javascript\" src=\"/PlotKit/EasyPlot.js\"></script>\n"
-	     "<style type=\"text/css\">\n"
-	     "body {\n"
-	     "font-family: \"Lucida Grande\", \"Tahoma\", \"Verdana\", \"Sans\", \"sans-serif\";\n"
-	     "font-size: 12px;\n"
-	     "}\n"
-	     "</style>\n"
-	     "<script type=\"text/javascript\">\n"
-	     "//<![CDATA[\n"
-	     "function drawchart() {\n"
-	     "   var hasCanvas = CanvasRenderer.isSupported();\n"
-	     "\n"
-	     "   var opts = {\n"
-	     );
+static void send_graph_header(char *name) {
+  sendString("<div id=\"");
+  sendString(name);
+  sendString("\" style=\"width: 350px; height: 320px; margin: 0 auto\"></div>\n");
+  sendString("\n\n<script type=\"text/javascript\">\n");
+  sendString("var data = [\n");
+}
+
+/**********************************************************/
+
+static void send_graph_footer(char *name) {
+  sendString("  $.jqplot('");
+  sendString(name);
+  sendString("', [data], \n");
+  sendString("			     { \n");
+  sendString("			     seriesDefaults: {\n");
+  sendString("			       renderer: jQuery.jqplot.PieRenderer, \n");
+  sendString("				   rendererOptions: {\n");
+  sendString("				 showDataLabels: true\n");
+  sendString("				     }\n");
+  sendString("			       }, \n");
+
+  /* Remove border around graphs */
+  sendString(" grid: { \n");
+  sendString("  gridLineColor: 'transparent', \n");
+  sendString("      background: 'transparent', \n");
+  sendString("      borderColor: 'transparent', \n");
+  sendString("      borderWidth: '1', \n");
+  sendString("      shadow: false \n");
+  sendString("      }, \n");
+
+  sendString("				 legend: { show:true, location: 'e' }\n");
+  sendString("			     }\n");
+  sendString("			     );\n");
+  sendString("  </script>\n\n");
+}
+
+/**********************************************************/
+
+static void build_pie(char *title, int num, float *p, char **lbl) {
+  int i, j, elems = 0;
+  char buf[256], name[256];
+  float tot = 0;
+  static u_int graph_id = 0;
+
+  if(num == 0) return;
+
+  for(j=0, i=0; title[i] != '\0'; i++) {
+    if(title[i] != ' ')
+      name[j++] = title[i];
+
+    if(j >= (sizeof(buf)-1))
+      break;
+  }
+
+  name[j] = '\0';
+
+  j = strlen(name);
+  snprintf(&name[j], sizeof(name)-j-1, "_%u", graph_id++);
   
-  if(is_pie) sendString("   \"pieRadius\": 0.42,	  \n");
-  
-  sendString("    \"colorScheme\": PlotKit.Base.palette(PlotKit.Base.baseColors()[0]),\n");
-  if(is_pie) sendString("   \"backgroundColor\": PlotKit.Base.baseColors()[0].lighterColorWithLevel(1),\n" );
-  sendString("   \"xTicks\": [");
-}
- 
-/**********************************************************/
+  send_graph_header(name);
 
-static void send_graph_middle(void) {
-sendString("]\n"
-	     "   };\n"
-	     "\n"
-	     "   var data1 = [");
-
-}
-
-/**********************************************************/
-
-static void send_graph_footer(char *the_type, u_int width, u_int height) {
-  char buf[256];
-
-  sendString("];\n"
-	     "   \n"
-	     "   if (hasCanvas) {\n"
-	     "       var pie = new EasyPlot(\"");
-
-sendString(the_type);
-sendString("\", opts, $(\'canvas");
-sendString(the_type);
-sendString("\'), [data1]);\n"
-	     "   }\n"
-	     "}\n"
-	     "\n"
-	     "addLoadEvent(drawchart);\n"
-	     "//]]>\n"
-	     "</script>\n"
-	     "</head>\n"
-	     "<body>\n"
-	     "<div id=\"canvas");
-sendString(the_type);
-
-
- snprintf(buf, sizeof(buf), 
-	  "\" width=\"%d\" height=\"%d\"></div>\n"
-	  "</body>\n"
-	  "</html>\n", 
-	  width, height);
-
- sendString(buf);
-}
-
-/**********************************************************/
-
-static void build_chart(u_char is_pie, char *the_type, int num, float *p, 
-			char **lbl, u_int width, u_int height) {
-  int i, num_printed;
-  char buf[64];
-
-  send_graph_header(is_pie);
-
-  for(i=0, num_printed=0; i<num; i++) {
+  for(i=0; i<num; i++) {
     if(p[i] > 0) {
-      snprintf(buf, sizeof(buf), "%c\n\t{v:%d, label:\"%s\"}", (num_printed == 0) ? ' ' : ',', i, lbl[i]);
+      if(elems > 0)
+	sendString(",\n");
+
+      snprintf(buf, sizeof(buf), "\t\t\t['%s',   %.1f]",
+	       lbl[i],
+	       p[i]
+	       //(i == (num-1)) ? (100-tot) : p[i]
+	       );
+      tot += p[i];
       sendString(buf);
-      num_printed++;
+      elems++;
     }
   }
 
-  send_graph_middle();
-
-  for(i=0, num_printed=0; i<num; i++) {
-    if(p[i] > 0) {
-      snprintf(buf, sizeof(buf), "%c [%d, %.1f]", (num_printed == 0) ? ' ' : ',', i, p[i]);
-      sendString(buf);
-      num_printed++;
-    }
-  }
-
-  if((num_printed == 1) && (p[0] == 100)) {
-	  /* Workaround for an Internet Explorer bug */
-      sendString(", [1, 0.01]");
-  }
-
-  send_graph_footer(the_type, width, height);
+  sendString("];\n");
+  sendString("\n");
+  send_graph_footer(name);
 }
 
-#define build_pie(a, b, c)  build_chart(1, "pie", a, b, c, 350, 200)
-#define build_line(a, b, c) build_chart(0, "line", a, b, c, 350, 200)
-#define build_bar(a, b, c)  build_chart(0, "bar", a, b, c, 600, 200)
-
 /* ******************************************************************* */
 
 void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
@@ -171,17 +136,16 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
       theHost->udpSentLoc.value+theHost->udpSentRem.value+
       theHost->icmpSent.value+theHost->ipv6BytesSent.value+
       theHost->greSent.value+theHost->ipsecSent.value;
-    
+
     if(theHost->nonIPTraffic != NULL)
       totTraffic.value += theHost->nonIPTraffic->stpSent.value+
-	theHost->nonIPTraffic->dlcSent.value+
 	theHost->nonIPTraffic->arp_rarpSent.value+
 	theHost->nonIPTraffic->netbiosSent.value+
 	theHost->nonIPTraffic->otherSent.value;
 
     idx = 0;
     while(protoList != NULL) {
-      if(theHost->ipProtosList[idx] != NULL) 
+      if(theHost->ipProtosList[idx] != NULL)
 	totTraffic.value += theHost->ipProtosList[idx]->sent.value;
       idx++, protoList = protoList->next;
     }
@@ -193,14 +157,13 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
 
     if(theHost->nonIPTraffic != NULL)
       totTraffic.value += theHost->nonIPTraffic->stpRcvd.value
-	+theHost->nonIPTraffic->dlcRcvd.value
 	+theHost->nonIPTraffic->arp_rarpRcvd.value
 	+theHost->nonIPTraffic->netbiosRcvd.value
 	+theHost->nonIPTraffic->otherRcvd.value;
-    
+
     idx = 0;
     while(protoList != NULL) {
-      if(theHost->ipProtosList[idx] != NULL) 
+      if(theHost->ipProtosList[idx] != NULL)
 	totTraffic.value += theHost->ipProtosList[idx]->rcvd.value;
       idx++, protoList = protoList->next;
     }
@@ -246,11 +209,6 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
 	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "STP";
 	}
 
-	if(theHost->nonIPTraffic->dlcSent.value > 0) {
-	  p[num] = (float)((100*theHost->nonIPTraffic->dlcSent.value)/totTraffic.value);
-	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "DLC";
-	}
-
 	if(theHost->nonIPTraffic->arp_rarpSent.value > 0) {
 	  p[num] = (float)((100*theHost->nonIPTraffic->arp_rarpSent.value)/totTraffic.value);
 	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "(R)ARP";
@@ -281,7 +239,7 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
 
       if(theHost->icmpRcvd.value > 0) {
 	p[num] = (float)((100*theHost->icmpRcvd.value)/totTraffic.value);
-	if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "ICMP";     
+	if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "ICMP";
       }
 
       if(theHost->ipv6BytesRcvd.value > 0) {
@@ -293,23 +251,18 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
 	p[num] = (float)((100*theHost->greRcvd.value)/totTraffic.value);
 	if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "GRE";
       }
-      
+
       if(theHost->ipsecRcvd.value > 0) {
 	p[num] = (float)((100*theHost->ipsecRcvd.value)/totTraffic.value);
 	if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "IPSEC";
       }
-      
+
       if(theHost->nonIPTraffic != NULL) {
 	if(theHost->nonIPTraffic->stpRcvd.value > 0) {
 	  p[num] = (float)((100*theHost->nonIPTraffic->stpRcvd.value)/totTraffic.value);
 	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "STP";
 	}
 
-	if(theHost->nonIPTraffic->dlcRcvd.value > 0) {
-	  p[num] = (float)((100*theHost->nonIPTraffic->dlcRcvd.value)/totTraffic.value);
-	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "DLC";
-	}
-
 	if(theHost->nonIPTraffic->arp_rarpRcvd.value > 0) {
 	  p[num] = (float)((100*theHost->nonIPTraffic->arp_rarpRcvd.value)/totTraffic.value);
 	  if(p[num] > MIN_SLICE_PERCENTAGE) lbl[num++] = "(R)ARP";
@@ -353,10 +306,46 @@ void hostTrafficDistrib(HostTraffic *theHost, short dataSent) {
 
     if(num == 1) p[0] = 100; /* just to be safe */
 
-    build_pie(num, p, lbl);
+    build_pie("Traffic Distribution", num, p, lbl);
   }
 }
 
+/* ******************************************************************* */
+
+void drawDeviceServiceDistribution(void) {
+  float p[20];
+  char	*lbl[] = { "", "", "", "", "", "", "", "", "", "",
+		   "", "", "", "", "", "", "", "", "", "" };
+  int num = 0;
+  Counter totTraffic = 0;
+  int i;
+
+  for(i=0; i<myGlobals.l7.numSupportedProtocols; i++)
+    totTraffic += myGlobals.device[myGlobals.actualReportDeviceId].l7.protoTraffic[i];
+
+  for(i=0; i<myGlobals.l7.numSupportedProtocols; i++) {
+    float val;
+
+    val = (float)myGlobals.device[myGlobals.actualReportDeviceId].l7.protoTraffic[i];
+    if(val > 0) {
+      val = (float)((val * 100) / totTraffic);
+      p[num] = val, lbl[num] = getProtoName(0, i);
+      num++;
+
+      if(num == 20) break;
+    }
+  }
+
+  if(num == 0) {
+    traceEvent(CONST_TRACE_WARNING, "Graph failure (1)");
+    return; /* TODO: this has to be handled better */
+  }
+
+  if(num == 1) p[0] = 100; /* just to be safe */
+
+  build_pie("Service Distribution", num, p, lbl);
+}
+
 /* ************************ */
 
 void hostFragmentDistrib(HostTraffic *theHost, short dataSent) {
@@ -410,7 +399,8 @@ void hostFragmentDistrib(HostTraffic *theHost, short dataSent) {
     }
 
     if(num == 1) p[0] = 100; /* just to be safe */
-    build_pie(num, p, lbl);
+
+    build_pie("Fragment Distribution", num, p, lbl);
   }
 }
 
@@ -435,7 +425,7 @@ void hostTimeTrafficDistribution(HostTraffic *theHost, short dataSent) {
       traf.value = 0;
 
     if(traf.value > 0) {
-      p[num] = traf.value;
+      p[num] = (float)traf.value;
       switch(i) {
       case 0:
 	lbl[num++] = "12-1AM";
@@ -517,9 +507,9 @@ void hostTimeTrafficDistribution(HostTraffic *theHost, short dataSent) {
     traceEvent(CONST_TRACE_WARNING, "Graph failure (2)");
     return; /* TODO: this has to be handled better */
   }
-  
+
   if(num == 1) p[0] = 100; /* just to be safe */
-  build_pie(num, p, lbl);
+  build_pie("Time Traffic Distribution", num, p, lbl);
 }
 
 /* ************************ */
@@ -545,7 +535,7 @@ void hostTotalFragmentDistrib(HostTraffic *theHost, short dataSent) {
     p[num] = (float)((100*totFragmentedTraffic.value)/totTraffic.value);
     lbl[num++] = "Frag";
 
-    p[num] = 100-((float)(100*totFragmentedTraffic.value)/totTraffic.value);
+    p[num] = (float)(100-((float)(100*totFragmentedTraffic.value)/totTraffic.value));
     if(p[num] > 0) { lbl[num++] = "Non Frag"; }
 
     if(num == 0) {
@@ -554,76 +544,37 @@ void hostTotalFragmentDistrib(HostTraffic *theHost, short dataSent) {
     }
 
     if(num == 1) p[0] = 100; /* just to be safe */
-    build_pie(num, p, lbl);
+    build_pie("Fragmented Traffic", num, p, lbl);
   }
 }
 
-/* ************************ */
+/* ********************************** */
 
 void hostIPTrafficDistrib(HostTraffic *theHost, short dataSent) {
-  float p[MAX_NUM_PROTOS];
-  char	*lbl[] = { "", "", "", "", "", "", "", "", "",
-		   "", "", "", "", "", "", "", "", "", "" };
+  float p[MAX_NUM_PROTOS] = { 0 };
+  char	*lbl[MAX_NUM_PROTOS] = { "" };
   int i, num=0;
-  TrafficCounter traffic, totalIPTraffic, diffTraffic;
+  Counter sent, rcvd, traffic;
 
-  if(theHost->protoIPTrafficInfos == NULL) {
-    traceEvent(CONST_TRACE_WARNING, "Graph failure (5)");
-    return;
-  }
+  sent = rcvd = 0;
+  for(i=0; i<myGlobals.l7.numSupportedProtocols; i++)
+    sent += theHost->l7.traffic[i].bytesSent, rcvd += theHost->l7.traffic[i].bytesRcvd;
 
-  totalIPTraffic.value = 0;
-  diffTraffic.value = 0;
+  traffic = dataSent ? sent : rcvd;
 
-  if(dataSent)
-    totalIPTraffic.value = theHost->ipv4BytesSent.value;
-  else
-    totalIPTraffic.value = theHost->ipv4BytesRcvd.value;
+  for(i=0; i<myGlobals.l7.numSupportedProtocols; i++) {
+    Counter val = dataSent ? theHost->l7.traffic[i].bytesSent : theHost->l7.traffic[i].bytesRcvd;
 
-  if(totalIPTraffic.value > 0) {
-    if(dataSent)
-      traffic.value = theHost->ipsecSent.value;    
-    else
-      traffic.value = theHost->ipsecRcvd.value;
- 
-    if(traffic.value > 0) {
-      p[num] = (float)((100*traffic.value)/totalIPTraffic.value);
-      diffTraffic.value += traffic.value;
-      lbl[num++] = "IPSEC";
+    if(val > 0) {
+      p[num] = (float)((100*val)/traffic);
+      lbl[num++] = getProtoName(0, i);
     }
 
-    for(i=0; i<myGlobals.numIpProtosToMonitor; i++) {
-      if(theHost->protoIPTrafficInfos[i]) {
-	if(dataSent)
-	  traffic.value = theHost->protoIPTrafficInfos[i]->sentLoc.value+theHost->protoIPTrafficInfos[i]->sentRem.value;
-	else
-	  traffic.value = theHost->protoIPTrafficInfos[i]->rcvdLoc.value+theHost->protoIPTrafficInfos[i]->rcvdFromRem.value;
-      } else
-	traffic.value = 0;
-      
-      if(traffic.value > 0) {
-	p[num] = (float)((100*traffic.value)/totalIPTraffic.value);
-	diffTraffic.value += traffic.value;
-	lbl[num++] = myGlobals.ipTrafficProtosNames[i];
-      }
-
-      if(num >= MAX_NUM_PROTOS) break; /* Too much stuff */
-    }
-  }
-
-  if(num == 0) {
-    p[num] = 1;
-    lbl[num++] = "Other";
-  } else {
-    if(diffTraffic.value < totalIPTraffic.value) {
-      diffTraffic.value = totalIPTraffic.value - diffTraffic.value;
-      p[num] = (float)((100*diffTraffic.value)/totalIPTraffic.value);
-      lbl[num++] = "Other";
-    }
+    if(num >= MAX_NUM_PROTOS) break; /* Too much stuff */
   }
 
   if(num == 1) p[0] = 100; /* just to be safe */
-  build_pie(num, p, lbl);
+  build_pie("Application Traffic", num, p, lbl);
 }
 
 /* ********************************** */
@@ -636,72 +587,72 @@ void pktSizeDistribPie(void) {
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo64.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo64.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 64";
+    lbl[num++] = "&lt;= 64";
   };
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo128.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo128.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 128";
+    lbl[num++] = "&lt;= 128";
   };
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo256.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo256.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 256";
+    lbl[num++] = "&lt;= 256";
   };
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo512.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo512.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 512";
+    lbl[num++] = "&lt;= 512";
   };
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo1024.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo1024.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 1024";
+    lbl[num++] = "&lt;= 1024";
   };
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo1518.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo1518.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 1518";
+    lbl[num++] = "&lt;= 1518";
   };
 
 #ifdef MAKE_WITH_JUMBO_FRAMES
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo2500.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo2500.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 2500";
+    lbl[num++] = "&lt;= 2500";
   };
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo6500.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo6500.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 6500";
+    lbl[num++] = "&lt;= 6500";
   };
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo9000.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.upTo9000.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "<= 9000";
+    lbl[num++] = "&lt;= 9000";
   };
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.above9000.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.above9000.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "> 9000";
+    lbl[num++] = "&gt; 9000";
   };
 #else
   if(myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.above1518.value > 0) {
     p[num] = (float)(100*myGlobals.device[myGlobals.actualReportDeviceId].rcvdPktStats.above1518.value)/
       (float)myGlobals.device[myGlobals.actualReportDeviceId].ethernetPkts.value;
-    lbl[num++] = "> 1518";
+    lbl[num++] = "&gt; 1518";
   };
 #endif
 
 
   if(num == 1) p[0] = 100; /* just to be safe */
-  
-  build_pie(num, p, lbl);
+
+  build_pie("Packet Size Distribution", num, p, lbl);
 }
 
 /* ********************************** */
@@ -760,7 +711,7 @@ void pktTTLDistribPie(void) {
   };
 
   if(num == 1) p[0] = 100; /* just to be safe */
-  build_pie(num, p, lbl);
+  build_pie("TTL Distribution", num, p, lbl);
 }
 
 /* ************************ */
@@ -790,7 +741,7 @@ void ipProtoDistribPie(void) {
 
   if(num == 1) p[0] = 100; /* just to be safe */
 
-  build_pie(num, p, lbl);
+  build_pie("IP Distribution", num, p, lbl);
 }
 
 /* ************************ */
@@ -816,20 +767,20 @@ void interfaceTrafficPie(void) {
 
   for(i=0; i<myGlobals.numDevices; i++) {
     if(myGlobals.device[i].activeDevice) {
-      p[myDevices]   = 100*(((float)p[i])/totPkts.value);
+      p[myDevices]   = (float)(100*(((float)p[i])/totPkts.value));
       lbl[myDevices] = myGlobals.device[i].humanFriendlyName;
       myDevices++;
     }
   }
 
-  if(myDevices == 1) 
+  if(myDevices == 1)
     p[0] = 100; /* just to be safe */
   else if(myDevices == 0) {
     traceEvent(CONST_TRACE_WARNING, "interfaceTrafficPie: no interfaces to draw");
     return;
   }
 
-  build_pie(myDevices, p, lbl);
+  build_pie("Devices Distribution", myDevices, p, lbl);
 }
 
 /* ************************ */
@@ -866,7 +817,7 @@ void pktCastDistribPie(void) {
     lbl[num++] = "Multicast";
   };
 
-  build_pie(num, p, lbl);
+  build_pie("Packets Distribution", num, p, lbl);
 }
 
 /* ************************ */
@@ -875,20 +826,21 @@ void drawTrafficPie(void) {
   TrafficCounter ip;
   float p[2];
   char	*lbl[] = { "IP", "Non IP" };
-  int num=0;
+  int num = 0;
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].ethernetBytes.value == 0) return;
 
   ip.value = myGlobals.device[myGlobals.actualReportDeviceId].ipv4Bytes.value;
 
-  p[0] = ip.value*100/myGlobals.device[myGlobals.actualReportDeviceId].ethernetBytes.value; num++;
+  p[0] = (float)(ip.value*100/myGlobals.device[myGlobals.actualReportDeviceId].ethernetBytes.value);
+  num++;
   p[1] = 100-p[0];
 
   if(p[1] > 0)
     num++;
 
   if(num == 1) p[0] = 100; /* just to be safe */
-  build_pie(num, p, lbl);
+  build_pie("IP vs non IP", num, p, lbl);
 }
 
 /* ************************ */
@@ -899,29 +851,27 @@ void drawGlobalProtoDistribution(void) {
   int idx = 0;
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].tcpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].tcpBytes.value; lbl[idx] = "TCP";  idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].tcpBytes.value; lbl[idx] = "TCP";  idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].udpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].udpBytes.value; lbl[idx] = "UDP"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].udpBytes.value; lbl[idx] = "UDP"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].icmpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].icmpBytes.value; lbl[idx] = "ICMP"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].icmpBytes.value; lbl[idx] = "ICMP"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].otherIpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].otherIpBytes.value; lbl[idx] = "Other IP"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].otherIpBytes.value; lbl[idx] = "Other IP"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].arpRarpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].arpRarpBytes.value; lbl[idx] = "(R)ARP"; idx++; }
-  if(myGlobals.device[myGlobals.actualReportDeviceId].dlcBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].dlcBytes.value; lbl[idx] = "DLC"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].arpRarpBytes.value; lbl[idx] = "(R)ARP"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].ipsecBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].ipsecBytes.value;lbl[idx] = "IPsec";  idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].ipsecBytes.value;lbl[idx] = "IPsec";  idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].netbiosBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].netbiosBytes.value; lbl[idx] = "NetBios"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].netbiosBytes.value; lbl[idx] = "NetBios"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].greBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].greBytes.value; lbl[idx] = "GRE"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].greBytes.value; lbl[idx] = "GRE"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].ipv6Bytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].ipv6Bytes.value; lbl[idx] = "IPv6"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].ipv6Bytes.value; lbl[idx] = "IPv6"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].stpBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].stpBytes.value; lbl[idx] = "STP"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].stpBytes.value; lbl[idx] = "STP"; idx++; }
   if(myGlobals.device[myGlobals.actualReportDeviceId].otherBytes.value > 0) {
-    p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].otherBytes.value; lbl[idx] = "Other"; idx++; }
+    p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].otherBytes.value; lbl[idx] = "Other"; idx++; }
 
   if(myGlobals.device[myGlobals.actualReportDeviceId].ipProtosList) {
     ProtocolsList *protoList = myGlobals.ipProtosList;
@@ -929,7 +879,7 @@ void drawGlobalProtoDistribution(void) {
 
     while(protoList != NULL) {
       if(myGlobals.device[myGlobals.actualReportDeviceId].ipProtosList[idx1].value > 0) {
-	p[idx] = myGlobals.device[myGlobals.actualReportDeviceId].ipProtosList[idx1].value;
+	p[idx] = (float)myGlobals.device[myGlobals.actualReportDeviceId].ipProtosList[idx1].value;
 	lbl[idx] = protoList->protocolName; idx++;
       }
 
@@ -940,13 +890,13 @@ void drawGlobalProtoDistribution(void) {
 
   {
     int i;
-    float the_max = 0.1;
+    float the_max = (float)0.1;
 
     for(i=0; i<idx; i++) the_max = max(the_max, p[i]);
     for(i=0; i<idx; i++) p[i]    = (p[i]*100)/the_max;
   }
 
-  build_bar(idx, p, lbl);
+  build_pie("Protocol Distribution", idx, p, lbl);
 }
 
 /* ************************ */
@@ -972,10 +922,7 @@ void drawGlobalIpProtoDistribution(void) {
     }
 
     for(i=0; i<myGlobals.numIpProtosToMonitor; i++) {
-      p[idx]  = (float)myGlobals.device[myGlobals.actualReportDeviceId].ipProtoStats[i].local.value
-	+myGlobals.device[myGlobals.actualReportDeviceId].ipProtoStats[i].remote.value;
-      p[idx] += (float)myGlobals.device[myGlobals.actualReportDeviceId].ipProtoStats[i].remote2local.value
-	+myGlobals.device[myGlobals.actualReportDeviceId].ipProtoStats[i].local2remote.value;
+      p[idx]  = (float)myGlobals.device[myGlobals.actualReportDeviceId].l7.protoTraffic[i];
       if((p[idx] > 0) && ((p[idx]*100/total) > 1 /* the proto is at least 1% */)) {
 	partialTotal += p[idx];
 	lbl[idx] = myGlobals.ipTrafficProtosNames[i];
@@ -988,18 +935,9 @@ void drawGlobalIpProtoDistribution(void) {
 
   if(total == 0) total = 1;
 
-  /*  Add a bar for the Other TCP/UDP based protocols
-      Courtesy of Robbert Kouprie <r.kouprie@dto.tudelft.nl>
-  */
-  if (total > partialTotal) {
-    lbl[idx] = "Other";
-    p[idx] = total - partialTotal;
-    idx++;
-  }
-
   for(i=0; i<idx; i++) p[i] = (p[i] * 100)/total;
 
-  build_pie(idx, p, lbl);
+  build_pie("IP Distribution", idx, p, lbl);
 }
 
 /* ******************************** */
@@ -1043,10 +981,174 @@ int drawHostsDistanceGraph(int checkOnly) {
     graphData[0]++;
   }
 
-  build_pie(30, graphData, lbls);
+  build_pie("Hosts Distance", 30, graphData, lbls);
 
   return(numPoints);
 }
 
 /* ************************ */
 
+void buildTalkersGraph(char **labels, HostTalkerSeries *talkers,
+		      int num_talkers, int num_datapoints) {
+  int i, j;
+
+  sendString("<HTML>\n"
+	     "<HEAD>\n"
+	     "<META HTTP-EQUIV=REFRESH CONTENT=120>\n"
+	     "<META HTTP-EQUIV=Pragma CONTENT=no-cache>\n"
+	     "<META HTTP-EQUIV=Cache-Control CONTENT=no-cache>\n");
+
+  sendJSLibraries(1);
+
+  sendString("</head>\n<body>\n");
+
+  sendString("<script type=\"text/javascript\">\n");
+  sendString("  $(document).ready(function() {\n");
+
+  sendString("    var ticks = [ ");
+
+  for(i=0; i<num_datapoints; i++) {
+    char buf[32];
+
+    snprintf(buf, sizeof(buf), "'%u.'", i+1);
+    if(i > 0) sendString(",");
+    sendString(buf);
+  }
+
+  sendString("];\n");
+  sendString("    var series = [ ");
+
+  for(i=0; i<num_talkers; i++) {
+    HostTraffic *el, tmpEl;
+    char buf[LEN_GENERAL_WORK_BUFFER], *c;
+
+    el = quickHostLink(talkers[i].hostSerial, myGlobals.actualReportDeviceId, &tmpEl);
+
+    if((el->hostResolvedNameType == FLAG_HOST_SYM_ADDR_TYPE_NONE)
+       || (el->hostResolvedName[0] == '\0') /* Safety check */
+       ) {
+      /* Let's search name in cache */
+      if(getHostNameFromCache(&el->hostIpAddress, el->hostResolvedName, sizeof(el->hostResolvedName)) != NULL) {
+	el->hostResolvedNameType = FLAG_HOST_SYM_ADDR_TYPE_NAME;
+      }
+    }
+
+    c = (el->hostResolvedName[0] != '\0') ? el->hostResolvedName : el->hostNumIpAddress;
+
+    if(el->hostResolvedName[0] != '\0') {
+      c = el->hostResolvedName, j = 0;
+
+      while(c[j] != '\0') {
+	if(c[j] == '.') {
+	  c[j] = '\0';
+	  break;
+	}
+
+	j++;
+      }
+    } else
+      c = el->hostNumIpAddress;
+
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "{ label: '%s'}, ", c);
+    sendString(buf);
+  }
+  sendString("];\n\n");
+
+  for(i=0; i<num_talkers; i++) {
+    char buf[64];
+
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "var data%d =  [ ", i);
+    sendString(buf);
+
+    for(j=0; j<num_datapoints; j++) {
+
+      if(j > 0) sendString(", ");
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%.0f", talkers[i].bps_series[j]);
+      sendString(buf);
+    }
+
+    sendString("];\n");
+  }
+
+  sendString("        $.jqplot('container', [ ");
+  for(i=0; i<num_talkers; i++) {
+    char buf[32];
+
+    if(i > 0) sendString(", ");
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "data%d", i);
+    sendString(buf);
+  }
+
+  sendString("   ], {\n");
+  sendString("   animate: !$.jqplot.use_excanvas,\n");
+  sendString("            stackSeries: true,\n");
+  //  sendString("            title : 'Top Host Talkers',\n");
+  sendString("            seriesDefaults:{\n");
+  sendString("                renderer:$.jqplot.BarRenderer,\n");
+  sendString("\n");
+  sendString("            },\n");
+
+  sendString(" grid: { \n");
+  //  sendString("  gridLineColor: 'transparent', \n");
+  //  sendString("      background: 'transparent', \n");
+  sendString("      borderColor: 'transparent', \n");
+  sendString("      borderWidth: '1', \n");
+  sendString("      shadow: false \n");
+  sendString("  }, \n");
+
+
+  sendString("            axes: {\n");
+  sendString("                xaxis: {\n");
+  sendString("                    renderer: $.jqplot.CategoryAxisRenderer,\n");
+  sendString("                    ticks: ticks\n");
+  sendString("                }\n");
+  sendString("            },\n");
+  sendString("            series: series,\n");
+  sendString("            axesDefaults: { showTickMarks: false },\n");
+  sendString("            legend: {\n");
+  sendString("                show: true,\n");
+  sendString("                location: 'e',\n");
+  sendString("                placement: 'outside'\n");
+  sendString("      }\n");
+
+  sendString("  });\n");
+  sendString("});\n");
+  sendString("</script>\n");
+
+  sendString("<div id=\"container\" align=left style=\"width: 800px; margin: 0 auto\"></div>\n");
+}
+
+/* ************************ */
+
+void drawThroughputMeter() {
+  char buf[256];
+
+  sendString("<div id=\"netspeed\" style=\"align: center; width: 180px; height: 120px; margin: 0 auto\"></div>\n");
+
+  sendString("<script type=\"text/javascript\">\n");
+  sendString("	  s1 = [\n");
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%.1f", myGlobals.device[myGlobals.actualReportDeviceId].actualThpt); sendString(buf);
+  sendString("];\n");
+      
+  sendString("	  $.jqplot('netspeed',[s1],{\n");
+  sendString("	    seriesDefaults: {\n");
+  sendString("	      renderer: $.jqplot.MeterGaugeRenderer,\n");
+  sendString("		  rendererOptions: {\n");
+  sendString("            showTickLabels: false,\n");
+  sendString("		min: 0,\n");
+  sendString("		    max: ");
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%.1f,\n",
+		myGlobals.device[myGlobals.actualReportDeviceId].peakThroughput); sendString(buf);
+  sendString("		    intervals:[");
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%.1f, %.1f, %.1f],\n", 
+		myGlobals.device[myGlobals.actualReportDeviceId].peakThroughput*0.33,
+		myGlobals.device[myGlobals.actualReportDeviceId].peakThroughput*0.75,
+		myGlobals.device[myGlobals.actualReportDeviceId].peakThroughput);
+  sendString(buf);
+  sendString("		    intervalColors:['#66cc66', '#E7E658', '#cc6666']\n");
+  sendString("		    }\n");
+  sendString("	      }\n");
+  sendString("	    });\n");
+
+  sendString("</script>\n");
+}
