@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * 			    http://www.ntop.org/
  *
@@ -34,23 +34,11 @@ static u_int sec_idle_with_no_sessions, sec_idle_with_sessions;
 /* ******************************* */
 
 u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
-	       short* useIPAddressForSearching, HostTraffic **el,
-	       int actualDeviceId) {
+	       HostTraffic **el, int actualDeviceId) {
   u_int idx = 0;
   *el = NULL;
 
-  if(myGlobals.runningPref.dontTrustMACaddr)  /* MAC addresses don't make sense here */
-    (*useIPAddressForSearching) = 1;
-
-  if((*useIPAddressForSearching) && (hostIpAddress == NULL)) {
-#if 0
-    traceEvent(CONST_TRACE_WARNING, "Index calculation problem (hostIpAddress=%x, ether_addr=%x)",
-	       hostIpAddress, ether_addr);
-#endif
-    return(FLAG_NO_PEER);
-  }
-
-  if(((*useIPAddressForSearching) == 1) || ((ether_addr == NULL) && (hostIpAddress != NULL))) {
+  if((ether_addr == NULL) && (hostIpAddress != NULL)) {
     if(myGlobals.runningPref.trackOnlyLocalHosts
        && (!isLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL))
        && (!_pseudoLocalAddress(hostIpAddress, NULL, NULL))) {
@@ -62,17 +50,15 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 	idx = (hostIpAddress->Ip4Address.s_addr & 0xffff)
 	  ^ ((hostIpAddress->Ip4Address.s_addr >> 15) & 0xffff);
       else if(hostIpAddress->hostFamily == AF_INET6)
-	idx = in6_hash(&hostIpAddress->Ip6Address);
+	idx = (u_int)in6_hash(&hostIpAddress->Ip6Address);
     }
 
-    (*useIPAddressForSearching) = 1;
   } else if(memcmp(ether_addr, myGlobals.broadcastEntry->ethAddress, LEN_ETHERNET_ADDRESS) == 0) {
     *el = myGlobals.broadcastEntry;
     return(BROADCAST_HOSTS_ENTRY);
-  } else if((hostIpAddress == NULL) 
+  } else if((hostIpAddress == NULL)
 	    || isPseudoLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL)) {
     memcpy(&idx, &ether_addr[LEN_ETHERNET_ADDRESS-sizeof(u_int)], sizeof(u_int));
-    (*useIPAddressForSearching) = 0;
   } else if(isBroadcastAddress(hostIpAddress, NULL, NULL)) {
     *el = myGlobals.broadcastEntry;
     return(BROADCAST_HOSTS_ENTRY);
@@ -87,17 +73,15 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 	if(hostIpAddress->hostFamily == AF_INET)
 	  idx = (hostIpAddress->Ip4Address.s_addr & 0xffff) ^ ((hostIpAddress->Ip4Address.s_addr >> 15) & 0xffff);
 	else if(hostIpAddress->hostFamily == AF_INET6)
-	  idx = in6_hash(&hostIpAddress->Ip6Address);
+	  idx = (u_int)in6_hash(&hostIpAddress->Ip6Address);
       }
     } else {
       idx = FLAG_NO_PEER;
       traceEvent(CONST_TRACE_WARNING, "Index calculation problem (1)");
     }
-
-    (*useIPAddressForSearching) = 1;
   }
 
-  idx = idx % myGlobals.device[actualDeviceId].actualHashSize;
+  idx = idx % myGlobals.device[actualDeviceId].hosts.actualHashSize;
 
   /* Skip reserved entries */
   if((idx == BROADCAST_HOSTS_ENTRY) || (idx == OTHER_HOSTS_ENTRY))
@@ -109,7 +93,7 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
 /* **************************************** */
 
 void freeHostInfo(HostTraffic *host, int actualDeviceId) {
-  u_int i, deleteAddressFromCache = 1;
+  u_int i;
 
   if(host == NULL) {
     traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(NULL)");
@@ -120,14 +104,17 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
   /* If this is one of the special ones, let's clear the other pointer to it
    * to prevent a free of freed memory error later.
    */
-  if(host == myGlobals.otherHostEntry) {
-    traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(otherHostEntry)");
-    return;
-  }
 
-  if(host == myGlobals.broadcastEntry) {
-    traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(broadcastEntry)");
-    return;
+  if(myGlobals.ntopRunState < FLAG_NTOPSTATE_SHUTDOWN) {
+    if(host == myGlobals.otherHostEntry) {
+      traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(otherHostEntry)");
+      return;
+    }
+    
+    if(host == myGlobals.broadcastEntry) {
+      traceEvent(CONST_TRACE_WARNING, "Attempting to call freeHostInfo(broadcastEntry)");
+      return;
+    }
   }
 
   if((host->magic != CONST_MAGIC_NUMBER) && (host->magic != CONST_UNMAGIC_NUMBER)) {
@@ -150,31 +137,9 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
 	     host->ethAddressString, host->hostNumIpAddress, host->hostResolvedName, host->hostTrafficBucket);
 #endif
 
-  if(deleteAddressFromCache) {
-    datum key_data;
-
-    if(host->hostIpAddress.hostFamily == AF_INET) {
-      key_data.dptr = (void*)&host->hostIpAddress.Ip4Address.s_addr;
-      key_data.dsize = 4;
-    } else if(host->hostIpAddress.hostFamily == AF_INET6) {
-      key_data.dptr = (void*)&host->hostIpAddress.Ip6Address.s6_addr;
-      key_data.dsize = 16;
-    }
-    else
-      key_data.dsize = 0;
-  }
-
   handlePluginHostCreationDeletion(host, (u_short)actualDeviceId, 0 /* host deletion */);
 
-  myGlobals.device[actualDeviceId].hostsno--;
-
-  if(host->protoIPTrafficInfos != NULL) {
-    for(i=0; i<myGlobals.numIpProtosToMonitor; i++)
-      if(host->protoIPTrafficInfos[i] != NULL)
-	free(host->protoIPTrafficInfos[i]);
-
-    free(host->protoIPTrafficInfos);
-  }
+  myGlobals.device[actualDeviceId].hosts.hostsno--;
 
   if(host->ipProtosList != NULL) {
     for(i=0; i<myGlobals.numIpProtosList; i++)
@@ -184,6 +149,8 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     free(host->ipProtosList);
   }
 
+  if(host->l7.traffic) free(host->l7.traffic);
+
   if(host->nonIPTraffic) {
     if(host->nonIPTraffic->nbHostName != NULL)          free(host->nonIPTraffic->nbHostName);
     if(host->nonIPTraffic->nbAccountName != NULL)       free(host->nonIPTraffic->nbAccountName);
@@ -245,8 +212,12 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
     if(host->protocolInfo->httpStats != NULL) free(host->protocolInfo->httpStats);
     if(host->protocolInfo->dhcpStats != NULL) free(host->protocolInfo->dhcpStats);
   }
+
   if(host->protocolInfo != NULL) free(host->protocolInfo);
 
+  if(host->sent_to_matrix)   { CM_Destroy(host->sent_to_matrix);   host->sent_to_matrix = NULL;   }
+  if(host->recv_from_matrix) { CM_Destroy(host->recv_from_matrix); host->recv_from_matrix = NULL; }
+
   /* ************************************* */
 
   if(host->icmpInfo != NULL) free(host->icmpInfo);
@@ -307,10 +278,12 @@ void freeHostInstances(int actualDeviceId) {
     hashSanityCheck();
 #endif
 
-    for(idx=FIRST_HOSTS_ENTRY; idx<myGlobals.device[actualDeviceId].actualHashSize; idx++) {
-      HostTraffic *el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+    for(idx=FIRST_HOSTS_ENTRY; idx<myGlobals.device[actualDeviceId].hosts.actualHashSize; idx++) {
+      HostTraffic *el;
 
-    if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) break;
+      // if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) break;
+
+      el = myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx];
 
       while(el != NULL) {
 	HostTraffic *nextEl = el->next;
@@ -321,7 +294,7 @@ void freeHostInstances(int actualDeviceId) {
 	el = nextEl;
       }
 
-      myGlobals.device[actualDeviceId].hash_hostTraffic[idx] = NULL;
+      myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx] = NULL;
     } /* for */
 
 #ifdef HASH_DEBUG
@@ -357,29 +330,35 @@ void readSessionPurgeParams() {
 }
 
 /* ************************************ */
-		     
+
 int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now) {
   /* Time used to decide whether a host need to be purged */
   time_t noSessionPurgeTime   = now-sec_idle_with_no_sessions;
   time_t withSessionPurgeTime = now-sec_idle_with_sessions;
+
+  if(!myGlobals.runningPref.stickyHosts) {
+    if(el->to_be_deleted) {
+      if(el->numHostSessions > 0) {
+	return(0);
+      }
+    }
+
+    if(el->to_be_deleted) return(1);
+
+    if(myGlobals.pcap_file_list == NULL)
+      if(el->refCount == 0)
+	if((!broadcastHost(el)) && (el != myGlobals.otherHostEntry))
+	  if(((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
+	     || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime)))
+	    if((myGlobals.device[actDevice].virtualDevice) /* e.g. sFlow/NetFlow */
+	       || (!myGlobals.runningPref.stickyHosts)
+	       || ((el->hostNumIpAddress[0] == '\0') /* Purge MAC addresses too */
+		   || (!subnetPseudoLocalHost(el))) /* Purge remote hosts only */
+	       )	      
+      return(1);
+  }
   
-  if(el->to_be_deleted
-     || ((myGlobals.pcap_file_list == NULL)
-	 && (el->refCount == 0)
-	 && ((((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
-	      || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime))))
-	 && (!broadcastHost(el)) && (el != myGlobals.otherHostEntry)
-	 && (myGlobals.device[actDevice].virtualDevice /* e.g. sFlow/NetFlow */
-	     || (!myGlobals.runningPref.stickyHosts)
-	     || ((el->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) &&
-		 ((el->hostNumIpAddress[0] == '\0') /* Purge MAC addresses too */
-		  || (!subnetPseudoLocalHost(el)))) /* Purge remote hosts only */
-	     )
-	 )
-     )
-    return(1);
-  else
-    return(0);
+  return(0);
 }
 
 /* ************************************ */
@@ -413,12 +392,10 @@ int purgeIdleHosts(int actDevice) {
   else
     lastPurgeTime[actDevice] = now;
 
-  maxHosts = myGlobals.device[actDevice].hostsno; /* save it as it can change */
-  myGlobals.piMem = maxHosts*sizeof(HostTraffic*);
+  maxHosts = myGlobals.device[actDevice].hosts.hostsno; /* save it as it can change */
+  myGlobals.piMem = (u_int)(maxHosts*sizeof(HostTraffic*));
   theFlaggedHosts = (HostTraffic**)calloc(1, myGlobals.piMem);
 
-  purgeOldFragmentEntries(actDevice); /* let's do this too */
-
 #ifdef IDLE_PURGE_DEBUG
   traceEvent(CONST_TRACE_INFO, "IDLE_PURGE_DEBUG: accessMutex(purgeMutex)...calling");
 #endif
@@ -433,11 +410,12 @@ int purgeIdleHosts(int actDevice) {
 
   accessMutex(&myGlobals.hostsHashLockMutex, "scanIdleLoop");
 
-  for(idx=0; idx<myGlobals.device[actDevice].actualHashSize; idx++) {
+  /* Step 1 - Mark hosts that will be purged */
+  for(idx=0; idx<myGlobals.device[actDevice].hosts.actualHashSize; idx++) {
     if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) break;
 
-    if((el = myGlobals.device[actDevice].hash_hostTraffic[idx]) != NULL) {
-      prev = NULL;     
+    if((el = myGlobals.device[actDevice].hosts.hash_hostTraffic[idx]) != NULL) {
+      prev = NULL;
 
       while(el) {
 	if(is_host_ready_to_purge(actDevice, el, now)) {
@@ -446,18 +424,18 @@ int purgeIdleHosts(int actDevice) {
 
 	    /* Skip it and move to next host */
 	    prev = el;
-	    el = el->next;	    
+	    el = el->next;
 	  } else {
 	  /* Host selected for deletion */
 	  theFlaggedHosts[numHosts++] = el;
-	  el->magic = CONST_UNMAGIC_NUMBER;
-	  remove_valid_ptr(el);
+	  el->magic = CONST_UNMAGIC_NUMBER; /* This means that this host os about to be freed */
+	  //remove_valid_ptr(el);
 	  next = el->next;
 
 	  if(prev != NULL)
 	    prev->next = next;
 	  else
-	    myGlobals.device[actDevice].hash_hostTraffic[idx] = next;
+	    myGlobals.device[actDevice].hosts.hash_hostTraffic[idx] = next;
 
           el->next = NULL;
 	  el = next;
@@ -499,7 +477,12 @@ int purgeIdleHosts(int actDevice) {
 	     "%d [out of %d] hosts selected",
 	     actDevice, myGlobals.device[actDevice].name, numHosts, scannedHosts);
 
-  /* Now free the entries */
+  /* Step 2 - Remove all timedout sessions and all those
+     whose one of the peers has been marked in step 1 */
+  if(myGlobals.runningPref.enableSessionHandling)
+    scanTimedoutTCPSessions(actDevice); /* let's check timedout sessions too */
+
+  /* Step 3 - Now free the entries */
   for(idx=0; idx<numHosts; idx++) {
 #ifdef IDLE_PURGE_DEBUG
     traceEvent(CONST_TRACE_INFO, "IDLE_PURGE_DEBUG: Purging host %d [last seen=%d]... %s",
@@ -512,9 +495,6 @@ int purgeIdleHosts(int actDevice) {
 
   free(theFlaggedHosts);
 
-  if(myGlobals.runningPref.enableSessionHandling)
-    scanTimedoutTCPSessions(actDevice); /* let's check timedout sessions too */
-
   gettimeofday(&hiresTimeEnd, NULL);
   hiresDeltaTime = timeval_subtract(hiresTimeEnd, hiresTimeStart);
 
@@ -534,12 +514,98 @@ int purgeIdleHosts(int actDevice) {
 
 /* **************************************************** */
 
+/* NOTE - myGlobals.serialLockMutex MUST be locked by caller */
+void dumpHostSerial(HostSerial *serial, HostSerialIndex serialHostIndex) {
+  HostSerialIndexDump siDump;
+  HostSerialDump      sDump;
+  datum data_data, key_data;
+
+  // FIX - Implement periodic serial purging
+  // traceEvent(CONST_TRACE_WARNING, "dumpHostSerial(%u)", serialHostIndex);
+
+  /* 1 - Dump key(serial) */
+  sDump.dump_date = siDump.dump_date = myGlobals.actTime;
+
+  siDump.idx = serialHostIndex;
+  key_data.dptr = (char*)serial, key_data.dsize = sizeof(HostSerial);
+  data_data.dptr = (char*)&siDump, data_data.dsize = sizeof(siDump);
+
+  if(gdbm_store(myGlobals.serialFile, key_data, data_data, GDBM_REPLACE) != 0)
+    traceEvent(CONST_TRACE_ERROR, "While adding host serial %u", serialHostIndex);
+
+  /* 2 - Dump key(serialHostIndex) */
+  memcpy(&sDump.serial, serial, sizeof(HostSerial));
+  key_data.dptr = (char*)&serialHostIndex, key_data.dsize = sizeof(HostSerialIndex);
+  data_data.dptr = (char*)&sDump, data_data.dsize = sizeof(sDump);
+
+  if(gdbm_store(myGlobals.serialFile, key_data, data_data, GDBM_REPLACE) != 0)
+    traceEvent(CONST_TRACE_ERROR, "While adding host serial %u", serialHostIndex);
+}
+
+/* **************************************************** */
+
+HostSerial* getHostSerialFromId(HostSerialIndex serialHostIndex, HostSerial *serial) {
+  datum return_data, key_data;
+
+  accessMutex(&myGlobals.serialLockMutex, "getHostSerialFromId");
+
+  key_data.dptr = (char*)&serialHostIndex, key_data.dsize = sizeof(serialHostIndex);
+  return_data = gdbm_fetch(myGlobals.serialFile, key_data);
+
+  if(return_data.dptr != NULL) {
+    HostSerialDump *dump = (HostSerialDump*)return_data.dptr;
+
+    memcpy(serial, &dump->serial, sizeof(HostSerial));
+    free(return_data.dptr);
+  } else {
+    /* Not found */
+    memset(serial, 0, sizeof(HostSerial));
+    serial->serialType = SERIAL_NONE;
+    traceEvent(CONST_TRACE_WARNING, "Failed getHostSerialFromId(%u)", serialHostIndex);
+  }
+
+  releaseMutex(&myGlobals.serialLockMutex);
+
+  return(serial);
+}
+
+/* **************************************************** */
+
+HostSerialIndex getHostIdFromSerial(HostSerial *serial) {
+  datum return_data, key_data;
+  HostSerialIndex serialHostIndex;
+
+  accessMutex(&myGlobals.serialLockMutex, "getHostSerialFromId");
+
+  key_data.dptr = (char*)serial, key_data.dsize = sizeof(HostSerial);
+  return_data = gdbm_fetch(myGlobals.serialFile, key_data);
+
+  if(return_data.dptr != NULL) {
+    HostSerialIndexDump *dump = (HostSerialIndexDump*)return_data.dptr;
+
+    memcpy(&serialHostIndex, &dump->idx, sizeof(serialHostIndex));
+    free(return_data.dptr);
+  } else {
+    /* Not found */
+    serialHostIndex = 0;
+    traceEvent(CONST_TRACE_WARNING, "Failed getHostIdFromSerial(%u)", serialHostIndex);
+  }
+
+  releaseMutex(&myGlobals.serialLockMutex);
+
+  return(serialHostIndex);
+}
+
+/* **************************************************** */
+
 void setHostSerial(HostTraffic *el) {
   /* Nothing to do */
   if(el->hostSerial.serialType != SERIAL_NONE)
     return;
 
-    if(el->hostNumIpAddress[0] == '\0') {
+  memset(&el->hostSerial, 0, sizeof(HostSerial));
+
+  if(el->hostNumIpAddress[0] == '\0') {
     el->hostSerial.serialType = SERIAL_MAC;
     memcpy(&el->hostSerial.value.ethSerial.ethAddress, el->ethAddress, LEN_ETHERNET_ADDRESS);
     el->hostSerial.value.ethSerial.vlanId = el->vlanId;
@@ -553,6 +619,12 @@ void setHostSerial(HostTraffic *el) {
     addrcpy(&el->hostSerial.value.ipSerial.ipAddress, &el->hostIpAddress);
     el->hostSerial.value.ipSerial.vlanId = el->vlanId;
   }
+
+  /* We now need to fill in the serialId field */
+  accessMutex(&myGlobals.serialLockMutex, "setHostSerial");
+  el->serialHostIndex = ++myGlobals.hostSerialCounter; /* Start from 1 (0 = UNKNOWN_SERIAL_INDEX) */
+  dumpHostSerial(&el->hostSerial, el->serialHostIndex);
+  releaseMutex(&myGlobals.serialLockMutex);
 }
 
 /* ********************************************************* */
@@ -563,11 +635,11 @@ void setHostSerial(HostTraffic *el) {
 */
 HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t vlanId,
 			 u_char checkForMultihoming, u_char forceUsingIPaddress,
-			 int actualDeviceId, char *file, int line) {
+			 int actualDeviceId, char *file, int line,
+			 const struct pcap_pkthdr *h, const u_char *p) {
   u_int idx, isMultihomed = 0;
   HostTraffic *el=NULL;
   char buf[MAX_LEN_SYM_HOST_NAME_HTML];
-  short useIPAddressForSearching = forceUsingIPaddress;
   char* symEthName = NULL, *ethAddr;
   u_char setSpoofingFlag = 0, locked_mutex = 0;
   u_short numRuns=0;
@@ -576,8 +648,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
   u_int32_t the_local_network, the_local_network_mask;
 
   if((hostIpAddress == NULL) && (ether_addr == NULL)) {
-    traceEvent(CONST_TRACE_WARNING, 
-	       "Both Ethernet and IP addresses are NULL in lookupHost()[%s:%d]", 
+    traceEvent(CONST_TRACE_WARNING,
+	       "Both Ethernet and IP addresses are NULL in lookupHost()[%s:%d]",
 	       file, line);
     return(NULL);
   }
@@ -586,8 +658,9 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
   hashSanityCheck();
 #endif
 
-  idx = hashHost(hostIpAddress, ether_addr,
-		 &useIPAddressForSearching,
+  idx = hashHost(hostIpAddress, 
+		 /* Either use IP or MAC address */
+		 hostIpAddress ? NULL : ether_addr,
 		 &el, actualDeviceId);
 
 #ifdef DEBUG
@@ -605,25 +678,21 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	       idx);
 #endif
 
-  /* Remember the side effect of above routine - if -o | --no-mac is set,\
-   * useIPAddressForSearching is now 1
-   */
-
   /* If we found it or had an error */
   if(el != NULL)
     return(el); /* Found */
   else if(idx == FLAG_NO_PEER)
     return(NULL);
   else {
-    el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+    el = myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx];
     if(el) {
       lockHostsHashMutex(el, "_lookupHost");
-      el = myGlobals.device[actualDeviceId].hash_hostTraffic[idx];
+      el = myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx];
       locked_mutex = 1;
     }
   }
 
-  while (el != NULL) {
+  while(el != NULL) {
     if(el->magic != CONST_MAGIC_NUMBER) {
       traceEvent(CONST_TRACE_ERROR,
                  "Bad magic number (expected=%d/real=%d) [deviceId=%d] lookupHost()[%s:%d]",
@@ -641,72 +710,19 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     }
 
     if(!is_host_ready_to_purge(actualDeviceId, el, myGlobals.actTime)) {
-      if(useIPAddressForSearching == 0) {
-	/* compare with the ethernet-address then the IP address */
-	if(memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0) {
-	  if((hostIpAddress != NULL) && (hostIpAddress->hostFamily == el->hostIpAddress.hostFamily)) {
-	    if((!isMultihomed) && checkForMultihoming) {
-	      /* This is a local address hence this is a potential multihomed host. */
-
-	      if(!(addrnull(&el->hostIpAddress)) &&
-		 (addrcmp(&el->hostIpAddress,hostIpAddress) != 0)) {
-		isMultihomed = 1;
-		setHostFlag(FLAG_HOST_TYPE_MULTIHOMED, el);
-	      } else {
-		updateIPinfo = 1;
-	      }
-	    }
-	    hostFound = 1;
-	    break;
-	  } else if(hostIpAddress == NULL) {  /* Only Mac Addresses */
-	    hostFound = 1;
-	    break;
-	  } else { /* MAC match found and we have the IP - need to update... */
-	    updateIPinfo = 1;
-	    hostFound = 1;
-	    break;
-	  }
-	} else if((hostIpAddress != NULL) &&
-		  (addrcmp(&el->hostIpAddress, hostIpAddress) == 0)) {
-	  /* Spoofing or duplicated MAC address:
-	     two hosts with the same IP address and different MAC
-	     addresses
-	  */
-
-	  if(!hasDuplicatedMac(el)) {
-	    setHostFlag(FLAG_HOST_DUPLICATED_MAC, el);
-
-	    if(myGlobals.runningPref.enableSuspiciousPacketDump) {
-	      char etherbuf[LEN_ETHERNET_ADDRESS_DISPLAY];
-
-	      traceEvent(CONST_TRACE_WARNING,
-			 "Two MAC addresses found for the same IP address "
-			 "%s: [%s/%s] (spoofing detected?)",
-			 el->hostNumIpAddress,
-			 etheraddr_string(ether_addr, etherbuf), el->ethAddressString);
-	      dumpSuspiciousPacket(actualDeviceId);
-	    }
-	  }
-
-	  setSpoofingFlag = 1;
-	  hostFound = 1;
-	  break;
-	}
-      } else {
-	/* -o | --no-mac (or NetFlow, which doesn't have MACs) - compare with only the IP address */
-	if((addrcmp(&el->hostIpAddress, hostIpAddress) == 0) 
-	   || (ether_addr && (memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0))) {
-	  hostFound = 1;
-	  break;
-	}
+      if((el->l2Host && ether_addr && (memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0)) 
+	 || ((!el->l2Host) && hostIpAddress && (addrcmp(&el->hostIpAddress,hostIpAddress) == 0))) {
+	hostFound = 1;
+	break;
       }
     }
+    
     el = el->next;
     numRuns++;
   } /* while */
 
-  if(locked_mutex) 
-    unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
+  if(locked_mutex)
+    unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
 
   if((hostFound == 1) && (vlanId != NO_VLAN) && (el->vlanId != NO_VLAN)
      && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
@@ -730,8 +746,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     }
   }
 
-  if(numRuns > myGlobals.device[actualDeviceId].hashListMaxLookups)
-    myGlobals.device[actualDeviceId].hashListMaxLookups = numRuns;
+  if(numRuns > myGlobals.device[actualDeviceId].hosts.hashListMaxLookups)
+    myGlobals.device[actualDeviceId].hosts.hashListMaxLookups = numRuns;
 
   if(hostFound) {
     /* Existing host entry */
@@ -754,7 +770,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     /* New host entry */
     int len;
 
-    if(myGlobals.device[actualDeviceId].hostsno >= myGlobals.runningPref.maxNumHashEntries) {
+    if(myGlobals.device[actualDeviceId].hosts.hostsno >= myGlobals.runningPref.maxNumHashEntries) {
       static char messageShown = 0;
 
       if(!messageShown) {
@@ -763,18 +779,22 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 		   myGlobals.runningPref.maxNumHashEntries);
       }
 
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
-        
+
     if((el = (HostTraffic*)malloc(sizeof(HostTraffic))) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
-     
+
     memset(el, 0, sizeof(HostTraffic));
     el->firstSeen = myGlobals.actTime;
 
+    /*
+      NOTE: resetHostsVariables allocates some memory thus
+      before re-allcoating is, make sure it's necessary
+    */
     resetHostsVariables(el);
     el->vlanId = vlanId;
 
@@ -784,37 +804,50 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
     if(el->portsUsage != NULL)
       freePortsUsage(el);
 
-    len = (size_t)myGlobals.numIpProtosList*sizeof(ShortProtoTrafficInfo**);
-    if((el->ipProtosList = (ShortProtoTrafficInfo**)malloc(len)) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
-      return(NULL);
+    /* We need to do +1 as protocol id's are starting from 1 */
+    if(el->l7.traffic == NULL) {
+      if((el->l7.traffic = (ProtoTraffic*)calloc(myGlobals.l7.numSupportedProtocols+1,
+						 sizeof(ProtoTraffic))) == NULL) {
+	if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
+	return(NULL);
+      }
+    }
+
+    if(el->ipProtosList == NULL) {
+      len = (int)(myGlobals.numIpProtosList*sizeof(ShortProtoTrafficInfo**));
+
+      if((el->ipProtosList = (ShortProtoTrafficInfo**)malloc(len)) == NULL) {
+	if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
+	return(NULL);
+      }
+      memset(el->ipProtosList, 0, len);
     }
-    memset(el->ipProtosList, 0, len);
 
     /*
     len = (size_t)myGlobals.numIpProtosToMonitor*sizeof(ProtoTrafficInfo**);
     if((el->protoIPTrafficInfos = (ProtoTrafficInfo**)malloc(len)) == NULL) {
-      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
+      if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
       return(NULL);
     }
-    memset(el->protoIPTrafficInfos, 0, len);   
+    memset(el->protoIPTrafficInfos, 0, len);
     */
 
-    el->magic = CONST_MAGIC_NUMBER;
-    el->hostTrafficBucket = idx; /* Set the bucket index */
+    el->magic = CONST_MAGIC_NUMBER, el->hostTrafficBucket = idx; /* Set the bucket index */
 
     /* traceEvent(CONST_TRACE_INFO, "new entry added at bucket %d", idx); */
 
     /* Put the new entry on top of the list */
-    el->next = myGlobals.device[actualDeviceId].hash_hostTraffic[el->hostTrafficBucket];
-    myGlobals.device[actualDeviceId].hash_hostTraffic[el->hostTrafficBucket] = el;  /* Insert a new entry */
-    myGlobals.device[actualDeviceId].hostsno++;
+    el->next = myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[el->hostTrafficBucket];
+    myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[el->hostTrafficBucket] = el;  /* Insert a new entry */
+    myGlobals.device[actualDeviceId].hosts.hostsno++;
 
-    if(0) 
-      traceEvent(CONST_TRACE_INFO, "-> Allocated(%d) [tot=%d]", 
-		 actualDeviceId, myGlobals.device[actualDeviceId].hostsno);
+    if(0)
+      traceEvent(CONST_TRACE_INFO, "-> Allocated(%d) [tot=%d]",
+		 actualDeviceId, myGlobals.device[actualDeviceId].hosts.hostsno);
 
     the_local_network = 0, the_local_network_mask = 0;
+    
+    el->l2Host = (hostIpAddress != NULL) ? 0 : 1;
 
     if(ether_addr != NULL) {
       if((hostIpAddress == NULL)
@@ -839,23 +872,21 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	   remote network that uses the same physical wire (or forged)*/
 	memcpy(el->lastEthAddress, ether_addr, LEN_ETHERNET_ADDRESS);
 
-	if(hostIpAddress != NULL) {
-	  if(hostIpAddress->hostFamily == AF_INET)
-	    memcpy(el->ethAddress, &hostIpAddress->Ip4Address.s_addr, 4); /* Dummy/unique eth address */
-	  else if(hostIpAddress->hostFamily == AF_INET6)
-	    memcpy(el->ethAddress, &hostIpAddress->Ip6Address.s6_addr[8], 4);
+	if(hostIpAddress->hostFamily == AF_INET)
+	  memcpy(el->ethAddress, &hostIpAddress->Ip4Address.s_addr, 4); /* Dummy/unique eth address */
+	else if(hostIpAddress->hostFamily == AF_INET6)
+	  memcpy(el->ethAddress, &hostIpAddress->Ip6Address.s6_addr[8], 4);
 
-	  clearHostFlag(FLAG_SUBNET_LOCALHOST, el);
+	clearHostFlag(FLAG_SUBNET_LOCALHOST, el);
 
-	  if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))
-	    setHostFlag(FLAG_PRIVATE_IP_ADDRESS, el);
+	if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))
+	  setHostFlag(FLAG_PRIVATE_IP_ADDRESS, el);
 
-	  if(!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) {
-	    if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask))
-	      setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
-	    else
-	      clearHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
-	  }
+	if(!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) {
+	  if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask))
+	    setHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
+	  else
+	    clearHostFlag(FLAG_SUBNET_PSEUDO_LOCALHOST, el);
 	}
       } else {
 	clearHostFlag(FLAG_SUBNET_LOCALHOST, el);
@@ -911,21 +942,21 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	*/
 	printf("Added a new hash_hostTraffic entry [%s/%s/%s/%d][idx=%d]\n",
 	       etheraddr_string(ether_addr, etherbuf), el->hostResolvedName,
-	       el->hostNumIpAddress, myGlobals.device[actualDeviceId].hostsno, idx);
+	       el->hostNumIpAddress, myGlobals.device[actualDeviceId].hosts.hostsno, idx);
       }
 #endif
 
       el->lastSeen = myGlobals.actTime;
 
       if(myGlobals.runningPref.enableSuspiciousPacketDump)
-	checkSpoofing(el, actualDeviceId);
+	checkSpoofing(el, actualDeviceId, h, p);
     }
 
     if(hostIpAddress != NULL) {
-      if(myGlobals.runningPref.dontTrustMACaddr && (ether_addr != NULL))
+      if(ether_addr != NULL)
 	memcpy(el->lastEthAddress, ether_addr, LEN_ETHERNET_ADDRESS);
 
-      addrcpy(&el->hostIpAddress,hostIpAddress);
+      addrcpy(&el->hostIpAddress, hostIpAddress);
       strncpy(el->hostNumIpAddress,
 	      _addrtostr(hostIpAddress, buf, sizeof(buf)),
 	      sizeof(el->hostNumIpAddress));
@@ -947,7 +978,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
       /* This is a new entry and hostIpAddress was NOT set.  Fill in MAC address, if we have it */
       if(symEthName[0] != '\0') {
         /* This is a local address so we have the MAC address */
-	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s%s", 
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s%s",
 		      symEthName, &el->ethAddressString[8]);
 
 	buf[MAX_LEN_SYM_HOST_NAME-1] = '\0';
@@ -959,8 +990,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
       traceEvent(CONST_TRACE_INFO, "HASH_DEBUG: Adding %s/%s [idx=%d][device=%d]"
 		 "[actualHashSize=%d][#hosts=%d]",
 		 el->ethAddressString, el->hostNumIpAddress, idx, actualDeviceId,
-		 myGlobals.device[actualDeviceId].actualHashSize, 
-		 myGlobals.device[actualDeviceId].hostsno);
+		 myGlobals.device[actualDeviceId].hosts.actualHashSize,
+		 myGlobals.device[actualDeviceId].hosts.hostsno);
 #endif
 
     setHostSerial(el);
@@ -982,7 +1013,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	traceEvent(CONST_TRACE_INFO, "lookupHost(idx=%d/actualDeviceId=%d) [%s/%s/%s/%d/%d]",
 		   idx, actualDeviceId,
 		   etheraddr_string(ether_addr, etherbuf), el->hostResolvedName,
-		   el->hostNumIpAddress, myGlobals.device[actualDeviceId].hostsno,
+		   el->hostNumIpAddress, myGlobals.device[actualDeviceId].hosts.hostsno,
 		   useIPAddressForSearching);
       }
     }
@@ -998,8 +1029,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
   hashSanityCheck();
 #endif
 
-  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]), locked_mutex = 0;
-  
+  if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hosts.hash_hostTraffic[idx]), locked_mutex = 0;
+
   return(el);
 }
 
@@ -1024,8 +1055,8 @@ static void dumpHash() {
 static void hashSanityCheck() {
   int i=0;
 
-  for(i=FIRST_HOSTS_ENTRY; i<myGlobals.device[0].actualHashSize; i++) {
-    HostTraffic *el = myGlobals.device[0].hash_hostTraffic[i];
+  for(i=FIRST_HOSTS_ENTRY; i<myGlobals.device[0].hosts.actualHashSize; i++) {
+    HostTraffic *el = myGlobals.device[0].hosts.hash_hostTraffic[i];
 
     while(el != NULL) {
       if(el->hostTrafficBucket != i)
@@ -1042,8 +1073,8 @@ static void hashSanityCheck() {
 static void hostHashSanityCheck(HostTraffic *host) {
   int i=0;
 
-  for(i=FIRST_HOSTS_ENTRY; i<myGlobals.device[0].actualHashSize; i++) {
-    HostTraffic *el = myGlobals.device[0].hash_hostTraffic[i];
+  for(i=FIRST_HOSTS_ENTRY; i<myGlobals.device[0].hosts.actualHashSize; i++) {
+    HostTraffic *el = myGlobals.device[0].hosts.hash_hostTraffic[i];
 
     while(el != NULL) {
       if(el == host)
@@ -1057,10 +1088,10 @@ static void hostHashSanityCheck(HostTraffic *host) {
 
 #endif /* HASH_DEBUG */
 
-/* ****************************** 
-   
+/* ******************************
+
    Utility functions used by the remote plugin
-   
+
    ****************************** */
 
 #define MAX_NUM_VALID_PTRS   8
@@ -1068,7 +1099,7 @@ static void* valid_ptrs[MAX_NUM_VALID_PTRS] = { NULL };
 
 void add_valid_ptr(void* ptr) {
   int i;
-  
+
   traceEvent(CONST_TRACE_INFO, "add_valid_ptr(%p)", ptr);
 
   for(i=0; i<MAX_NUM_VALID_PTRS; i++) {
@@ -1084,14 +1115,14 @@ void add_valid_ptr(void* ptr) {
 /* ****************************** */
 
 void remove_valid_ptr(void* ptr) {
-  int i;  
+  int i;
 
   for(i=0; i<MAX_NUM_VALID_PTRS; i++) {
     if(valid_ptrs[i] == ptr) {
       valid_ptrs[i] = NULL;
       return;
     }
-  } 
+  }
 
   /* traceEvent(CONST_TRACE_ERROR, "remove_valid_ptr(%p) failed", ptr); */
 }
@@ -1109,7 +1140,7 @@ int is_valid_ptr(void* ptr) {
 	valid_ptrs[i-1] = valid_ptrs[i];
 	valid_ptrs[i] = swap;
       }
-      
+
       traceEvent(CONST_TRACE_INFO, "is_valid_ptr(%p): 1", ptr);
       return(1);
     }
