@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2003      Abdelkader Lahmadi <Abdelkader.Lahmadi@loria.fr>
  *                         Olivier Festor <Olivier.Festor@loria.fr>
- * Copyright (C) 2003-10   Luca Deri <deri@ntop.org>
+ * Copyright (C) 2003-11   Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -25,8 +25,7 @@
 
 #include "ntop.h"
 
-#ifdef INET6
-
+#if 0 
 #define ROUNDUP(a) \
         ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
 
@@ -50,7 +49,6 @@ static struct in_addr *in4_cpy(struct in_addr *dst, struct in_addr *src) {
   memcpy(dst, src, sizeof(struct in_addr));
   return(dst);
 }
-#endif
 
 /* ************************************************* */
 
@@ -436,22 +434,18 @@ int iface_addrcount(struct iface_handler *hdlr) {
   return hdlr->addr_count;
 }
 
-
 int iface_ifcount(struct iface_handler *hdlr) {
   return hdlr->if_count;
 }
 
-
 struct iface_if *iface_getif_first(struct iface_handler *hdlr) {
   return hdlr->if_count ? &hdlr->if_list[0] : NULL;
 }
 
-
 struct iface_if *iface_getif_next(struct iface_if *ii) {
   return ii ? ii->next : NULL;
 }
 
-
 int iface_if_addrcount(struct iface_if *ii, int family) {
   struct iface_addr *ia;
   int count = 0;
@@ -462,8 +456,6 @@ int iface_if_addrcount(struct iface_if *ii, int family) {
   return count;
 }
 
-
-
 struct iface_if *iface_getif_byindex(struct iface_handler *hdlr, int idx) {
   int i;
 
@@ -473,7 +465,6 @@ struct iface_if *iface_getif_byindex(struct iface_handler *hdlr, int idx) {
   return NULL;
 }
 
-
 struct iface_if *iface_getif_byname(struct iface_handler *hdlr, char *name) {
   int i;
 
@@ -483,7 +474,6 @@ struct iface_if *iface_getif_byname(struct iface_handler *hdlr, char *name) {
   return NULL;
 }
 
-
 char *iface_if_getname(struct iface_if *ii, char *name, int size) {
   if (name) {
     name[size-1] = '\0';
@@ -498,19 +488,14 @@ int iface_if_getindex(struct iface_if *ii) {
   return ii->index;
 }
 
-
-
 int iface_if_getinfo(struct iface_if *ii) {
   return ii->info;
 }
 
-
 int iface_if_gettype(struct iface_if *ii) {
   return ii->type;
 }
 
-
-
 int iface_if_getphys(struct iface_if *ii,int *type,char *addr,int *addrsize) {
   if (type)
     *type = ii->type;
@@ -522,7 +507,6 @@ int iface_if_getphys(struct iface_if *ii,int *type,char *addr,int *addrsize) {
   return ii->phys.size;
 }
 
-
 struct iface_addr *iface_getaddr_first(struct iface_if *ii, int family) {
   struct iface_addr	*ia;
 
@@ -541,17 +525,14 @@ struct iface_addr *iface_getaddr_next(struct iface_addr *ia, int family) {
   return ia;
 }
 
-
 int iface_addr_getfamily(struct iface_addr *ia) {
   return ia->family;
 }
 
-
 int iface_addr_ifindex(struct iface_addr *ia) {
   return ia->ifi->index;
 }
 
-
 void *iface_addr_getinfo(struct iface_addr *ia, void *infobuf) {
   switch(ia->family) {
   case AF_INET:
@@ -742,6 +723,8 @@ int iface_getalladdr(int type, int *size, char **addr){
 
 #endif /* INET6 */
 
+#endif /* 0 */
+
 /* ********************************************************************** */
 
 void sanitizeIfName(char *deviceDescr) {
