@@ -1,8 +1,9 @@
 /*
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
- *                          HutuBttp://www.ntop.org
  *
- * Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *                          http://www.ntop.org
+ *
+ *                  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -57,15 +58,12 @@ static void setDomainName(void) {
     if(myGlobals.runningPref.domainName[0] == '\0') {
       char szLclHost[64];
       struct hostent *lpstHostent;
-      struct in_addr stLclAddr;
 
       gethostname(szLclHost, 64);
       lpstHostent = gethostbyname(szLclHost);
       if(lpstHostent) {
 	struct hostent *hp;
 
-	stLclAddr.s_addr = ntohl(*(lpstHostent->h_addr));
-
 	hp = (struct hostent*)gethostbyaddr((char*)lpstHostent->h_addr, 4, AF_INET);
 
 	if(hp && (hp->h_name)) {
@@ -89,12 +87,12 @@ static void setDomainName(void) {
   }
 #endif
 
-  len = strlen(myGlobals.runningPref.domainName)-1;
+  len = (int)(strlen(myGlobals.runningPref.domainName)-1);
 
   while((len > 0) && (myGlobals.runningPref.domainName[len] != '.'))
     len--;
 
-  if((len > 0) 
+  if((len > 0)
      && ((len+1) <  strlen(myGlobals.runningPref.domainName)))
     myGlobals.shortDomainName = strdup(&myGlobals.runningPref.domainName[len+1]);
   else
@@ -139,7 +137,7 @@ void initIPServices(void) {
 
   /* ************************************* */
 
-  len = sizeof(ServiceEntry*)*myGlobals.numActServices;
+  len = (int)(sizeof(ServiceEntry*)*myGlobals.numActServices);
   myGlobals.udpSvc = (ServiceEntry**)malloc(len);
   memset(myGlobals.udpSvc, 0, len);
   myGlobals.tcpSvc = (ServiceEntry**)malloc(len);
@@ -230,15 +228,15 @@ void resetDevice(int devIdx, short fullReset) {
 
   if(myGlobals.device[devIdx].dummyDevice) return;
 
-  myGlobals.device[devIdx].actualHashSize = CONST_HASH_INITIAL_SIZE;
+  myGlobals.device[devIdx].hosts.actualHashSize = CONST_HASH_INITIAL_SIZE;
 
   len = CONST_HASH_INITIAL_SIZE * sizeof(HostTraffic*);
-  if(myGlobals.device[devIdx].hash_hostTraffic == NULL) {
+  if(myGlobals.device[devIdx].hosts.hash_hostTraffic == NULL) {
     ptr = calloc(CONST_HASH_INITIAL_SIZE, sizeof(HostTraffic*));
-    myGlobals.device[devIdx].hash_hostTraffic = ptr;
+    myGlobals.device[devIdx].hosts.hash_hostTraffic = ptr;
   }
 
-  memset(myGlobals.device[devIdx].hash_hostTraffic, 0, len);
+  memset(myGlobals.device[devIdx].hosts.hash_hostTraffic, 0, len);
 
   resetTrafficCounter(&myGlobals.device[devIdx].receivedPkts);
   resetTrafficCounter(&myGlobals.device[devIdx].droppedPkts);
@@ -253,12 +251,10 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].udpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].otherIpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].icmpBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].dlcBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].stpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].ipsecBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].netbiosBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].arpRarpBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].egpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].greBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].ipv6Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].otherBytes);
@@ -291,9 +287,6 @@ void resetDevice(int devIdx, short fullReset) {
   myGlobals.device[devIdx].throughput = 0;
   myGlobals.device[devIdx].packetThroughput = 0;
   myGlobals.device[devIdx].numThptSamples = 0;
-  myGlobals.device[devIdx].last60MinutesThptIdx = 0;
-  myGlobals.device[devIdx].last24HoursThptIdx = 0;
-  myGlobals.device[devIdx].last30daysThptIdx = 0;
 
   if(myGlobals.pcap_file_list == NULL) {
     myGlobals.device[devIdx].lastThptUpdate = myGlobals.device[devIdx].lastMinThptUpdate =
@@ -304,29 +297,21 @@ void resetDevice(int devIdx, short fullReset) {
   memset(&myGlobals.device[devIdx].tcpGlobalTrafficStats, 0, sizeof(SimpleProtoTrafficInfo));
   memset(&myGlobals.device[devIdx].udpGlobalTrafficStats, 0, sizeof(SimpleProtoTrafficInfo));
   memset(&myGlobals.device[devIdx].icmpGlobalTrafficStats, 0, sizeof(SimpleProtoTrafficInfo));
-  memset(myGlobals.device[devIdx].last60MinutesThpt, 0, sizeof(myGlobals.device[devIdx].last60MinutesThpt));
-  memset(myGlobals.device[devIdx].last24HoursThpt, 0, sizeof(myGlobals.device[devIdx].last24HoursThpt));
-  memset(myGlobals.device[devIdx].last30daysThpt, 0, sizeof(myGlobals.device[devIdx].last30daysThpt));
-  myGlobals.device[devIdx].last60MinutesThptIdx=0, myGlobals.device[devIdx].last24HoursThptIdx=0,
-    myGlobals.device[devIdx].last30daysThptIdx=0;
-  myGlobals.device[devIdx].hostsno = 1; /* Broadcast entry */
+  memset(myGlobals.device[devIdx].last60MinTopTalkers, 0, sizeof(myGlobals.device[devIdx].last60MinTopTalkers));
+  memset(myGlobals.device[devIdx].last24HoursTopTalkers, 0, sizeof(myGlobals.device[devIdx].last24HoursTopTalkers));
+  myGlobals.device[devIdx].hosts.hostsno = 1; /* Broadcast entry */
 
   if(fullReset) {
     if(myGlobals.device[devIdx].netflowGlobals != NULL)
       free(myGlobals.device[devIdx].netflowGlobals);
     myGlobals.device[devIdx].netflowGlobals = NULL;
-    
+
     if(myGlobals.device[devIdx].sflowGlobals != NULL)
       free(myGlobals.device[devIdx].sflowGlobals);
     myGlobals.device[devIdx].sflowGlobals = NULL;
   }
-  
-  len = (size_t)myGlobals.numIpProtosToMonitor*sizeof(SimpleProtoTrafficInfo);
 
-  if(myGlobals.device[devIdx].ipProtoStats == NULL)
-    myGlobals.device[devIdx].ipProtoStats = (SimpleProtoTrafficInfo*)malloc(len);
-
-  memset(myGlobals.device[devIdx].ipProtoStats, 0, len);
+  len = (int)(myGlobals.numIpProtosToMonitor*sizeof(SimpleProtoTrafficInfo));
 
   if(myGlobals.device[devIdx].ipProtosList != NULL) {
     free(myGlobals.device[devIdx].ipProtosList);
@@ -368,23 +353,18 @@ void initCounters(void) {
 
   for(i=0; i<myGlobals.numDevices; i++) {
     if(myGlobals.runningPref.enableSessionHandling) {
-      len = sizeof(IPSession*)*MAX_TOT_NUM_SESSIONS;
-      myGlobals.device[i].tcpSession = (IPSession**)malloc(len);
-      memset(myGlobals.device[i].tcpSession, 0, len);
-    } else {
-      myGlobals.device[i].tcpSession     = NULL;
-    }
-
-    myGlobals.device[i].fragmentList = NULL;
+      myGlobals.device[i].sessions = (IPSession**)calloc(sizeof(IPSession*), MAX_TOT_NUM_SESSIONS);
+    } else
+      myGlobals.device[i].sessions = NULL;
   }
 
-  myGlobals.hashCollisionsLookup     = 0;
+  myGlobals.hashCollisionsLookup = 0;
 
   if(myGlobals.pcap_file_list == NULL)
-    myGlobals.initialSniffTime = myGlobals.lastRefreshTime = time(NULL);  
+    myGlobals.initialSniffTime = myGlobals.lastRefreshTime = time(NULL);
   else
     myGlobals.initialSniffTime = 0; /* We set the start when first pkt is
-                                       * read */  
+                                       * read */
 
 /* TODO why here AND in globals-core.c? */
   myGlobals.numHandledSIGPIPEerrors = 0;
@@ -405,16 +385,6 @@ void initCounters(void) {
   myGlobals.numHandledSSIRequests = 0;
 
   myGlobals.webServerRequestQueueLength = DEFAULT_WEBSERVER_REQUEST_QUEUE_LEN;
-
-  myGlobals.hostsCacheLen = 0;
-  myGlobals.hostsCacheLenMax = 0;
-  myGlobals.hostsCacheReused = 0;
-#ifdef PARM_USE_SESSIONS_CACHE
-  myGlobals.sessionsCacheLen = 0;
-  myGlobals.sessionsCacheLenMax = 0;
-  myGlobals.sessionsCacheReused = 0;
-#endif
-
 }
 
 /* ******************************* */
@@ -428,8 +398,8 @@ void resetStats(int deviceId) {
   if(myGlobals.purgeMutex.isInitialized)
     accessMutex(&myGlobals.purgeMutex, "resetStats");
 
-  for(j=FIRST_HOSTS_ENTRY; j<myGlobals.device[deviceId].actualHashSize; j++) {
-    HostTraffic *el = myGlobals.device[deviceId].hash_hostTraffic[j], *elNext;
+  for(j=FIRST_HOSTS_ENTRY; j<myGlobals.device[deviceId].hosts.actualHashSize; j++) {
+    HostTraffic *el = myGlobals.device[deviceId].hosts.hash_hostTraffic[j], *elNext;
 
     if(el) lockExclusiveHostsHashMutex(el, "resetStats");
 
@@ -441,34 +411,34 @@ void resetStats(int deviceId) {
 	freeHostInfo(el, deviceId);
 	if(elNext) lockExclusiveHostsHashMutex(elNext, "resetStats");
       } else {
-	if(!elNext) 
+	if(!elNext)
 	  unlockExclusiveHostsHashMutex(el);
       }
 
       el = elNext;
     }
 
-    myGlobals.device[deviceId].hash_hostTraffic[j] = NULL;
+    myGlobals.device[deviceId].hosts.hash_hostTraffic[j] = NULL;
   }
 
   resetDevice(deviceId, 0);
 
-  if(myGlobals.device[deviceId].tcpSession != NULL) {
+  if(myGlobals.device[deviceId].sessions != NULL) {
     for(j=0; j<MAX_TOT_NUM_SESSIONS; j++)
-      if(myGlobals.device[deviceId].tcpSession[j] != NULL) {
-	free(myGlobals.device[deviceId].tcpSession[j]);
-	myGlobals.device[deviceId].tcpSession[j] = NULL;
+      if(myGlobals.device[deviceId].sessions[j] != NULL) {
+	free(myGlobals.device[deviceId].sessions[j]);
+	myGlobals.device[deviceId].sessions[j] = NULL;
       }
   }
-  
-  myGlobals.device[deviceId].hash_hostTraffic[BROADCAST_HOSTS_ENTRY] = myGlobals.broadcastEntry;
+
+  myGlobals.device[deviceId].hosts.hash_hostTraffic[BROADCAST_HOSTS_ENTRY] = myGlobals.broadcastEntry;
   myGlobals.broadcastEntry->hostSerial.serialType = SERIAL_IPV4;
   myGlobals.broadcastEntry->hostSerial.value.ipSerial.ipAddress.Ip4Address.s_addr = -1;
   myGlobals.broadcastEntry->next = NULL;
   setHostFlag(FLAG_BROADCAST_HOST, myGlobals.broadcastEntry);
 
   if(myGlobals.otherHostEntry != myGlobals.broadcastEntry) {
-    myGlobals.device[deviceId].hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
+    myGlobals.device[deviceId].hosts.hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
     /* Dirty trick */
     myGlobals.otherHostEntry->hostSerial.serialType = SERIAL_IPV4;
     myGlobals.otherHostEntry->hostSerial.value.ipSerial.ipAddress.Ip4Address.s_addr = -1;
@@ -487,15 +457,15 @@ void initSingleGdbm(GDBM_FILE *database,
   char tmpBuf[200], theDate[48];
   time_t st_time, now;
   struct tm t;
-  int d;
+  double d;
 
   /* Courtesy of Andreas Pfaller <apfaller@yahoo.com.au>. */
   memset(&tmpBuf, 0, sizeof(tmpBuf));
- 
+
 #ifdef WIN32
   {
     unsigned long driveSerial;
-  
+
     get_serial(&driveSerial);
 
     safe_snprintf(__FILE__, __LINE__, tmpBuf, sizeof(tmpBuf), "%s/%u",
@@ -517,7 +487,6 @@ void initSingleGdbm(GDBM_FILE *database,
     if(stat(tmpBuf, statbuf) == 0) {
       /* File already exists */
       if((doUnlink != TRUE) && (doUnlink != FALSE)) {
-
 	traceEvent(CONST_TRACE_INFO, "Checking age of database %s", tmpBuf);
 
 	/* Some systems or mounts don't maintain atime so fox 'em */
@@ -534,7 +503,7 @@ void initSingleGdbm(GDBM_FILE *database,
 	theDate[sizeof(theDate)-1] = '\0';
 	now  = time(NULL);
 	traceEvent(CONST_TRACE_NOISY,
-		   "...last create/modify/access was %s, %d second(s) ago",
+		   "...last create/modify/access was %s, %.1f second(s) ago",
 		   theDate,
 		   d = difftime(now, st_time));
 
@@ -606,7 +575,7 @@ void reinitMutexes (void) {
   createMutex(&myGlobals.portsMutex);       /* Avoid race conditions while handling ports */
 
   for(i=0; i<NUM_SESSION_MUTEXES; i++)
-    createMutex(&myGlobals.tcpSessionsMutex[i]); /* data to synchronize TCP sessions access */
+    createMutex(&myGlobals.sessionsMutex[i]); /* data to synchronize TCP sessions access */
 
     createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
@@ -647,7 +616,7 @@ void initThreads(void) {
 
   if(myGlobals.runningPref.numericFlag != noDnsResolution) {
     createMutex(&myGlobals.addressResolutionMutex);
- 
+
 #if defined(HAVE_GETHOSTBYADDR_R)
     myGlobals.numDequeueAddressThreads = MAX_NUM_DEQUEUE_ADDRESS_THREADS;
 #else
@@ -655,10 +624,10 @@ void initThreads(void) {
 #endif
 
     initAddressResolution();
-   
+
     /*
      * Create the thread (5) - DNSAR - DNS Address Resolution - optional
-     */  
+     */
     for(i=0; i<myGlobals.numDequeueAddressThreads; i++) {
       createThread(&myGlobals.dequeueAddressThreadId[i], dequeueAddress, (char*)((long)i));
       traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Started thread for DNS address resolution",
@@ -680,12 +649,12 @@ void initApps(void) {
 void initDeviceSemaphores(int deviceId) {
   traceEvent(CONST_TRACE_INFO, "Initializing device %s (%d)",
 	     myGlobals.device[deviceId].name, deviceId);
-    
+
   createMutex(&myGlobals.device[deviceId].counterMutex);
   createMutex(&myGlobals.device[deviceId].asMutex);
   createMutex(&myGlobals.device[deviceId].packetProcessMutex);
   createMutex(&myGlobals.device[deviceId].packetQueueMutex);
-  if(myGlobals.device[deviceId].packetQueue) 
+  if(myGlobals.device[deviceId].packetQueue)
     memset(myGlobals.device[deviceId].packetQueue, 0,
 	   sizeof(PacketInformation) * (CONST_PACKET_QUEUE_LENGTH+1));
   myGlobals.device[deviceId].packetQueueLen           = 0;
@@ -700,12 +669,15 @@ void initDeviceSemaphores(int deviceId) {
 
 void allocDeviceMemory(int deviceId) {
   if(!myGlobals.device[deviceId].ipPorts)
-    myGlobals.device[deviceId].ipPorts = 
+    myGlobals.device[deviceId].ipPorts =
       (PortCounter**)calloc(sizeof(PortCounter*), MAX_IP_PORT);
-  
+
   if(!myGlobals.device[deviceId].packetQueue)
-    myGlobals.device[deviceId].packetQueue = 
+    myGlobals.device[deviceId].packetQueue =
       (PacketInformation*)calloc(sizeof(PacketInformation), (CONST_PACKET_QUEUE_LENGTH+1));
+
+  initL7DeviceDiscovery(deviceId);
+  myGlobals.device[deviceId].l7.protoTraffic = (Counter*)calloc(myGlobals.l7.numSupportedProtocols, sizeof(Counter));
 }
 
 /* ******************************* */
@@ -818,12 +790,12 @@ void addDevice(char* deviceName, char* deviceDescr) {
     }
 #endif
 
-    
+
     myGlobals.device[deviceId].pcapPtr =
       pcap_open_live(myGlobals.device[deviceId].name,
 		     MAX_PACKET_LEN,
 		     myGlobals.runningPref.disablePromiscuousMode == 1 ? 0 : 1,
-		     1000 /* ms */, ebuf);    
+		     1000 /* ms */, ebuf);
 
       if(myGlobals.device[deviceId].pcapPtr == NULL) {
 	traceEvent(CONST_TRACE_ERROR, "pcap_open_live(): '%s'", ebuf);
@@ -837,6 +809,10 @@ void addDevice(char* deviceName, char* deviceDescr) {
         exit(10); /* Just in case */
       }
 
+#ifdef HAVE_PF_RING
+      pcap_set_appl_name_linux(myGlobals.device[deviceId].pcapPtr, "ntop");
+#endif
+
       if(myGlobals.runningPref.pcapLog != NULL) {
 	if(strlen(myGlobals.runningPref.pcapLog) > 64)
 	  myGlobals.runningPref.pcapLog[64] = '\0';
@@ -879,24 +855,6 @@ void addDevice(char* deviceName, char* deviceDescr) {
         } else
 	  traceEvent(CONST_TRACE_NOISY, "Saving packets into file %s", myName);
       }
-
-      if(myGlobals.runningPref.enableOtherPacketDump) {
-        safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-other-pkts.%s.pcap",
-		      myGlobals.runningPref.pcapLogBasePath,
-		      CONST_PATH_SEP,
-		      myGlobals.device[deviceId].uniqueIfName != NULL ?
-		      myGlobals.device[deviceId].uniqueIfName :
-		      myGlobals.device[deviceId].name);
-
-	myGlobals.device[deviceId].pcapOtherDumper = pcap_dump_open(myGlobals.device[deviceId].pcapPtr, myName);
-
-	if(myGlobals.device[deviceId].pcapOtherDumper == NULL) {
-          myGlobals.runningPref.enableOtherPacketDump = 0;
-	  traceEvent(CONST_TRACE_ERROR, "pcap_dump_open(..., '%s') failed (other (unknown) packets)", myName);
-	  traceEvent(CONST_TRACE_INFO, "Continuing without other (unknown) packet dump");
-        } else
-	  traceEvent(CONST_TRACE_NOISY, "Saving packets into file %s", myName);
-      }
     } else {
       myGlobals.device[deviceId].virtualDevice = 1;
       if(column != NULL) column[0] = ':';
@@ -938,7 +896,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
        * is designed to ensure a certain minimal # even for smaller networks
        */
       myGlobals.device[deviceId].numHosts +=
-	ceil(log((double)(0xFFFFFFFF - myGlobals.device[deviceId].netmask.s_addr + 1))+1.0)*50;
+	(int)(ceil(log((double)(0xFFFFFFFF - myGlobals.device[deviceId].netmask.s_addr + 1))+1.0)*50);
     }
 
     if(myGlobals.device[deviceId].numHosts > MAX_SUBNET_HOSTS) {
@@ -956,23 +914,24 @@ void addDevice(char* deviceName, char* deviceDescr) {
 
   /* ********************************************* */
 
+  #ifdef INET6
   if(!(myGlobals.device[deviceId].dummyDevice || myGlobals.device[deviceId].virtualDevice)) {
     u_int8_t netmask_v6;
-    
+
     getLocalHostAddress(&myGlobals.device[deviceId].ifAddr, &netmask_v6, myGlobals.device[deviceId].name);
-    myGlobals.device[deviceId].v6Addrs = getLocalHostAddressv6(myGlobals.device[deviceId].v6Addrs, 
-							       myGlobals.device[deviceId].name);
+    myGlobals.device[deviceId].v6Addrs = getLocalHostAddressv6(myGlobals.device[deviceId].v6Addrs, myGlobals.device[deviceId].name);
 	if(myGlobals.device[deviceId].network.s_addr == 0) {
 		myGlobals.device[deviceId].netmask.s_addr = 0xFFFFFF00;/* /24 */
-		myGlobals.device[deviceId].network.s_addr = myGlobals.device[deviceId].ifAddr.s_addr 
+		myGlobals.device[deviceId].network.s_addr = myGlobals.device[deviceId].ifAddr.s_addr
 		  & myGlobals.device[deviceId].netmask.s_addr;
 	}
   }
+#endif
 
   mallocLen = 2;
   for(i=0; i<myGlobals.numDevices; i++) {
     if(myGlobals.device[i].name != NULL)
-      mallocLen += strlen(myGlobals.device[i].name) + 2;
+      mallocLen += (int)(strlen(myGlobals.device[i].name) + 2);
   }
 
   workDevices = calloc(mallocLen+1, 1);
@@ -982,9 +941,10 @@ void addDevice(char* deviceName, char* deviceDescr) {
 
   for(i=0; i<myGlobals.numDevices; i++) {
     if(myGlobals.device[i].name != NULL) {
-      int len = strlen(workDevices);
-      safe_snprintf(__FILE__, __LINE__, 
-		    &workDevices[len], mallocLen-len, 
+      int len = (int)strlen(workDevices);
+
+      safe_snprintf(__FILE__, __LINE__,
+		    &workDevices[len], mallocLen-len,
 		    "%s%s", (i > 0) ? ", " : "",
 		    myGlobals.device[i].name);
     }
@@ -1010,7 +970,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
       traceEvent(CONST_TRACE_INFO, "Checking %s for additional devices", myGlobals.device[deviceId].name);
       for(k=0; k<=MAX_NUM_DEVICES_VIRTUAL; k++) {
 	u_int8_t netmask_v6;
-	
+
 	safe_snprintf(__FILE__, __LINE__, tmpDeviceName, sizeof(tmpDeviceName), "%s:%d", myGlobals.device[deviceId].name, k);
 
 	traceEvent(CONST_TRACE_NOISY, "Checking %s", tmpDeviceName);
@@ -1040,20 +1000,20 @@ void addDevice(char* deviceName, char* deviceDescr) {
   initDeviceDatalink(deviceId);
 
   if((myGlobals.actualReportDeviceId == 0) && myGlobals.device[0].dummyDevice)
-    myGlobals.actualReportDeviceId = deviceId;  
+    myGlobals.actualReportDeviceId = deviceId;
 }
 
 /* ******************************* */
 
 int validInterface(char *name) {
-  if(name && 
+  if(name &&
      (strstr(name, "PPP") /* Avoid to use the PPP interface */
       || strstr(name, "dialup")  /* Avoid to use the dialup interface */
       || strstr(name, "ICSHARE")  /* Avoid to use the internet sharing interface */
       || strstr(name, "NdisWan"))) { /* Avoid to use the internet sharing interface */
     return(0);
   }
-  
+
   return(1);
 }
 
@@ -1183,7 +1143,7 @@ void initDevices(char* devices) {
 
       traceEvent(CONST_TRACE_NOISY, "Checking requested device '%s'", tmpDev);
 
-      if(((nwInterface = strchr(tmpDev, ':')) != NULL) 
+      if(((nwInterface = strchr(tmpDev, ':')) != NULL)
 	 && (!strstr(tmpDev, "dag")) /* Endace DAG cards are valid (e.g. dag0:0) */
 	 )
 	{
@@ -1229,7 +1189,7 @@ void initDevices(char* devices) {
 	  tmpDescr = intDescr[atoi(tmpDev)];
 	  tmpDev   = intNames[atoi(tmpDev)];
 	} else {
-	  traceEvent(CONST_TRACE_ERROR, "Interface index '%d' is out of range [0..%d]", 
+	  traceEvent(CONST_TRACE_ERROR, "Interface index '%d' is out of range [0..%d]",
 		  atoi(tmpDev), ifIdx > 0 ? (ifIdx -1) : 0);
 	  return;
 	}
@@ -1419,7 +1379,6 @@ u_int createDummyInterface(char *ifName) {
   u_int mallocLen;
 #endif
   u_int deviceId = myGlobals.numDevices;
-  int i;
 
   traceEvent(CONST_TRACE_INFO, "Creating dummy interface, '%s'", ifName);
 
@@ -1431,32 +1390,28 @@ u_int createDummyInterface(char *ifName) {
   memset(&myGlobals.device[deviceId], 0, sizeof(NtopInterface));
 
   resetDevice(deviceId, 1);
+  initL7DeviceDiscovery(deviceId);
+
   myGlobals.device[deviceId].network.s_addr = 0xFFFFFFFF;
   myGlobals.device[deviceId].netmask.s_addr = 0xFFFFFFFF;
   myGlobals.device[deviceId].numHosts = myGlobals.device[0].numHosts;
   myGlobals.device[deviceId].name = strdup(ifName);
   myGlobals.device[deviceId].humanFriendlyName = strdup(ifName);
   myGlobals.device[deviceId].datalink = DLT_EN10MB;
-  myGlobals.device[deviceId].hash_hostTraffic[BROADCAST_HOSTS_ENTRY] = myGlobals.broadcastEntry;
+  myGlobals.device[deviceId].hosts.hash_hostTraffic[BROADCAST_HOSTS_ENTRY] = myGlobals.broadcastEntry;
   myGlobals.broadcastEntry->next = NULL;
   myGlobals.device[deviceId].dummyDevice   = 1; /* This is basically a fake device */
   myGlobals.device[deviceId].virtualDevice = 0;
   myGlobals.device[deviceId].activeDevice  = 0;
   myGlobals.device[deviceId].samplingRate =  myGlobals.runningPref.samplingRate;
-  calculateUniqueInterfaceName(deviceId); 
+  calculateUniqueInterfaceName(deviceId);
+
+  myGlobals.device[deviceId].l7.protoTraffic = (Counter*)calloc(myGlobals.l7.numSupportedProtocols, sizeof(Counter));
 
   if(myGlobals.otherHostEntry != NULL) {
-    myGlobals.device[deviceId].hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
+    myGlobals.device[deviceId].hosts.hash_hostTraffic[OTHER_HOSTS_ENTRY] = myGlobals.otherHostEntry;
     myGlobals.otherHostEntry->next = NULL;
   }
 
-  /* Allocate memory for dhcp stats */
-  for(i=0; i<myGlobals.numKnownSubnets; i++) {
-    myGlobals.device[deviceId].networkHost[i].protocolInfo = calloc(1, sizeof(ProtocolInfo));
-    myGlobals.device[deviceId].networkHost[i].protocolInfo->dnsStats = calloc(1, sizeof(ServiceStats));
-    myGlobals.device[deviceId].networkHost[i].protocolInfo->httpStats = calloc(1, sizeof(ServiceStats));
-    myGlobals.device[deviceId].networkHost[i].protocolInfo->dhcpStats = calloc(1, sizeof(DHCPStats));
-  }
-
   return(deviceId);
 }
