@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *          Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *          Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -44,8 +44,8 @@ void welcome (FILE * fp) {
 	   myGlobals.program_name, version, sizeof(long) == 8 ? 64 : 32,
 	   configureDate, buildDate);
 
-  fprintf (fp, "Copyright 1998-2011 by %s.\n", author);
-  fprintf (fp, "Get the freshest ntop from http://www.ntop.org/\n");
+  fprintf (fp, "Copyright 1998-2012 by %s\n", ntop_author);
+  fprintf (fp, "\nGet the freshest ntop from http://www.ntop.org/\n");
 }
 
 
@@ -100,10 +100,9 @@ void usage(FILE * fp) {
   fprintf(fp, "    [-l <path>      | --pcap-log <path>]                  %sDump packets captured to a file (debug only!)\n", newLine);
   fprintf(fp, "    [-m <addresses> | --local-subnets <addresses>]        %sLocal subnetwork(s) (see man page)\n", newLine);
   fprintf(fp, "    [-n <mode>      | --numeric-ip-addresses <mode>]      %sNumeric IP addresses DNS resolution mode:\n", newLine);
-  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 0, "No DNS resolution at all");
-  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 1, "DNS resolution for local hosts only");
-  fprintf(fp, "                                                          %s  %d - %s\n", newLine, 2, "DNS resolution for remote hosts only");
-  fprintf(fp, "    [-o             | --no-mac]                           %sntop will trust just IP addresses (no MACs)\n", newLine);
+  fprintf(fp, "                                                          %s%d - %s\n", newLine, 0, "No DNS resolution at all");
+  fprintf(fp, "                                                          %s%d - %s\n", newLine, 1, "DNS resolution for local hosts only");
+  fprintf(fp, "                                                          %s%d - %s\n", newLine, 2, "DNS resolution for remote hosts only");
   fprintf(fp, "    [-p <list>      | --protocols <list>]                 %sList of IP protocols to monitor (see man page)\n", newLine);
   fprintf(fp, "    [-q             | --create-suspicious-packets]        %sCreate file ntop-suspicious-pkts.XXX.pcap file\n", newLine);
   fprintf(fp, "    [-r <number>    | --refresh-time <number>]            %sRefresh time in seconds, default is %d\n",
@@ -145,10 +144,11 @@ void usage(FILE * fp) {
 	  newLine, myGlobals.runningPref.maxNumSessions);
 
 /*  Please keep long-only options alphabetically ordered */
-
   fprintf(fp, "    [--disable-instantsessionpurge]                       %sDisable instant FIN session purge\n", newLine);
   fprintf(fp, "    [--disable-mutexextrainfo]                            %sDisable extra mutex info\n", newLine);
   fprintf(fp, "    [--disable-stopcap]                                   %sCapture packets even if there's no memory left\n", newLine);
+  fprintf(fp, "    [--disable-ndpi]                                      %sDisable nDPI for protocol discovery\n", newLine);
+  fprintf(fp, "    [--disable-python]                                    %sDisable Python interpreter\n", newLine);
 
   fprintf(fp, "    [--instance <name>]                                   %sSet log name for this ntop instance\n", newLine);
   fprintf(fp, "    [--p3p-cp]                                            %sSet return value for p3p compact policy, header\n", newLine);
@@ -577,7 +577,7 @@ int main(int argc, char *argv[]) {
 
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "ntop v.%s (%d bit)", version, sizeof(long) == 8 ? 64 : 32);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Configured on %s, built on %s.", configureDate, buildDate);
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2011 by %s", author);
+  traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Copyright 1998-2012 by %s", ntop_author);
   traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Get the freshest ntop from http://www.ntop.org/");
 
 #ifndef WIN32
@@ -637,7 +637,9 @@ int main(int argc, char *argv[]) {
   }
 
   if((ifStr == NULL) || (ifStr[0] == '\0')) {
-    traceEvent(CONST_TRACE_ERROR, "No interface has been selected. Capture not started...");
+    traceEvent(CONST_TRACE_ERROR, 
+	       "No interface has been selected [%d defined interfaces]", myGlobals.numDevices);
+    traceEvent(CONST_TRACE_NOISY, "Creating interface 'none'");
     createDummyInterface("none");
   } else
     traceEvent(CONST_TRACE_ALWAYSDISPLAY, "Listening on [%s]", ifStr);
@@ -650,26 +652,39 @@ int main(int argc, char *argv[]) {
     traceEvent(CONST_TRACE_NOISY, "Plugins started... continuing with initialization");
   }
 
-  /* ******************************* */
-
-#ifndef WIN32
-  saveNtopPid();
-#endif
-
   checkUserIdentity(userSpecified);
 
-  /* ******************************* */
-
   addDefaultAdminUser();
   readSessionPurgeParams();
   initReports();
 
+  /* Sanity checks */
+  if(myGlobals.actualReportDeviceId >= myGlobals.numDevices)
+    myGlobals.actualReportDeviceId = myGlobals.numDevices;
+
+  /*
+    In case the initial device is "none" and there are other defined devices
+    we switch to another device.
+    
+    TODO In the future we should remember the last value for 
+         myGlobals.actualReportDeviceId and use it at next start
+	 in case such device is still present
+  */
+
+  if((strcmp(myGlobals.device[myGlobals.actualReportDeviceId].name, "none") == 0)
+     && ((myGlobals.actualReportDeviceId+1) < myGlobals.numDevices))
+     myGlobals.actualReportDeviceId++;
+
   traceEvent(CONST_TRACE_NOISY, "MEMORY: Base interface structure (no hashes loaded) is %.2fMB each",
 	     xvertDOT00MB(sizeof(NtopInterface)));
   traceEvent(CONST_TRACE_NOISY, "MEMORY:     or %.2fMB for %d interfaces",
 	     xvertDOT00MB(myGlobals.numDevices*sizeof(NtopInterface)),
 	     myGlobals.numDevices);
 
+#ifndef WIN32
+  saveNtopPid();
+#endif
+
   /*
    * OK, ntop is up... if we have't failed during init, start running with the actual packet capture...
    *
