@@ -2,7 +2,7 @@
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *                          http://www.ntop.org
  *
- *            Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
+ *            Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -23,11 +23,6 @@
 
 #include "ntop.h"
 
-#ifdef HAVE_BACKTRACE
-extern size_t backtrace(void* thearray[], int thearraylen);
-extern char** backtrace_symbols(void* thearray[], int thearraylen);
-#endif
-
 static int *servicesMapper = NULL; /* temporary value */
 
 /* *************************** */
@@ -365,7 +360,7 @@ static int handleProtocolList(char* protoName, char *protocolList) {
       myGlobals.ipTrafficProtosNames = (char**)malloc(sizeof(char*));
     else
       myGlobals.ipTrafficProtosNames = (char**)realloc(myGlobals.ipTrafficProtosNames,
-						       sizeof(char*)*(myGlobals.numIpProtosToMonitor+1));
+						       (int)(sizeof(char*)*(myGlobals.numIpProtosToMonitor+1)));
 
     rc = myGlobals.numIpProtosToMonitor;
     myGlobals.ipTrafficProtosNames[myGlobals.numIpProtosToMonitor] = strdup(protoName);
@@ -376,6 +371,8 @@ static int handleProtocolList(char* protoName, char *protocolList) {
 #endif
   }
 
+  myGlobals.l7.numSupportedProtocols = IPOQUE_MAX_SUPPORTED_PROTOCOLS + myGlobals.numIpProtosToMonitor;
+
 #ifdef DEBUG
   traceEvent(CONST_TRACE_INFO, "handleProtocolList(%s) = %d", protoName, rc);
 #endif
@@ -416,7 +413,7 @@ void createPortHash(void) {
     an array to a hash table.
   */
   myGlobals.ipPortMapper.numSlots = 2*myGlobals.ipPortMapper.numElements;
-  theSize = sizeof(PortProtoMapper)*2*myGlobals.ipPortMapper.numSlots;
+  theSize = (int)(sizeof(PortProtoMapper)*2*myGlobals.ipPortMapper.numSlots);
   myGlobals.ipPortMapper.theMapper = (PortProtoMapper*)malloc(theSize);
   memset(myGlobals.ipPortMapper.theMapper, 0, theSize);
   for(i=0; i<myGlobals.ipPortMapper.numSlots; i++) myGlobals.ipPortMapper.theMapper[i].portProto = -1;
@@ -494,7 +491,7 @@ void handleProtocols(void) {
 	       tmpStr, (long)(buf.st_size+8));
 
     for (;;) {
-      bufferCurrent = fgets(bufferCurrent, buf.st_size, fd);
+      bufferCurrent = fgets(bufferCurrent, (int)buf.st_size, fd);
       /* On EOF, we're finished */
       if(bufferCurrent == NULL) {
 	break;
@@ -546,7 +543,7 @@ void handleProtocols(void) {
 
     ignore_protocol = 0;
 
-    len = strlen(proto);
+    len = (int)strlen(proto);
     for(i=0; i<len; i++) {
       if(iscntrl(proto[i]) || (!isascii(proto[i]))) {
 	ignore_protocol = 1;
@@ -565,7 +562,7 @@ void handleProtocols(void) {
       else {
 	protoName[0] = '\0';
 	strncpy(protoStr, &protoName[1], sizeof(protoStr));
-	len = strlen(protoStr);
+	len = (int)strlen(protoStr);
 
 	if(protoStr[len-1] != '|') {
 	  /* Make sure that the string ends with '|' */
@@ -591,31 +588,20 @@ void handleProtocols(void) {
 /* **************************************** */
 
 void addDefaultProtocols(void) {
-  myGlobals.FTPIdx = handleProtocolList("FTP",      "ftp|ftp-data|");
-  handleProtocolList("HTTP",     "http|www|https|3128|"); /* 3128 is HTTP cache */
   handleProtocolList("DNS",      "name|domain|");
-  handleProtocolList("Telnet",   "telnet|login|");
-  handleProtocolList("NBios/CIFS", "netbios-ns|netbios-dgm|netbios-ssn|445");
-  handleProtocolList("Mail",     "pop-2|pop-3|pop3|kpop|smtp|imap|imap2|");
-  handleProtocolList("DHCP-BOOTP", "67-68|");
+  handleProtocolList("NetBios",  "netbios-ns|netbios-dgm|netbios-ssn|445|");
+  handleProtocolList("Mail",     "pop-2|pop-3|pop3|kpop|smtp|imap|imap2|585|993|995|");
+  handleProtocolList("DHCP",     "67-68|"); /* Bootp */
   handleProtocolList("SNMP",     "snmp|snmp-trap|");
   handleProtocolList("NNTP",     "nntp|");
-  handleProtocolList("NFS/AFS",      "mount|pcnfs|bwnfs|nfsd|nfs|nfsd-status|7000-7009");
-  myGlobals.VoipIdx = handleProtocolList("VoIP", "5060|2000|54045|"); /* 54045 = Skype default port */
+  handleProtocolList("NFS/AFS",  "mount|pcnfs|bwnfs|nfsd|nfs|nfsd-status|7000-7009");
   handleProtocolList("X11",      "6000-6010|");
-  /* 22 == ssh (just to make sure the port is defined) */
-  handleProtocolList("SSH",      "22|");
-
-  /* Peer-to-Peer Protocols */
-  myGlobals.GnutellaIdx = handleProtocolList("Gnutella", "6346|6347|6348|");
-  myGlobals.KazaaIdx = handleProtocolList("Kazaa",       "1214|");
-    myGlobals.EdonkeyIdx = handleProtocolList("eDonkey", "4661-4665|");
-  myGlobals.BitTorrentIdx = handleProtocolList("BitTorrent", "6881-6999|6969|"); /* http://www.dessent.net/btfaq/#ports */
-  handleProtocolList("Messenger", "1863|5000|5001|5190-5193|");
 }
 
 /* **************************************** */
 
+/* Maps a given port to an id that represents the known protocol myGlobals.ipTrafficProtosNames[ID] */
+
 int mapGlobalToLocalIdx(int port) {
   if((port < 0) || (port >= MAX_IP_PORT))
     return(-1);
@@ -861,10 +847,13 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
         safe_strncat(buf, sizeofbuf, buf2);
       } else {
         /* Wake up thread */
-        signalCondvar(&myGlobals.queueAddressCondvar);
+        traceEvent(CONST_TRACE_INFO, "Signaling thread DNSAR%d", i+1);
+        signalCondvar(&myGlobals.queueAddressCondvar, 1);
+#if 0
         traceEvent(CONST_TRACE_INFO, "Joining thread DNSAR%d", i+1);
         if(joinThread(&myGlobals.dequeueAddressThreadId[i]) != 0)
           traceEvent(CONST_TRACE_INFO, "joinThread() returned %s", strerror(errno));
+#endif
       }
     }
   }
@@ -898,7 +887,7 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	  traceEvent(CONST_TRACE_INFO, "STATS: %s packets dropped (by ntop)",
 		     formatPkts(myGlobals.device[i].droppedPkts.value, buf2, sizeof(buf2)));
 
-	  /* signalCondvar(&myGlobals.device[i].queueCondvar); */
+	  /* signalCondvar(&myGlobals.device[i].queueCondvar, 1); */
 	  /* pcap_close(myGlobals.device[i].pcapPtr); */
 
 	  traceEvent(CONST_TRACE_INFO, "Joining thread NPS(%s) [t%lu]",
@@ -950,7 +939,7 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 	} else {
 	  traceEvent(CONST_TRACE_INFO, "Signaling thread NPA(%s)",
 		     myGlobals.device[i].humanFriendlyName);
-	  signalCondvar(&myGlobals.device[i].queueCondvar);
+	  signalCondvar(&myGlobals.device[i].queueCondvar, 1);
 	  /*
 	  traceEvent(CONST_TRACE_INFO, "Joining thread NPA(%s)",
 		     myGlobals.device[i].humanFriendlyName);
@@ -969,7 +958,7 @@ void runningThreads(char *buf, int sizeofbuf, int do_join) {
 
 /* Report statistics and write out the raw packet file */
 RETSIGTYPE cleanup(int signo) {
-  int i, j, idx;
+  int i, idx;
   char buf[128];
   static int cleanup_called = 0;
 
@@ -985,38 +974,15 @@ RETSIGTYPE cleanup(int signo) {
     exit(0);
   }
 
+  traceEvent(CONST_TRACE_INFO, "Active sessions=%u/Active hosts=%u",
+	     myGlobals.device[myGlobals.actualReportDeviceId].numSessions,
+	     myGlobals.device[myGlobals.actualReportDeviceId].hosts.hostsno);
+
 #ifndef WIN32
   signal(SIGALRM, cleanup);
-  alarm(10);
+  alarm(20);
 #endif
 
-#ifdef HAVE_BACKTRACE
-  if(signo == SIGSEGV) {
-    void *array[20];
-    size_t size;
-    char **strings;
-
-    /* Don't double fault... */
-    signal(SIGSEGV, SIG_DFL);
-
-    /* Grab the backtrace before we do much else... */
-    size = backtrace(array, 20);
-    strings = (char**)backtrace_symbols(array, size);
-
-    traceEvent(CONST_TRACE_ERROR, "BACKTRACE: *****ntop error: Signal(%d)", signo);
-
-    traceEvent(CONST_TRACE_ERROR, "BACKTRACE:     backtrace is:");
-    if(size < 2) {
-      traceEvent(CONST_TRACE_ERROR, "BACKTRACE:         **unavailable!");
-    } else {
-      /* Ignore the 0th entry, that's our cleanup() */
-      for (i=1; i<size; i++) {
-	traceEvent(CONST_TRACE_ERROR, "BACKTRACE:          %2d. %s", i, strings[i]);
-      }
-    }
-  }
-#endif /* HAVE_BACKTRACE */
-
   if(myGlobals.ntopRunState >= FLAG_NTOPSTATE_SHUTDOWN) {
     return;
   }
@@ -1063,18 +1029,11 @@ RETSIGTYPE cleanup(int signo) {
 
   for(i=0; i<myGlobals.numDevices; i++) {
     freeHostInstances(i);
-
-    while(myGlobals.device[i].fragmentList != NULL) {
-      IpFragment *fragment = myGlobals.device[i].fragmentList->next;
-      free(myGlobals.device[i].fragmentList);
-      myGlobals.device[i].fragmentList = fragment;
-    }
+    freeDeviceSessions(i);
   }
 
-  for(i=0; i<myGlobals.hostsCacheLen; i++)
-    free(myGlobals.hostsCache[i]);
-  myGlobals.hostsCacheLen = 0;
-
+  freeHostInfo(myGlobals.broadcastEntry, 0); myGlobals.broadcastEntry = NULL;
+  freeHostInfo(myGlobals.otherHostEntry, 0); myGlobals.otherHostEntry = NULL;
   unloadPlugins();
 
   (void)fflush(stdout);
@@ -1109,7 +1068,6 @@ RETSIGTYPE cleanup(int signo) {
   }
 
   termGdbm();
-  termDB();
   
   if(myGlobals.geo_ip_db != NULL) {
     GeoIP_delete(myGlobals.geo_ip_db);
@@ -1135,27 +1093,11 @@ RETSIGTYPE cleanup(int signo) {
     traceEvent(CONST_TRACE_INFO, "CLEANUP: Freeing device %s",
 	       myGlobals.device[i].humanFriendlyName);
 
-    if(myGlobals.device[i].ipTrafficMatrix != NULL) {
-      /* Courtesy of Wies-Software <wies@wiessoft.de> */
-      for(j=0; j<(myGlobals.device[i].numHosts*myGlobals.device[i].numHosts); j++)
-        if(myGlobals.device[i].ipTrafficMatrix[j] != NULL)
-	  free(myGlobals.device[i].ipTrafficMatrix[j]);
-
-      free(myGlobals.device[i].ipTrafficMatrix);
-    }
-
-    /* FIX
-    if(myGlobals.device[i].ipTrafficMatrixHosts != NULL)
-      free(myGlobals.device[i].ipTrafficMatrixHosts);
-    */
-    if(myGlobals.device[i].ipProtoStats != NULL)
-      free(myGlobals.device[i].ipProtoStats);
-
     if(myGlobals.device[i].ipProtosList != NULL)
       free(myGlobals.device[i].ipProtosList);
 
-    if(myGlobals.device[i].hash_hostTraffic != NULL)
-      free(myGlobals.device[i].hash_hostTraffic);
+    if(myGlobals.device[i].hosts.hash_hostTraffic != NULL)
+      free(myGlobals.device[i].hosts.hash_hostTraffic);
 
     if(myGlobals.device[i].ipPorts != NULL) {
       int port;
@@ -1171,7 +1113,7 @@ RETSIGTYPE cleanup(int signo) {
     if(myGlobals.device[i].packetQueue)
       free(myGlobals.device[i].packetQueue);
 
-    if(myGlobals.device[i].tcpSession) free(myGlobals.device[i].tcpSession);
+    if(myGlobals.device[i].sessions) free(myGlobals.device[i].sessions);
 
     free(myGlobals.device[i].humanFriendlyName);
     free(myGlobals.device[i].uniqueIfName);
@@ -1218,8 +1160,8 @@ RETSIGTYPE cleanup(int signo) {
   if(myGlobals.startedAs != NULL) free(myGlobals.startedAs);
 
     for(idx=0; idx<NUM_SESSION_MUTEXES; idx++) {
-      tryLockMutex(&myGlobals.tcpSessionsMutex[idx], "cleanup");
-      deleteMutex(&myGlobals.tcpSessionsMutex[idx]);
+      tryLockMutex(&myGlobals.sessionsMutex[idx], "cleanup");
+      deleteMutex(&myGlobals.sessionsMutex[idx]);
     }
 
   tryLockMutex(&myGlobals.purgePortsMutex, "cleanup");
