@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -82,7 +82,7 @@ void initWinsock32() {
     exit(-1);
   }
 
-  author  = "Luca Deri <deri@ntop.org>";
+  ntop_author  = "Luca Deri <deri@ntop.org>";
 
   if(!isWinNT()) {
     osName = "Win95/98/ME";
@@ -418,10 +418,11 @@ int waitCondvar(ConditionalVariable *condvarId) {
 
 /* ************************************ */
 
-int signalCondvar(ConditionalVariable *condvarId) {
+int signalCondvar(ConditionalVariable *condvarId, u_int8_t broadcast) {
 #ifdef DEBUG
   traceEvent(CONST_TRACE_INFO, "Signaling (%x)...", condvarId->condVar);
 #endif
+  /* FIX: handle "u_int8_t broadcast" */
   return((int)PulseEvent(condvarId->condVar));
 }
 
