@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *  			    http://www.ntop.org/
  *
@@ -21,10 +21,8 @@
 #include "ntop.h"
 
 #ifdef MAKE_STATIC_PLUGIN
-extern PluginInfo* icmpPluginEntryFctn(void);
 extern PluginInfo* sflowPluginEntryFctn(void);
 extern PluginInfo* rrdPluginEntryFctn(void);
-extern PluginInfo* cpacketPluginEntryFctn(void);
 extern PluginInfo* netflowPluginEntryFctn(void);
 #endif
 
@@ -78,16 +76,12 @@ static void loadPlugin(char* dirName, char* pluginName) {
   pluginInfo = pluginJumpFunc();
 #else /* MAKE_STATIC_PLUGIN */
 
-  if(strcmp(pluginName, "icmpPlugin") == 0)
-    pluginInfo = icmpPluginEntryFctn();
-  else if(strcmp(pluginName, "sflowPlugin") == 0)
+ if(strcmp(pluginName, "sflowPlugin") == 0)
     pluginInfo = sflowPluginEntryFctn();
   else if(strcmp(pluginName, "netflowPlugin") == 0)
     pluginInfo = netflowPluginEntryFctn();
   else if(strcmp(pluginName, "rrdPlugin") == 0)
     pluginInfo = rrdPluginEntryFctn();
-  else if(strcmp(pluginName, "cpacketPlugin") == 0)
-    pluginInfo = cpacketPluginEntryFctn();
   else
     pluginInfo = NULL;
 
@@ -239,11 +233,9 @@ void loadPlugins(void) {
 
   closedir(directoryPointer);
 #else /* MAKE_STATIC_PLUGIN */
-  loadPlugin(NULL, "icmpPlugin");
   loadPlugin(NULL, "sflowPlugin");
   loadPlugin(NULL, "netflowPlugin");
   loadPlugin(NULL, "rrdPlugin");
-  loadPlugin(NULL, "cpacketPlugin");
 #endif /* MAKE_STATIC_PLUGIN */
 }
 
