@@ -1,8 +1,9 @@
 /*
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
+ *
  *                          http://www.ntop.org
  *
- * Copyright (C) 1998-2010 Luca Deri <deri@ntop.org>
+ *             Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  * -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  *
@@ -56,28 +57,19 @@ static char __see__ []    =
    * ('P' option) and so these are processed separately.
    */
 #ifdef WIN32
-static char*  short_options = "46a:bce:f:ghi:jl:m:n:op:qr:st:w:x:zAB:C:D:F:M" 
-#ifdef ENABLE_FC
-  "N:"
-#endif
+static char*  short_options = "46a:bce:f:ghi:l:m:n:p:qr:st:w:x:zAB:C:D:F:M" 
 #if defined(DARWIN) && (!defined(TIGER))
   "v"
 #endif
   "O:P:Q:S:U:VX:W:";
 #elif defined(MAKE_WITH_SYSLOG)
-static char*  short_options = "46a:bcde:f:ghi:jl:m:n:op:qr:st:u:w:x:zAB:C:D:F:IKLM" 
-#ifdef ENABLE_FC
-  "N:"
-#endif
+static char*  short_options = "46a:bcde:f:ghi:l:m:n:p:qr:st:u:w:x:zAB:C:D:F:IKLM" 
 #if defined(DARWIN) && (!defined(TIGER))
   "v"
 #endif
   "O:P:Q:S:U:VX:W:";
 #else
-static char*  short_options = "46a:bcde:f:ghi:jl:m:n:op:qr:st:u:w:x:zAB:C:D:F:IKM"
-#ifdef ENABLE_FC
-  "N:"
-#endif
+static char*  short_options = "46a:bcde:f:ghi:l:m:n:p:qr:st:u:w:x:zAB:C:D:F:IKM"
 #if defined(DARWIN) && (!defined(TIGER))
   "v"
 #endif
@@ -100,11 +92,10 @@ static struct option const long_options[] = {
   { "track-local-hosts",                no_argument,       NULL, 'g' },
   { "help",                             no_argument,       NULL, 'h' },
   { "interface",                        required_argument, NULL, 'i' },
-  { "create-other-packets",             no_argument,       NULL, 'j' },
   { "pcap-log",                         required_argument, NULL, 'l' },
   { "local-subnets",                    required_argument, NULL, 'm' },
   { "numeric-ip-addresses",             required_argument, NULL, 'n' },
-  { "no-mac",                           no_argument,       NULL, 'o' },
+  /* 'o' is free */
 
   { "protocols",                        required_argument, NULL, 'p' },
   { "create-suspicious-packets",        no_argument,       NULL, 'q' },
@@ -134,10 +125,6 @@ static struct option const long_options[] = {
 
   { "no-interface-merge",               no_argument,       NULL, 'M' },
 
-#ifdef ENABLE_FC
-  { "wwn-map",                          required_argument, NULL, 'N' },
-#endif
-
   { "output-packet-path",               required_argument, NULL, 'O' },
   { "db-file-path",                     required_argument, NULL, 'P' },
   { "spool-file-path",                  required_argument, NULL, 'Q' },
@@ -172,19 +159,11 @@ static struct option const long_options[] = {
   { "disable-stopcap",                  no_argument,       NULL, 142 },
   { "disable-instantsessionpurge",      no_argument,       NULL, 144 },
   { "disable-mutexextrainfo",           no_argument,       NULL, 145 },
-
-#ifdef ENABLE_FC
-  { "fc-only",                          no_argument,       NULL, 147 },
-  { "no-fc",                            no_argument,       0,    148 },
-  { "no-invalid-lun",                   no_argument,       0,    149 },
-#endif
-
+  { "disable-ndpi",                     no_argument,       NULL, 146 },
+  { "disable-python",                   no_argument,       NULL, 147 },
+  
   { "skip-version-check",               required_argument, NULL, 150 },
   { "known-subnets",                    required_argument, NULL, 151 },
-  { "live",                             no_argument,       NULL, 152 },
-#ifdef ENABLE_EFFICIENCY
-  { "enable-efficiency",                no_argument,       NULL, 153 },
-#endif
 
   {NULL, 0, NULL, 0}
 };
@@ -195,9 +174,6 @@ void loadPrefs(int argc, char* argv[]) {
   datum key, nextkey;
   char buf[1024];
   int opt_index, opt;
-#ifndef WIN32
-  bool userSpecified = FALSE;
-#endif
   bool mergeInterfacesSave = myGlobals.runningPref.mergeInterfaces;
 
   memset(&buf, 0, sizeof(buf));
@@ -229,7 +205,6 @@ void loadPrefs(int argc, char* argv[]) {
 	myGlobals.groupId = pw->pw_gid;
 	endpwent();
       }
-      userSpecified = TRUE;
       break;
 #endif /* WIN32 */
 
@@ -265,9 +240,8 @@ void loadPrefs(int argc, char* argv[]) {
     /* Handle key w/o trailing \0 so valgrind is happy */
     zeroPadMallocString(key.dsize, key.dptr);
 
-    if(fetchPrefsValue(key.dptr, buf, sizeof(buf)) == 0) {
-      processNtopPref(key.dptr, buf, FALSE, &myGlobals.runningPref);
-    }
+    if(fetchPrefsValue(key.dptr, buf, sizeof(buf)) == 0)
+      processNtopPref(key.dptr, buf, FALSE, &myGlobals.runningPref);    
 
     nextkey = gdbm_nextkey (myGlobals.prefsFile, key);
     free (key.dptr);
@@ -425,10 +399,6 @@ int parseOptions(int argc, char* argv[]) {
       myGlobals.runningPref.devices = strdup(optarg);
       break;
 
-    case 'j': /* save other (unknown) packets in a file */
-      myGlobals.runningPref.enableOtherPacketDump = 1;
-      break;
-
     case 'l':
       pathSanityCheck(optarg, "-l | --pcap-log");
       myGlobals.runningPref.pcapLog = strdup(optarg);
@@ -447,10 +417,6 @@ int parseOptions(int argc, char* argv[]) {
       }
       break;
 
-    case 'o': /* Do not trust MAC addresses */
-      myGlobals.runningPref.dontTrustMACaddr = 1;
-      break;
-
     case 'p': /* the TCP/UDP protocols being monitored */
       stringSanityCheck(optarg, "-p | --protocols");
       myGlobals.runningPref.protoSpecs = strdup(optarg);
@@ -525,7 +491,8 @@ int parseOptions(int argc, char* argv[]) {
       break;
 
     case 'x':
-      myGlobals.runningPref.maxNumHashEntries = atoi(optarg);
+      //myGlobals.runningPref.maxNumHashEntries = atoi(optarg);
+      myGlobals.runningPref.maxNumHashEntries = strtoul(optarg, NULL, 0);
       break;
 
     case 'z':
@@ -574,13 +541,6 @@ int parseOptions(int argc, char* argv[]) {
       traceEvent(CONST_TRACE_ALWAYSDISPLAY, "NOTE: Interface merge disabled due to command line switch");
       break;
 
-#ifdef ENABLE_FC
-    case 'N':
-      pathSanityCheck(optarg, "-N | --wwn-map");
-      myGlobals.runningPref.fcNSCacheFile = strdup (optarg);
-      break;
-#endif
-
     case 'O': /* pcap log path - Ola Lundqvist <opal@debian.org> */
       pathSanityCheck(optarg, "-O | --output-packet-path");
        myGlobals.runningPref.pcapLogBasePath = strdup(optarg);
@@ -617,7 +577,8 @@ int parseOptions(int argc, char* argv[]) {
       exit(0);
 
     case 'X':
-      myGlobals.runningPref.maxNumSessions = atoi(optarg);
+      //myGlobals.runningPref.maxNumSessions = atoi(optarg);
+      myGlobals.runningPref.maxNumSessions = strtoul(optarg, NULL, 0);
       break;
 
 #ifdef HAVE_OPENSSL
@@ -751,25 +712,16 @@ int parseOptions(int argc, char* argv[]) {
       myGlobals.runningPref.disableStopcap = TRUE;
       break;
 
-    case 144: /* disable-instantsessionpurge */
-      myGlobals.runningPref.disableInstantSessionPurge = TRUE;
-      break;
-
     case 145: /* disable-mutexextrainfo */
       myGlobals.runningPref.disableMutexExtraInfo = TRUE;
       break;
 
+    case 146: /* disable-ndpi */
+      myGlobals.runningPref.disablenDPI = TRUE;
+      break;
+
     case 147:
-      myGlobals.runningPref.printFcOnly = TRUE;
-      myGlobals.runningPref.stickyHosts = TRUE;
-      break;
-
-    case 148:
-      myGlobals.runningPref.printIpOnly = TRUE;
-      break;
-
-    case 149:
-      myGlobals.runningPref.noInvalidLunDisplay = TRUE;
+      myGlobals.runningPref.disablePython = TRUE;
       break;
 
     case 150:
@@ -781,16 +733,6 @@ int parseOptions(int argc, char* argv[]) {
       myGlobals.runningPref.knownSubnets = strdup(optarg);
       break;
 
-    case 152:
-      myGlobals.runningPref.liveMode = 1;
-      break;
-
-#ifdef ENABLE_EFFICIENCY
-    case 153:
-      myGlobals.runningPref.calculateEfficiency = 1;
-      break;
-#endif
-
     default:
       printf("FATAL ERROR: unknown ntop option, '%c'\n", opt);
 #ifdef DEBUG
@@ -802,12 +744,6 @@ int parseOptions(int argc, char* argv[]) {
     }
   }
 
-  if(myGlobals.runningPref.liveMode) {
-    myGlobals.runningPref.trackOnlyLocalHosts   = 1;
-    myGlobals.runningPref.dontTrustMACaddr      = 1;
-    myGlobals.runningPref.enableSessionHandling = 1;
-  }
-
   /* *********************** */
 
   if(myGlobals.runningPref.daemonMode) {
@@ -912,7 +848,7 @@ static int fetchGdbmValue(GDBM_FILE gdbmfile, char *key, char *value, int valueL
   value[0] = '\0';
 
   key_data.dptr  = key;
-  key_data.dsize = strlen(key_data.dptr)+1;
+  key_data.dsize = (int)(strlen(key_data.dptr)+1);
 
   if(gdbmfile == NULL) {
 #ifdef DEBUG
@@ -964,11 +900,11 @@ static void storeGdbmValue(GDBM_FILE gdbmfile, char *key, char *value) {
 
   memset(&key_data, 0, sizeof(key_data));
   key_data.dptr   = key;
-  key_data.dsize  = strlen(key_data.dptr)+1;
+  key_data.dsize  = (int)(strlen(key_data.dptr)+1);
 
   memset(&data_data, 0, sizeof(data_data));
   data_data.dptr  = value;
-  data_data.dsize = strlen(value)+1;
+  data_data.dsize = (int)(strlen(value)+1);
 
   if(gdbmfile == NULL) {
 #ifdef DEBUG
@@ -1013,7 +949,7 @@ static void delGdbmValue(GDBM_FILE gdbmfile, char *key) {
 
   memset(&key_data, 0, sizeof(key_data));
   key_data.dptr   = key;
-  key_data.dsize  = strlen(key_data.dptr)+1;
+  key_data.dsize  = (int)(strlen(key_data.dptr)+1);
 
   if(gdbmfile == NULL) {
 #ifdef DEBUG
@@ -1111,6 +1047,22 @@ void processIntPref(char *key, char *value, int *globalVar, bool savePref)
 
 /* ******************************** */
 
+void processUIntPref(char *key, char *value, u_int *globalVar, bool savePref)
+{
+  char buf[512];
+
+  if((key == NULL) || (value == NULL)) return;
+
+  *globalVar = strtoul(value, NULL, 0);
+
+  if(savePref) {
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d", *globalVar);
+    storePrefsValue(key, buf);
+  }
+}
+
+/* ******************************** */
+
 void processBoolPref(char *key, bool value, bool *globalVar, bool savePref)
 {
   char buf[512];
@@ -1140,9 +1092,6 @@ static bool value2bool(char* value) {
 bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
   bool startCap = FALSE;
   char buf[16], *tmpStr = NULL;
-#ifdef ENABLE_FC
-  int tmpInt;
-#endif
 
   if(value == NULL) value = ""; /* Safer */
 
@@ -1265,22 +1214,6 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     }
     processIntPref(NTOP_PREF_MAXLINES, value, &pref->maxNumLines,
 		    savePref);
-#ifdef ENABLE_FC
-  } else if(strcmp(key, NTOP_PREF_PRINT_FCORIP) == 0) {
-    tmpInt = atoi (value);
-    if(tmpInt == NTOP_PREF_VALUE_PRINT_IPONLY) {
-      pref->printIpOnly = TRUE, pref->printFcOnly = FALSE;
-    } else if(tmpInt == NTOP_PREF_VALUE_PRINT_FCONLY) {
-      pref->printIpOnly = FALSE, pref->printFcOnly = TRUE;
-    } else {
-      pref->printIpOnly = FALSE, pref->printFcOnly = FALSE;
-    }
-
-    processIntPref(NTOP_PREF_PRINT_FCORIP, value, &tmpInt, savePref);
-  } else if(strcmp(key, NTOP_PREF_NO_INVLUN) == 0) {
-    processBoolPref(NTOP_PREF_NO_INVLUN, value2bool(value),
-		     &pref->noInvalidLunDisplay, savePref);
-#endif
   } else if(strcmp(key, NTOP_PREF_W3C) == 0) {
     processBoolPref(NTOP_PREF_W3C, value2bool(value), &pref->w3c, savePref);
   } else if(strcmp(key, NTOP_PREF_IPV4V6) == 0) {
@@ -1302,50 +1235,22 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     processStrPref(NTOP_PREF_P3PCP, value, &pref->P3Pcp, savePref);
   } else if(strcmp(key, NTOP_PREF_P3PURI) == 0) {
     processStrPref(NTOP_PREF_P3PURI, value, &pref->P3Puri, savePref);
-#ifdef ENABLE_FC
-  } else if(strcmp(key, NTOP_PREF_WWN_MAP) == 0) {
-    processStrPref(NTOP_PREF_WWN_MAP, value, &pref->fcNSCacheFile,
-		    savePref);
-#endif
   } else if(strcmp(key, NTOP_PREF_MAXHASH) == 0) {
     if(value == NULL) {
       safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     -1);
       value = buf;
     }
-    processIntPref(NTOP_PREF_MAXHASH, value,
-		   (int*)&pref->maxNumHashEntries, savePref);
-  } else if(strcmp(key, NTOP_PREF_SQL_DB_CONFIG) == 0) {
-    processStrPref(NTOP_PREF_SQL_DB_CONFIG, value, &tmpStr, savePref);
-    if(tmpStr != NULL) {
-      strncpy(pref->sqlDbConfig, tmpStr, sizeof(pref->sqlDbConfig));
-      free(tmpStr);      /* alloc'd in processStrPref() */
-    }
-  } else if(strcmp(key, NTOP_PREF_SQL_REC_LIFETIME) == 0) {
-    if(value == NULL) {
-      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), 
-		    "%d",
-		    -1);
-      value = buf;
-    }
-    processIntPref(NTOP_PREF_SQL_REC_LIFETIME, value,
-		   (int*)&pref->sqlRecDaysLifetime, savePref);
-  } else if(strcmp(key, NTOP_PREF_SAVE_REC_INTO_DB) == 0) {
-    processBoolPref(NTOP_PREF_SAVE_REC_INTO_DB, value2bool(value),
-		    &pref->saveRecordsIntoDb, savePref);
-    myGlobals.runningPref.saveRecordsIntoDb = pref->saveRecordsIntoDb; /* activate immediately */
-  } else if(strcmp(key, NTOP_PREF_SAVE_SESSIONS_INTO_DB) == 0) {
-    processBoolPref(NTOP_PREF_SAVE_SESSIONS_INTO_DB, value2bool(value),
-		    &pref->saveSessionsIntoDb, savePref);   
-    myGlobals.runningPref.saveSessionsIntoDb = pref->saveSessionsIntoDb; /* activate immediately */
+    processUIntPref(NTOP_PREF_MAXHASH, value,
+		   (u_int*)&pref->maxNumHashEntries, savePref);
   } else if(strcmp(key, NTOP_PREF_MAXSESSIONS) == 0) {
     if(value == NULL) {
       safe_snprintf (__FILE__, __LINE__, buf, sizeof(buf), "%d",
 		     -1);
       value = buf;
     }
-    processIntPref(NTOP_PREF_MAXSESSIONS, value,
-		    (int*)&pref->maxNumSessions, savePref);
+    processUIntPref(NTOP_PREF_MAXSESSIONS, value,
+		   (u_int*)&pref->maxNumSessions, savePref);
   } else if(strcmp(key, NTOP_PREF_MERGEIF) == 0) {
     processBoolPref(NTOP_PREF_MERGEIF, value2bool(value),
 		     &pref->mergeInterfaces, savePref);
@@ -1355,12 +1260,6 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
   } else if(strcmp(key, NTOP_PREF_ENABLE_L7PROTO) == 0) {
     processBoolPref(NTOP_PREF_ENABLE_L7PROTO, value2bool(value),
 		     &pref->enableL7, savePref);
-  } else if(strcmp(key, NTOP_PREF_NO_ISESS_PURGE) == 0) {
-    processBoolPref(NTOP_PREF_NO_ISESS_PURGE, value2bool(value),
-		     &pref->disableInstantSessionPurge, savePref);
-  } else if(strcmp(key, NTOP_PREF_NO_TRUST_MAC) == 0) {
-    processBoolPref(NTOP_PREF_NO_TRUST_MAC, value2bool(value),
-		     &pref->dontTrustMACaddr, savePref);
   } else if(strcmp(key, NTOP_PREF_PCAP_LOGBASE) == 0) {
     processStrPref(NTOP_PREF_PCAP_LOGBASE, value,
 		    &pref->pcapLogBasePath, savePref);
@@ -1376,9 +1275,6 @@ bool processNtopPref(char *key, char *value, bool savePref, UserPref *pref) {
     }
     processIntPref(NTOP_PREF_TRACE_LVL, value, &pref->traceLevel,
 		    savePref);
-  } else if(strcmp(key, NTOP_PREF_DUMP_OTHER) == 0) {
-    processBoolPref(NTOP_PREF_DUMP_OTHER, value2bool(value),
-		     &pref->enableOtherPacketDump, savePref);
   } else if(strcmp(key, NTOP_PREF_DUMP_SUSP) == 0) {
     processBoolPref(NTOP_PREF_DUMP_SUSP, value2bool(value),
 		     &pref->enableSuspiciousPacketDump, savePref);
@@ -1424,11 +1320,9 @@ void initUserPrefs(UserPref *pref) {
   pref->daemonMode = DEFAULT_NTOP_DAEMON_MODE;
   pref->trackOnlyLocalHosts    = DEFAULT_NTOP_TRACK_ONLY_LOCAL;
   pref->devices = DEFAULT_NTOP_DEVICES;
-  pref->enableOtherPacketDump = DEFAULT_NTOP_OTHER_PKT_DUMP;
   pref->pcapLog = DEFAULT_NTOP_PCAP_LOG_FILENAME;
   pref->localAddresses = DEFAULT_NTOP_LOCAL_SUBNETS;
   pref->numericFlag = DEFAULT_NTOP_NUMERIC_IP_ADDRESSES;
-  pref->dontTrustMACaddr = DEFAULT_NTOP_DONT_TRUST_MAC_ADDR;
   pref->protoSpecs = DEFAULT_NTOP_PROTO_SPECS;
   pref->enableSuspiciousPacketDump = DEFAULT_NTOP_SUSPICIOUS_PKT_DUMP;
   pref->refreshRate = DEFAULT_NTOP_AUTOREFRESH_INTERVAL;
@@ -1436,10 +1330,6 @@ void initUserPrefs(UserPref *pref) {
   pref->traceLevel = DEFAULT_TRACE_LEVEL;
   pref->maxNumHashEntries  = DEFAULT_NTOP_MAX_HASH_ENTRIES;
   pref->maxNumSessions     = DEFAULT_NTOP_MAX_NUM_SESSIONS;
-  strncpy(pref->sqlDbConfig, DEFAULT_NTOP_SQL_DB_CONFIG, sizeof(pref->sqlDbConfig));
-  pref->sqlRecDaysLifetime = DEFAULT_NTOP_SQL_REC_DAYS_LIFETIME;
-  pref->saveRecordsIntoDb  = DEFAULT_NTOP_SAVE_REC_INTO_DB;
-  pref->saveSessionsIntoDb = DEFAULT_NTOP_SAVE_SESSIONS_INTO_DB;
   pref->webAddr = DEFAULT_NTOP_WEB_ADDR;
   pref->webPort = DEFAULT_NTOP_WEB_PORT;
   pref->ipv4or6 = DEFAULT_NTOP_FAMILY;
@@ -1457,9 +1347,6 @@ void initUserPrefs(UserPref *pref) {
 					      * break the logic */
 #else
   pref->pcapLogBasePath = strdup(CFG_DBFILE_DIR);
-#endif
-#ifdef ENABLE_FC
-  pref->fcNSCacheFile   = DEFAULT_NTOP_FCNS_FILE;
 #endif
   /* note that by default ntop will merge network interfaces */
   pref->mapperURL = DEFAULT_NTOP_MAPPER_URL;
@@ -1473,10 +1360,6 @@ void initUserPrefs(UserPref *pref) {
    pref->P3Puri = DEFAULT_NTOP_P3PURI;
 
    pref->disableStopcap = DEFAULT_NTOP_DISABLE_STOPCAP;
-   pref->disableInstantSessionPurge = DEFAULT_NTOP_DISABLE_IS_PURGE;
-   pref->printIpOnly = DEFAULT_NTOP_PRINTIPONLY;
-   pref->printFcOnly = DEFAULT_NTOP_PRINTFCONLY;
-   pref->noInvalidLunDisplay   = DEFAULT_NTOP_NO_INVLUN_DISPLAY;
    pref->disableMutexExtraInfo = DEFAULT_NTOP_DISABLE_MUTEXINFO;
    pref->skipVersionCheck      = DEFAULT_NTOP_SKIP_VERSION_CHECK;
 }
