@@ -3,7 +3,7 @@
  *
  *                          http://www.ntop.org
  *
- *             Copyright (C) 2010 Luca Deri <deri@ntop.org>
+ *             Copyright (C) 2012 Luca Deri <deri@ntop.org>
  *                                Daniele Sgandurra <sgandurra@ntop.org>
  *                                Jaime Blasco <jaime.blasco@alienvault.com>
  *                                Gianluca Medici <gmedici@ntop.org>
@@ -32,10 +32,6 @@
 
 #ifdef HAVE_PYTHON
 
-#ifdef HAVE_FASTBIT
-#include <capi.h>
-#endif
-
 static HostTraffic *ntop_host = NULL;
 static char query_string[2048];
 static PthreadMutex python_mutex;
@@ -256,94 +252,91 @@ static PyObject* python_updateRRDGauge(PyObject *self, PyObject *args) {
 
 /* **************************************** */
 
-
 static PyObject* python_rrd_fetch(PyObject *self, PyObject *args) {
-	PyObject *r;
-	rrd_value_t *data, *datai;
-	unsigned long step, ds_cnt;
-	time_t    start, end;
-	int       argc=0, rc=0;
-	char     **ds_namv;
-	char *argv[7], *pPathFilename, *pFunction, *pStart, *pEnd;
+  PyObject *r;
+  rrd_value_t *data, *datai;
+  unsigned long step, ds_cnt;
+  time_t    start, end;
+  int       argc=0, rc=0;
+  char     **ds_namv;
+  char *argv[7], *pPathFilename, *pFunction, *pStart, *pEnd;
 
-	/* parse the incoming arguments */
-	  if(!PyArg_ParseTuple(args, "ssss", &pPathFilename, &pFunction, &pStart, &pEnd))
-	    return NULL;
+  /* parse the incoming arguments */
+  if(!PyArg_ParseTuple(args, "ssss", &pPathFilename, &pFunction, &pStart, &pEnd))
+    return NULL;
 
-    argv[argc++] = "rrd_fetch";
-    argv[argc++] = pPathFilename;
-    argv[argc++] = pFunction;
-    argv[argc++] = "--start";
-    argv[argc++] = pStart;
-    argv[argc++] = "--end";
-    argv[argc++] = pEnd;
+  argv[argc++] = "rrd_fetch";
+  argv[argc++] = pPathFilename;
+  argv[argc++] = pFunction;
+  argv[argc++] = "--start";
+  argv[argc++] = pStart;
+  argv[argc++] = "--end";
+  argv[argc++] = pEnd;
 
 
-    optind=0; /* reset gnu getopt */
-    opterr=0; /* no error messages */
+  optind=0; /* reset gnu getopt */
+  opterr=0; /* no error messages */
+  rrd_clear_error();
+
+  rc = rrd_fetch(argc, argv, &start, &end, &step, &ds_cnt, &ds_namv, &data);
+
+  //Inspired by rrdtoolmodule.c (pyrrdtool) from Hye-Shik Chang <perky@fallin.lv>
+  if (rc == -1) {
+    //traceEvent(CONST_TRACE_ERROR, "%s - (%s) <%s-%s-%s-%s> ", "python_RRD_fetch", rrd_get_error(),pPathFilename, pFunction, pStart, pEnd);//TODO
+    PyErr_SetString(PyErr_NewException("rrdtool.error", NULL, NULL), rrd_get_error());
     rrd_clear_error();
+    r = NULL;
+  } else {
+    /* Return :
+       ((start, end, step), (name1, name2, ...), [(data1, data2, ..), ...]) */
+    PyObject *range_tup, *dsnam_tup, *data_list, *t;
+    unsigned long i, j, row;
+    rrd_value_t dv;
 
-    rc = rrd_fetch(argc, argv, &start, &end, &step, &ds_cnt, &ds_namv, &data);
+    row = (unsigned long)((end - start) / step);
 
-	//Inspired by rrdtoolmodule.c (pyrrdtool) from Hye-Shik Chang <perky@fallin.lv>
-	if (rc == -1) {
-		//traceEvent(CONST_TRACE_ERROR, "%s - (%s) <%s-%s-%s-%s> ", "python_RRD_fetch", rrd_get_error(),pPathFilename, pFunction, pStart, pEnd);//TODO
-		PyErr_SetString(PyErr_NewException("rrdtool.error", NULL, NULL), rrd_get_error());
-		rrd_clear_error();
-		r = NULL;
+    r = PyTuple_New(3);
+    range_tup = PyTuple_New(3);
+    dsnam_tup = PyTuple_New(ds_cnt);
+    data_list = PyList_New(row);
+    PyTuple_SET_ITEM(r, 0, range_tup);
+    PyTuple_SET_ITEM(r, 1, dsnam_tup);
+    PyTuple_SET_ITEM(r, 2, data_list);
+
+    datai = data;
+
+    PyTuple_SET_ITEM(range_tup, 0, PyInt_FromLong((long) start));
+    PyTuple_SET_ITEM(range_tup, 1, PyInt_FromLong((long) end));
+    PyTuple_SET_ITEM(range_tup, 2, PyInt_FromLong((long) step));
+
+    for (i = 0; i < ds_cnt; i++)
+      PyTuple_SET_ITEM(dsnam_tup, i, PyString_FromString(ds_namv[i]));
+
+    for (i = 0; i < row; i++) {
+      t = PyTuple_New(ds_cnt);
+      PyList_SET_ITEM(data_list, i, t);
+
+      for (j = 0; j < ds_cnt; j++) {
+	dv = *(datai++);
+	if (isnan(dv)) {
+	  PyTuple_SET_ITEM(t, j, Py_None);
+	  Py_INCREF(Py_None);
 	} else {
-		/* Return :
-		   ((start, end, step), (name1, name2, ...), [(data1, data2, ..), ...]) */
-		PyObject *range_tup, *dsnam_tup, *data_list, *t;
-		unsigned long i, j, row;
-		rrd_value_t dv;
-
-		row = (end - start) / step;
-
-		r = PyTuple_New(3);
-		range_tup = PyTuple_New(3);
-		dsnam_tup = PyTuple_New(ds_cnt);
-		data_list = PyList_New(row);
-		PyTuple_SET_ITEM(r, 0, range_tup);
-		PyTuple_SET_ITEM(r, 1, dsnam_tup);
-		PyTuple_SET_ITEM(r, 2, data_list);
-
-		datai = data;
-
-		PyTuple_SET_ITEM(range_tup, 0, PyInt_FromLong((long) start));
-		PyTuple_SET_ITEM(range_tup, 1, PyInt_FromLong((long) end));
-		PyTuple_SET_ITEM(range_tup, 2, PyInt_FromLong((long) step));
-
-		for (i = 0; i < ds_cnt; i++)
-			PyTuple_SET_ITEM(dsnam_tup, i, PyString_FromString(ds_namv[i]));
-
-		for (i = 0; i < row; i++) {
-			t = PyTuple_New(ds_cnt);
-			PyList_SET_ITEM(data_list, i, t);
-
-			for (j = 0; j < ds_cnt; j++) {
-				dv = *(datai++);
-				if (isnan(dv)) {
-					PyTuple_SET_ITEM(t, j, Py_None);
-					Py_INCREF(Py_None);
-				} else {
-					PyTuple_SET_ITEM(t, j, PyFloat_FromDouble((double) dv));
-				}
-			}
-		}
-
-		for (i = 0; i < ds_cnt; i++){
-			rrd_freemem(ds_namv[i]);
-		}
-		rrd_freemem(ds_namv);
-		rrd_freemem(data);
+	  PyTuple_SET_ITEM(t, j, PyFloat_FromDouble((double) dv));
 	}
+      }
+    }
 
+    for (i = 0; i < ds_cnt; i++){
+      rrd_freemem(ds_namv[i]);
+    }
+    rrd_freemem(ds_namv);
+    rrd_freemem(data);
+  }
 
-	return r;
+  return r;
 }
 
-
 /* **************************************** */
 
 static PyObject* python_ntopVersion(PyObject *self, PyObject *args) {
@@ -574,6 +567,20 @@ static PyObject* python_isPOPhost(PyObject *self,
 
 /* **************************************** */
 
+static PyObject* python_isFacebookClient(PyObject *self,
+					 PyObject *args) {
+  return PyBool_FromLong(isFacebookClient(ntop_host));
+}
+
+/* **************************************** */
+
+static PyObject* python_isTwitterClient(PyObject *self,
+					PyObject *args) {
+  return PyBool_FromLong(isTwitterClient(ntop_host));
+}
+
+/* **************************************** */
+
 static PyObject* python_isIMAPhost(PyObject *self,
 				   PyObject *args) {
   //traceEvent(CONST_TRACE_WARNING, "-%s-", "python_ipAddress");
@@ -707,11 +714,11 @@ static PyObject* python_fingerprint(PyObject *self, PyObject *args) {
 /* **************************************** */
 
 static PyObject* python_pktsSent(PyObject *self, PyObject *args) {
-  return PyLong_FromUnsignedLong((unsigned long)(unsigned long)(ntop_host->pktSent.value));
+  return PyLong_FromUnsignedLong((unsigned long)(unsigned long)(ntop_host->pktsSent.value));
 }
 
 static PyObject* python_pktsRcvd(PyObject *self, PyObject *args) {
-  return PyLong_FromUnsignedLong((unsigned long)(unsigned long)(ntop_host->pktRcvd.value));
+  return PyLong_FromUnsignedLong((unsigned long)(unsigned long)(ntop_host->pktsRcvd.value));
 }
 
 /* **************************************** */
@@ -754,70 +761,86 @@ static PyObject* python_receiveThpt(PyObject *self, PyObject *args) {
 
 /* **************************************** */
 
-#ifdef HAVE_FASTBIT
-static PyObject* python_fastbit_query(PyObject *self, PyObject *args) {
-  PyObject *obj = NULL;
-  char *select_clause, *where, *partition;
-  int limit, nres;
-  FastBitQueryHandle qh;
-  FastBitResultSetHandle rh;
+static PyObject* python_dumpHostRawFlows(PyObject *self, PyObject *args) {
+  PyObject *ret = PyList_New(0);
+  int found = 0, idx;
+  HostTraffic *el = NULL;
+  char buf[LEN_GENERAL_WORK_BUFFER/2], *host;
 
-  if(!PyArg_ParseTuple(args, "sssi", &partition, &select_clause, &where, &limit)) return NULL;
+  /* ****************************** */
 
-  qh = fastbit_build_query(select_clause, partition, where);
-  if(qh == NULL) {
-    traceEvent(CONST_TRACE_WARNING, "Error while executing SELECT %s FROM %s WHERE %s",
-	       select_clause, partition, where);
-    return NULL;
-  }
+  if( (!PyArg_ParseTuple(args, "s", &host)) && 
+      (!PyArg_ParseTuple(args, "i", &host)) ) return(ret);
 
-  if((nres = fastbit_get_result_rows(qh)) > 0) {
-    rh = fastbit_build_result_set(qh);
-    if(rh != NULL) {
-      int ncols = fastbit_get_result_columns(qh), n;
-      PyObject *list;
-      char *header = strdup((char*)fastbit_get_select_clause(qh)), *elem, *state = NULL;
+  if(host == NULL) return(ret);
 
-      obj = PyDict_New();
-      elem = strtok_r(header, ",", &state);
-      list = PyList_New(ncols), n = 0;
+  for(el = getFirstHost(myGlobals.actualReportDeviceId); 
+      el != NULL; el = getNextHost(myGlobals.actualReportDeviceId, el)) {
+    if ((strcmp(el->hostNumIpAddress, host) == 0) ||
+	(strcmp(el->hostResolvedName, host) == 0)) {
+      found = 1;
+      break;
+    }
+  } /* for */
 
-      while(elem != NULL) {
-	PyList_SetItem(list, n++, PyString_FromString(elem));
-	elem = strtok_r(NULL, ",", &state);
-      }
-      PyDict_SetItem(obj, PyString_FromString("columns"), list);
+  if(!found) return(ret);
 
-      if(nres > limit) nres = limit;
-      list = PyList_New(nres);
+  for(idx=0; idx<MAX_TOT_NUM_SESSIONS; idx++) {
+    int mutex_idx;
 
-      if(list != NULL) {
-	n = 0;
+    mutex_idx = idx % NUM_SESSION_MUTEXES;
+    accessMutex(&myGlobals.sessionsMutex[mutex_idx], "dumpHostFlowsRaw");
 
-	while((nres > 0) && (fastbit_result_set_next(rh) == 0)) {
-	  int i;
-	  PyObject *list_elem = PyList_New(ncols);
+    if(myGlobals.device[myGlobals.actualReportDeviceId].sessions[idx] != NULL) {
+      IPSession *session = myGlobals.device[myGlobals.actualReportDeviceId].sessions[idx];
 
-	  if(!list_elem) break;
-
-	  for (i = 0; i < ncols; ++ i)
-	    PyList_SetItem(list_elem, i, PyInt_FromLong(fastbit_result_set_getUnsigned(rh, i)));
-
-	  PyList_SetItem(list, n++, list_elem);
-	  nres--;
+      while(session != NULL) {
+	char *symProto, *l7Proto;
+	
+	if((session->initiator->magic != CONST_MAGIC_NUMBER)
+	   || (session->remotePeer->magic != CONST_MAGIC_NUMBER)) {
+#if 0
+	  traceEvent(CONST_TRACE_WARNING, "Session with expired peer (%d/%d)",
+		     session->initiator->magic, session->remotePeer->magic);
+#endif
+	  session = session->next;
+	  continue;
 	}
 
-	PyDict_SetItem(obj, PyString_FromString("values"), list);
-      }
+#ifndef PARM_PRINT_ALL_SESSIONS
+	if(session->sessionState != FLAG_STATE_ACTIVE) {
+	  session = session->next;
+	  continue;
+	}
+#endif
 
-      fastbit_destroy_result_set(rh);
+	if(el && (session->initiator != el) && (session->remotePeer != el)) {
+	  session = session->next;
+	  continue;
+	}
+
+	symProto = proto2name(session->proto);
+	l7Proto = getProtoName(session->proto, session->l7.major_proto);
+
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
+		      "%s|%s|%llu|%llu|%s|%s\n",
+		      (session->initiator->hostResolvedName[0] != '\0') ? session->initiator->hostResolvedName :session->initiator->hostNumIpAddress, 
+		      (session->remotePeer->hostNumIpAddress[0] != '\0') ? session->remotePeer->hostResolvedName : session->remotePeer->hostNumIpAddress,
+		      session->bytesSent.value, session->bytesRcvd.value, symProto, l7Proto);
+
+	PyList_Append(ret, PyString_FromString(buf));
+
+	session = session->next;
+      }
     }
+
+    releaseMutex(&myGlobals.sessionsMutex[mutex_idx]);
   }
 
-  return obj;
+  return(ret);
 }
 
-#endif
+/* **************************************** */
 
 /* **************************************** */
 
@@ -899,8 +922,7 @@ static PyObject* python_interface_network(PyObject *self, PyObject *args) {
   if(interfaceId >= myGlobals.numDevices) return NULL;
   return PyString_FromFormat("%s/%s",
 			     _intoa(myGlobals.device[interfaceId].network, buf, sizeof(buf)),
-			     _intoa(myGlobals.device[interfaceId].netmask, buf1, sizeof(buf1))
-			     );
+			     _intoa(myGlobals.device[interfaceId].netmask, buf1, sizeof(buf1)));
 }
 
 static PyObject* python_interface_numHosts(PyObject *self, PyObject *args) {
@@ -912,20 +934,26 @@ static PyObject* python_interface_numHosts(PyObject *self, PyObject *args) {
   return PyLong_FromUnsignedLong(myGlobals.device[interfaceId].numHosts);
 }
 
-#ifdef INET6
+
 static PyObject* python_interface_ipv6Address(PyObject *self, PyObject *args) {
   u_int interfaceId;
+#ifdef INET6
   char buf[64];
+#endif
 
   if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
   if(interfaceId >= myGlobals.numDevices) return NULL;
+
+#ifdef INET6
   if(myGlobals.device[interfaceId].v6Addrs == NULL) return PyString_FromString("");
 
   return PyString_FromFormat("%s/%d",
 			     _intop(&myGlobals.device[interfaceId].v6Addrs->af.inet6.ifAddr, buf, sizeof(buf)),
 			     myGlobals.device[interfaceId].v6Addrs->af.inet6.prefixlen);
-}
+#else
+  return NULL;
 #endif
+}
 
 static PyObject* python_interface_time(PyObject *self, PyObject *args) {
   u_int interfaceId;
@@ -997,50 +1025,6 @@ static PyObject* python_interface_pktsStats(PyObject *self, PyObject *args) {
   return obj;
 }
 
-#ifdef ENABLE_FC
-
-static PyObject* python_interface_fcPktsStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-  PyObject *obj;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  if((obj = PyDict_New()) == NULL) return NULL;
-
-  PyDict_SetItem(obj, PyString_FromString("total"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].fcPkts.value));
-  PyDict_SetItem(obj, PyString_FromString("eofa"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].fcEofaPkts.value));
-  PyDict_SetItem(obj, PyString_FromString("eofAbnormal"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].fcEofAbnormalPkts.value));
-  PyDict_SetItem(obj, PyString_FromString("abnormal"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].fcAbnormalPkts.value));
-  PyDict_SetItem(obj, PyString_FromString("broadcast"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].fcBroadcastPkts.value));
-  return obj;
-}
-
-static PyObject* python_interface_fcBytesStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-  PyObject *obj;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  if((obj = PyDict_New()) == NULL) return NULL;
-
-  PyDict_SetItem(obj, PyString_FromString("total"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("fragmented"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fragmentedFcBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("fcp"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcFcpBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("ficon"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcFiconBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("ip"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcIpfcBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("swils"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcSwilsBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("dns"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcDnsBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("els"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcElsBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("other"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].otherFcBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("broadcast"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].fcBroadcastBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("class2"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].class2Bytes.value));
-  PyDict_SetItem(obj, PyString_FromString("class3"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].class3Bytes.value));
-  PyDict_SetItem(obj, PyString_FromString("classFb"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].classFBytes.value));
-  return obj;
-}
-
-#endif
-
 static PyObject* python_interface_bytesStats(PyObject *self, PyObject *args) {
   u_int interfaceId;
   PyObject *obj;
@@ -1056,14 +1040,10 @@ static PyObject* python_interface_bytesStats(PyObject *self, PyObject *args) {
   PyDict_SetItem(obj, PyString_FromString("udp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].udpBytes.value));
   PyDict_SetItem(obj, PyString_FromString("otherIp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].otherIpBytes.value));
   PyDict_SetItem(obj, PyString_FromString("icmp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].icmpBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("dlc"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].dlcBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("ipx"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].ipxBytes.value));
   PyDict_SetItem(obj, PyString_FromString("stp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].stpBytes.value));
   PyDict_SetItem(obj, PyString_FromString("ipsec"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].ipsecBytes.value));
   PyDict_SetItem(obj, PyString_FromString("netbios"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].netbiosBytes.value));
   PyDict_SetItem(obj, PyString_FromString("arp_rarp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].arpRarpBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("appletalk"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].atalkBytes.value));
-  PyDict_SetItem(obj, PyString_FromString("egp"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].egpBytes.value));
   PyDict_SetItem(obj, PyString_FromString("gre"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].greBytes.value));
   PyDict_SetItem(obj, PyString_FromString("ipv6"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].ipv6Bytes.value));
   PyDict_SetItem(obj, PyString_FromString("icmp6"), PyLong_FromUnsignedLong((unsigned long)myGlobals.device[interfaceId].icmp6Bytes.value));
@@ -1090,52 +1070,6 @@ static PyObject* python_interface_throughputStats(PyObject *self, PyObject *args
   return(obj);
 }
 
-static PyObject* python_interface_SimpleProtoTrafficInfo(SimpleProtoTrafficInfo *info) {
-  PyObject *obj;
-
-  if((obj = PyDict_New()) == NULL) return NULL;
-
-  PyDict_SetItem(obj, PyString_FromString("local"), PyLong_FromUnsignedLong((unsigned long)info->local.value));
-  PyDict_SetItem(obj, PyString_FromString("local2remote"), PyLong_FromUnsignedLong((unsigned long)info->local2remote.value));
-  PyDict_SetItem(obj, PyString_FromString("remote"), PyLong_FromUnsignedLong((unsigned long)info->remote.value));
-  PyDict_SetItem(obj, PyString_FromString("remote2local"), PyLong_FromUnsignedLong((unsigned long)info->remote2local.value));
-
-  return(obj);
-}
-
-static PyObject* python_interface_tcpStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  return(python_interface_SimpleProtoTrafficInfo(&myGlobals.device[interfaceId].tcpGlobalTrafficStats));
-}
-
-static PyObject* python_interface_udpStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  return(python_interface_SimpleProtoTrafficInfo(&myGlobals.device[interfaceId].udpGlobalTrafficStats));
-}
-
-static PyObject* python_interface_icmpStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  return(python_interface_SimpleProtoTrafficInfo(&myGlobals.device[interfaceId].icmpGlobalTrafficStats));
-}
-
-static PyObject* python_interface_ipStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  if(myGlobals.device[interfaceId].ipProtoStats == NULL) return NULL;
-  return(python_interface_SimpleProtoTrafficInfo(myGlobals.device[interfaceId].ipProtoStats));
-}
-
 static PyObject* python_interface_securityPkts(PyObject *self, PyObject *args) {
   u_int interfaceId;
   PyObject *obj;
@@ -1218,21 +1152,6 @@ static PyObject* python_interface_sflowStats(PyObject *self, PyObject *args) {
   return(obj);
 }
 
-
-static PyObject* python_interface_cpacketStats(PyObject *self, PyObject *args) {
-  u_int interfaceId;
-  PyObject *obj;
-
-  if(!PyArg_ParseTuple(args, "i", &interfaceId)) return NULL;
-  if(interfaceId >= myGlobals.numDevices) return NULL;
-  if(myGlobals.device[interfaceId].cpacketGlobals == NULL) return PyDict_New();
-  if((obj = PyDict_New()) == NULL) return NULL;
-
-  PyDict_SetItem(obj, PyString_FromString("total"), PyLong_FromUnsignedLong(myGlobals.device[interfaceId].cpacketGlobals->statsProcessed));
-
-  return(obj);
-}
-
 /* **************************************** */
 
 static PyMethodDef ntop_methods[] = {
@@ -1275,29 +1194,19 @@ static PyMethodDef interface_methods[] = {
   { "ipv4", python_interface_ipv4Address, METH_VARARGS, "Get interface address (IPv4)" },
   { "network", python_interface_network, METH_VARARGS, "Get network and mask to which the interface belongs" },
   { "numHosts", python_interface_numHosts, METH_VARARGS, "Get the number of hosts active on this interface" },
-#ifdef INET6
   { "ipv6", python_interface_ipv6Address, METH_VARARGS, "Get interface address (IPv6)" },
-#endif
   { "time", python_interface_time, METH_VARARGS, "Get interface time" },
   { "virtual", python_interface_virtual, METH_VARARGS, "Check if this is a virtual interface" },
   { "speed", python_interface_speed, METH_VARARGS, "Interface speed (0 if unknown)" },
   { "mtu", python_interface_mtu, METH_VARARGS, "Get interface MTU size" },
   { "bpf", python_interface_bpf, METH_VARARGS, "Get BPF filter set for this interface (if any)" },
   { "pktsStats", python_interface_pktsStats, METH_VARARGS, "Get packet statistics " },
-#ifdef ENABLE_FC
-  { "fcPktsStats", python_interface_fcPktsStats, METH_VARARGS, "Get FC pkts stats" },
-  { "fcBytesStats", python_interface_fcBytesStats, METH_VARARGS, "Get FC byte stats" },
-#endif
   { "bytesStats", python_interface_bytesStats, METH_VARARGS, "Get bytes statistics" },
   { "throughputStats", python_interface_throughputStats, METH_VARARGS, "" },
-  { "tcpStats", python_interface_tcpStats, METH_VARARGS, "Get TCP stats" },
-  { "udpStats", python_interface_udpStats, METH_VARARGS, "Get UDP stats" },
-  { "icmpStats", python_interface_icmpStats, METH_VARARGS, "Get ICMP stats" },
-  { "ipStats", python_interface_ipStats, METH_VARARGS, "Get IP stats" },
   { "securityPkts", python_interface_securityPkts, METH_VARARGS, "Get information about security packets" },
   { "netflowStats", python_interface_netflowStats, METH_VARARGS, "Get NetFlow interface information" },
   { "sflowStats", python_interface_sflowStats, METH_VARARGS, "Get sFlow interface information" },
-  { "cpacketStats", python_interface_cpacketStats, METH_VARARGS, "Get cPacket counter information" },
+  { "dumpHostRawFlows", python_dumpHostRawFlows, METH_VARARGS, "Get Raw Flows from host" },
   { NULL, NULL, 0, NULL }
 };
 
@@ -1323,6 +1232,8 @@ static PyMethodDef host_methods[] = {
   { "isPrinter",  python_isPrinter, METH_NOARGS, "Check isPrinter Host" },
   { "isSMTPhost",  python_isSMTPhost, METH_NOARGS, "Check isSMTPhost Host" },
   { "isPOPhost",  python_isPOPhost, METH_NOARGS, "Check isPOPhost Host" },
+  { "isFacebookClient",  python_isFacebookClient, METH_NOARGS, "Check isFacebookClient Host" },
+  { "isTwitterClient",  python_isTwitterClient, METH_NOARGS, "Check isTwitterClient Host" },
   { "isIMAPhost",  python_isIMAPhost, METH_NOARGS, "Check isIMAPhost Host" },
   { "isDirectoryHost",  python_isDirectoryHost, METH_NOARGS, "Check isDirectoryHost Host" },
   { "isHTTPhost",  python_isHTTPhost, METH_NOARGS, "Check isHTTPhost Host" },
@@ -1353,15 +1264,6 @@ static PyMethodDef host_methods[] = {
 
 /* **************************************** */
 
-#ifdef HAVE_FASTBIT
-static PyMethodDef fastbit_methods[] = {
-  { "query", python_fastbit_query, METH_VARARGS, "Exec a fastbit query using ntop" },
-  { NULL, NULL, 0, NULL }
-};
-#endif
-
-/* **************************************** */
-
 #if PY_MAJOR_VERSION >= 3
 
 struct module_state { PyObject *error; };
@@ -1373,9 +1275,6 @@ static int myextension_clear(PyObject *m) { Py_CLEAR(GETSTATE(m)->error); return
 static struct PyModuleDef _ntop_methods = { PyModuleDef_HEAD_INIT, "ntop", NULL, sizeof(struct module_state), ntop_methods, NULL, myextension_traverse, myextension_clear, NULL };
 static struct PyModuleDef _interface_methods = { PyModuleDef_HEAD_INIT, "interface", NULL, sizeof(struct module_state), interface_methods, NULL, myextension_traverse, myextension_clear, NULL };
 static struct PyModuleDef _host_methods = { PyModuleDef_HEAD_INIT, "host", NULL, sizeof(struct module_state), host_methods, NULL, myextension_traverse, myextension_clear, NULL };
-#ifdef HAVE_FASTBIT
-static struct PyModuleDef _fastbit_methods = { PyModuleDef_HEAD_INIT, "fastbit", NULL, sizeof(struct module_state), fastbit_methods, NULL, myextension_traverse, myextension_clear, NULL };
-#endif
 #endif
 
 /* **************************************** */
@@ -1385,9 +1284,6 @@ static void init_python_ntop(void) {
   Py_InitModule("ntop", ntop_methods);
   Py_InitModule("interface", interface_methods);
   Py_InitModule("host", host_methods);
-#ifdef HAVE_FASTBIT
-  Py_InitModule("fastbit", fastbit_methods);
-#endif
 }
 
 /* **************************************** */
@@ -1396,6 +1292,8 @@ char **_argv;
 
 void init_python(int argc, char *argv[]) {
 
+  if(myGlobals.runningPref.disablePython) return;
+
   if(_argc == 0) {
     _argc = argc;
     _argv = argv;
@@ -1414,20 +1312,14 @@ void init_python(int argc, char *argv[]) {
   PyEval_InitThreads();
 
   init_python_ntop();
-
-#ifdef HAVE_FASTBIT
-  fastbit_init(NULL);
-  fastbit_set_verbose_level(-1);
-#endif
 }
 
 /* **************************************** */
 
 void term_python(void) {
+  if(myGlobals.runningPref.disablePython) return;
+
   Py_Finalize();   /* Cleaning up the interpreter */
-#ifdef HAVE_FASTBIT
-  fastbit_cleanup();
-#endif
   deleteMutex(&python_mutex);
 }
 
@@ -1435,7 +1327,7 @@ void term_python(void) {
 
 int handlePythonHTTPRequest(char *url, u_int postLen) {
   int idx, found = 0;
-  char python_path[256], *document_root = strdup("."), buf[2048];
+  char python_path[256], *document_root = strdup("."), buf[2048], new_query_string[2048];
   struct stat statbuf;
 #ifdef WIN32
   PyObject *fd; 
@@ -1445,6 +1337,12 @@ int handlePythonHTTPRequest(char *url, u_int postLen) {
 
   char *question_mark = strchr(url, '?');
 
+  if(myGlobals.runningPref.disablePython) {
+    returnHTTPpageNotFound(NULL);
+    free(document_root);
+    return(1);
+  }
+
   // traceEvent(CONST_TRACE_INFO, "Calling python... [%s]", url);
 
   if(question_mark) question_mark[0] = '\0';
@@ -1510,21 +1408,25 @@ int handlePythonHTTPRequest(char *url, u_int postLen) {
 
     if(postLen == 0) {
       /* HTTP GET */
+      escape(new_query_string, sizeof(new_query_string), query_string);
+
       safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
 		    "import os\nos.environ['DOCUMENT_ROOT']='%s'\n"
 		    "os.environ['REQUEST_METHOD']='GET'\n"
 		    "os.environ['QUERY_STRING']='%s'\n",
-		    document_root, query_string);
+		    document_root, new_query_string);
     } else {
       /* HTTP POST */
 #if defined(WIN32)
       if((idx = readHTTPpostData(postLen, query_string, sizeof(query_string)-1)) >= 0) {
+	escape(new_query_string, sizeof(new_query_string), query_string);
+
 	/* Emulate a POST with a GET on Windows */
 	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
 		      "import os\nos.environ['DOCUMENT_ROOT']='%s'\n"
 		      "os.environ['REQUEST_METHOD']='GET'\n"
 		      "os.environ['QUERY_STRING']='%s'\n",
-		      document_root, query_string);
+		      document_root, new_query_string);
       }
 #else
       safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),
@@ -1539,13 +1441,16 @@ int handlePythonHTTPRequest(char *url, u_int postLen) {
     /* See http://bugs.python.org/issue1159 */
     PyRun_SimpleString(buf);
 
-    /* traceEvent(CONST_TRACE_INFO, "[PYTHON] Executing %s", buf); */
+    traceEvent(CONST_TRACE_INFO, "[PYTHON] Executing %s", buf); 
+
     /* sys.stdin <=> myGlobals.newSock */
 
 #ifndef WIN32
     /* if(myGlobals.runningPref.debugMode) */ /* -K */
     {
-      traceEvent(CONST_TRACE_INFO, "[PYTHON] Redirecting file descriptors");
+      /* Note that myGlobals.newSock is negative when HTTPS is used */
+      if(myGlobals.runningPref.debugMode) 
+	traceEvent(CONST_TRACE_INFO, "[PYTHON] Redirecting file descriptors");
 
       old_stdin = dup(STDIN_FILENO), old_stdout = dup(STDOUT_FILENO);
 
@@ -1554,18 +1459,18 @@ int handlePythonHTTPRequest(char *url, u_int postLen) {
 	 http://tangentsoft.net/wskfaq/articles/bsd-compatibility.html
 	 http://stackoverflow.com/questions/7664/windows-c-how-can-i-redirect-stderr-for-calls-to-fprintf
       */
-      if(dup2(myGlobals.newSock, STDOUT_FILENO) == -1)
-	traceEvent(CONST_TRACE_WARNING, "Failed to redirect stdout");
+      if(dup2(abs(myGlobals.newSock), STDOUT_FILENO) == -1)
+	traceEvent(CONST_TRACE_WARNING, "Failed to redirect stdout [%d][%s]", errno, strerror(errno));
 
-      if(dup2(myGlobals.newSock, STDIN_FILENO) == -1)
-	traceEvent(CONST_TRACE_WARNING, "Failed to redirect stdin");
+      if(dup2(abs(myGlobals.newSock), STDIN_FILENO) == -1)
+	traceEvent(CONST_TRACE_WARNING, "Failed to redirect stdin [%d][%s]",errno, strerror(errno));
     }
 #endif
 
     /* Run the actual program */
 #ifdef WIN32
-	/* http://python-forum.org/pythonforum/viewtopic.php?f=15&t=1554&p=6567 */
-	PyRun_SimpleFile(PyFile_AsFile(fd), python_path);
+    /* http://python-forum.org/pythonforum/viewtopic.php?f=15&t=1554&p=6567 */
+    PyRun_SimpleFile(PyFile_AsFile(fd), python_path);
 #else
     PyRun_SimpleFile(fd, python_path);
 #endif
@@ -1579,7 +1484,8 @@ int handlePythonHTTPRequest(char *url, u_int postLen) {
       if(dup2(old_stdout, STDIN_FILENO) == -1)
 	traceEvent(CONST_TRACE_WARNING, "Failed to restore stdout");
       
-      traceEvent(CONST_TRACE_INFO, "[PYTHON] Succesfully restored file descriptors");
+      if(myGlobals.runningPref.debugMode) 
+	traceEvent(CONST_TRACE_INFO, "[PYTHON] Succesfully restored file descriptors");
     }
 #endif
 
