@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1998-2011 Luca Deri <deri@ntop.org>
+ *  Copyright (C) 1998-2012 Luca Deri <deri@ntop.org>
  *
  *		 	    http://www.ntop.org/
  *
@@ -62,10 +62,10 @@ void termIPSessions(void) {
   int i, j;
 
   for(j=0; j<myGlobals.numDevices; j++) {
-    if(myGlobals.device[j].tcpSession == NULL) continue;
+    if(myGlobals.device[j].sessions == NULL) continue;
   
     for(i=0; i<MAX_TOT_NUM_SESSIONS; i++) {
-      IPSession *session = myGlobals.device[j].tcpSession[i];
+      IPSession *session = myGlobals.device[j].sessions[i];
 
       while(session != NULL) {
 	IPSession *nextSession;
@@ -75,23 +75,18 @@ void termIPSessions(void) {
       }
     }
 
-#ifdef PARM_USE_SESSIONS_CACHE
-    for(i=0; i<myGlobals.sessionsCacheLen; i++)
-      free(myGlobals.sessionsCache[i]);
-#endif
-
-    myGlobals.device[j].numTcpSessions = 0;
-    
-    while(myGlobals.device[j].fragmentList != NULL)
-      deleteFragment(myGlobals.device[j].fragmentList, j);
+    myGlobals.device[j].numSessions = 0;
   }
 }
 
 /* ************************************************ */
 
 void termGdbm() {
-  if(myGlobals.pwFile)           { gdbm_close(myGlobals.pwFile); myGlobals.pwFile = NULL;                     }
-  if(myGlobals.prefsFile)        { gdbm_close(myGlobals.prefsFile); myGlobals.prefsFile = NULL;               }
-  if(myGlobals.macPrefixFile)    { gdbm_close(myGlobals.macPrefixFile); myGlobals.macPrefixFile = NULL;       }
-  if(myGlobals.fingerprintFile)  { gdbm_close(myGlobals.fingerprintFile); myGlobals.fingerprintFile = NULL;   }
+  if(myGlobals.pwFile)            { gdbm_close(myGlobals.pwFile); myGlobals.pwFile = NULL;                       }
+  if(myGlobals.prefsFile)         { gdbm_close(myGlobals.prefsFile); myGlobals.prefsFile = NULL;                 }
+  if(myGlobals.macPrefixFile)     { gdbm_close(myGlobals.macPrefixFile); myGlobals.macPrefixFile = NULL;         }
+  if(myGlobals.fingerprintFile)   { gdbm_close(myGlobals.fingerprintFile); myGlobals.fingerprintFile = NULL;     }
+  if(myGlobals.serialFile)        { gdbm_close(myGlobals.serialFile); myGlobals.serialFile = NULL;               }
+  if(myGlobals.resolverCacheFile) { gdbm_close(myGlobals.resolverCacheFile); myGlobals.resolverCacheFile = NULL; }
+  if(myGlobals.topTalkersFile)    { gdbm_close(myGlobals.topTalkersFile); myGlobals.topTalkersFile = NULL;       }
 }
